package org.apache.turbine.admin.actions.admin.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Imports
import java.util.*;

// Turbine/Village/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.modules.actions.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;

// Webmacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;

/**
 * This is the Action for the admin.security.PermissionEditor Screen.
 * It handles the update/modify/delete portion of the Screen.
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @version $Id: UpdatePermission.java,v 1.2 2000/08/24 17:09:52 gonzalo Exp $
 */
public class UpdatePermission extends WebMacroSiteAction
{
    /**
     * Handles the doDelete button.
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doDelete(RunData data,
                         WebContext context)
        throws Exception
    {
        int permission_id = data.getParameters().getInt("permission_id", -1);
        String permission_text = data.getParameters().getString("permission_text","");

        if ( permission_id == -1 )
        {
            data.setMessage ( "Please choose a permission to delete!" );
            permission_text = "";
        }
        else
        {
            context.put ( "delete", new Boolean(true) );
            context.put ( "modify_id", new Integer(permission_id) );
            Permission perm = PermissionPeer.selectOnePermission(permission_id);
            permission_text = perm.getName();
        }
        context.put ( "permission_text", permission_text );
    }

    /**
     * Handles the doOkdelete button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doOkdelete(RunData data,
                           WebContext context)
        throws Exception
    {
        int modify_id = data.getParameters().getInt("modify_id", -1 );
        // delete the permission
        PermissionPeer.deletePermission ( modify_id );
        // delete the role_permission mappings
        RolePermissionPeer.deletePermission( modify_id );
        context.put ( "permission_text", "" );
        context.put ( "modify_id", new Integer(-1) );
        data.setMessage ( "Permission Deleted!" );
    }

    /**
     * Handles the doAdd button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doAdd(RunData data,
                      WebContext context)
        throws Exception
    {
        String permission_text = data.getParameters().getString("permission_text","");
        if (permission_text.length() == 0)
        {
            data.setMessage("Please type in a permission name to add!");
        }
        else
        {
            PermissionPeer.insertPermission ( permission_text );
            data.setMessage ( "Permission added!" );
        }
        context.put ( "permission_text", "" );

    }

    /**
     * Handles the doModify button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doModify(RunData data,
                         WebContext context)
        throws Exception
    {
        int permission_id = data.getParameters().getInt("permission_id", -1);
        String permission_text = data.getParameters().getString("permission_text","");

        if ( permission_id == -1 )
        {
            data.setMessage ( "Please choose a permission to modify!" );
            permission_text = "";
        }
        else
        {
            context.put ( "modify", new Boolean(true) );
            context.put ( "modify_id", new Integer(permission_id) );
            Permission perm = PermissionPeer.selectOnePermission(permission_id);
            permission_text = perm.getName();
        }
        context.put ( "permission_text", permission_text );
    }

    /**
     * Handles the doModify2 button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doModify2(RunData data,
                          WebContext context)
        throws Exception
    {
        String permission_text = data.getParameters().getString("permission_text","");
        int modify_id = data.getParameters().getInt("modify_id", -1 );
        if (modify_id == -1)
        {
            data.setMessage ( "Could not update permission! Invalid permission id." );
        }
        else
        {
            PermissionPeer.updatePermission ( modify_id, permission_text );
            data.setMessage ( "Permission Updated!" );
        }
        context.put ( "permission_text", "" );
    }

    /**
     * Handles the doCancel button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doCancel(RunData data,
                         WebContext context)
        throws Exception
    {
        data.setMessage("Action canceled!");
    }

    /**
     * Handles the doPerform button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doPerform( RunData data,
                           WebContext context )
        throws Exception
    {
        context.put ( "permission_text", "" );
    }
}
