package org.apache.turbine.admin.actions.admin.user;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Imports
import java.util.*;
import java.math.*;

// Turbine/Village/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.modules.actions.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;

// Webmacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;

/**
 * Action for adding a new user to the system.
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @version $Id: AddUser.java,v 1.2 2000/08/24 17:19:08 gonzalo Exp $
 */
public class AddUser extends WebMacroSiteAction
{
    /**
     * Handles the cancel button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doCancel( RunData data,
                          WebContext context )
        throws Exception
    {
        data.setMessage("Action Canceled!");
        setTemplate ( data, "admin,user,SelectUser.wm" );
    }

    /**
     * Just calls doCancel()
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doPerform( RunData data,
                           WebContext context )
        throws Exception
    {
        doCancel(data, context);
    }

    /**
     * Add the user into the system. First checks to make sure that
     * the user doesn't already exist. Also does minor form input
     * validation.
     *
     * FIXME: Add more form input validation!
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doAdd( RunData data,
                       WebContext context )
        throws Exception
    {
        // Check that all the required fields are completed.
        String LOGINID = data.getParameters().getString("LOGINID", null);
        String FIRST_NAME = data.getParameters().getString("FIRST_NAME", null);
        String LAST_NAME = data.getParameters().getString("LAST_NAME", null);
        String PASSWORD_VALUE = data.getParameters().getString("PASSWORD_VALUE", null);

        if ( LOGINID == null || FIRST_NAME == null || LAST_NAME == null || PASSWORD_VALUE == null)
        {
            data.setMessage("Sorry, you need to fill in the Login ID, First Name, Last Name and Password fields.");
            setTemplate ( data, "admin,user,AddUser.wm" );
            return;
        }

        // Pull the user information out of the form data and add it to the criteria.
        Criteria newdata = new Criteria();
        newdata.add(TurbineUserPeer.getColumnName("LOGINID"), data.getParameters().getString("LOGINID") );
        newdata.add(TurbineUserPeer.getColumnName("PASSWORD_VALUE"), data.getParameters().getString("PASSWORD_VALUE") );
        newdata.add(TurbineUserPeer.getColumnName("FIRST_NAME"), data.getParameters().getString("FIRST_NAME") );
        newdata.add(TurbineUserPeer.getColumnName("LAST_NAME"), data.getParameters().getString("LAST_NAME") );
        newdata.add(TurbineUserPeer.getColumnName("ADDRESS1"), data.getParameters().getString("ADDRESS1") );
        newdata.add(TurbineUserPeer.getColumnName("ADDRESS2"), data.getParameters().getString("ADDRESS2") );
        newdata.add(TurbineUserPeer.getColumnName("CITY"), data.getParameters().getString("CITY") );
        newdata.add(TurbineUserPeer.getColumnName("STATE"), data.getParameters().getString("STATE") );
        newdata.add(TurbineUserPeer.getColumnName("POSTALCODE"), data.getParameters().getString("POSTALCODE") );
        newdata.add(TurbineUserPeer.getColumnName("COUNTRY"), data.getParameters().getString("COUNTRY") );
        newdata.add(TurbineUserPeer.getColumnName("CITIZENSHIP"), data.getParameters().getString("CITIZENSHIP") );
        newdata.add(TurbineUserPeer.getColumnName("PHONE"), data.getParameters().getString("PHONE") );
        newdata.add(TurbineUserPeer.getColumnName("ALTPHONE"), data.getParameters().getString("ALTPHONE") );
        newdata.add(TurbineUserPeer.getColumnName("FAX"), data.getParameters().getString("FAX") );
        newdata.add(TurbineUserPeer.getColumnName("CELL"), data.getParameters().getString("CELL") );
        newdata.add(TurbineUserPeer.getColumnName("PAGER"), data.getParameters().getString("PAGER") );
        newdata.add(TurbineUserPeer.getColumnName("EMAIL"), data.getParameters().getString("EMAIL") );
        newdata.add(TurbineUserPeer.getColumnName("CONFIRM_VALUE"), data.getParameters().getString("CONFIRM_VALUE") );

        int visitor_id = -1;
        try
        {
            // Check to see if the user exists.
            Criteria crit = new Criteria().add ( TurbineUserPeer.getColumnName("LOGINID"), LOGINID );
            Vector result = TurbineUserPeer.doSelect(crit);
            if ( result.size() > 0 )
                throw new Exception ( "Sorry, a user with that loginid already exists!" );

            // Insert the new user.
            visitor_id = ((BigDecimal)TurbineUserPeer.doInsert(newdata)).intValue();
            if ( visitor_id > -1 )
            {
                // Get the list of users roles.
                String[] roles = data.getParameters().getStrings("roleSetBox", null);
                // Remove any existing roles and re-associate them to the user.
                UpdateUser.updateRoles(roles, visitor_id);
                // All done.
                data.setMessage ( "New user has been added to the system!" );
                return;
            }
            else
            {
                throw new Exception ( "Could not get a valid visitor_id when creating the user!" );
            }
        }
        catch(Exception e)
        {
            data.setMessage ("Error adding new user: " + e.toString() );
            setTemplate ( data, "admin,user,AddUser.wm" );
        }
    }
}
