package org.apache.turbine.admin.actions.admin.user;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Imports
import java.util.*;

// Turbine/Village/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.modules.actions.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;

// Webmacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;

/**
 * Updates a User's information back into the database.
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @version $Id: UpdateUser.java,v 1.2 2000/08/24 17:19:09 gonzalo Exp $
 */
public class UpdateUser extends WebMacroSiteAction
{
    /**
     * Handles the cancel button
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doCancel( RunData data,
                          WebContext context )
        throws Exception
    {
        data.setMessage("Action Canceled!");
        setTemplate ( data, "admin,user,SelectUser.wm" );
    }

    /**
     * Just calls doCancel()
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doPerform( RunData data,
                           WebContext context )
        throws Exception
    {
        doCancel(data, context);
    }

    /**
     * Attempts to update an existing user in the database.
     *
     * FIXME: Add more form input validation!
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doUpdate( RunData data,
                          WebContext context )
        throws Exception
    {
        // Check to see if we have a valid VISITORID.
        int visitor_id = data.getParameters().getInt("VISITORID",-1);
        if (visitor_id == -1)
        {
            data.setMessage("Sorry, that user could not be found in the system: " + visitor_id);
            setTemplate ( data, "admin,user,SelectUser.wm" );
            return;
        }

        // Get the user object from the database.
        User user = (TurbineUser) UserFactory.getUser(visitor_id);
        // Re-populate it with values from the form.
        user.setPerm(TurbineUserPeer.getColumnName("VISITORID"), data.getParameters().getString("VISITORID") );
        user.setPerm(TurbineUserPeer.getColumnName("LOGINID"), data.getParameters().getString("LOGINID") );
        user.setPerm(TurbineUserPeer.getColumnName("PASSWORD_VALUE"), data.getParameters().getString("PASSWORD_VALUE") );
        user.setPerm(TurbineUserPeer.getColumnName("FIRST_NAME"), data.getParameters().getString("FIRST_NAME") );
        user.setPerm(TurbineUserPeer.getColumnName("LAST_NAME"), data.getParameters().getString("LAST_NAME") );
        user.setPerm(TurbineUserPeer.getColumnName("ADDRESS1"), data.getParameters().getString("ADDRESS1") );
        user.setPerm(TurbineUserPeer.getColumnName("ADDRESS2"), data.getParameters().getString("ADDRESS2") );
        user.setPerm(TurbineUserPeer.getColumnName("CITY"), data.getParameters().getString("CITY") );
        user.setPerm(TurbineUserPeer.getColumnName("STATE"), data.getParameters().getString("STATE") );
        user.setPerm(TurbineUserPeer.getColumnName("POSTALCODE"), data.getParameters().getString("POSTALCODE") );
        user.setPerm(TurbineUserPeer.getColumnName("COUNTRY"), data.getParameters().getString("COUNTRY") );
        user.setPerm(TurbineUserPeer.getColumnName("CITIZENSHIP"), data.getParameters().getString("CITIZENSHIP") );
        user.setPerm(TurbineUserPeer.getColumnName("PHONE"), data.getParameters().getString("PHONE") );
        user.setPerm(TurbineUserPeer.getColumnName("ALTPHONE"), data.getParameters().getString("ALTPHONE") );
        user.setPerm(TurbineUserPeer.getColumnName("FAX"), data.getParameters().getString("FAX") );
        user.setPerm(TurbineUserPeer.getColumnName("CELL"), data.getParameters().getString("CELL") );
        user.setPerm(TurbineUserPeer.getColumnName("PAGER"), data.getParameters().getString("PAGER") );
        user.setPerm(TurbineUserPeer.getColumnName("EMAIL"), data.getParameters().getString("EMAIL") );
        user.setPerm(TurbineUserPeer.getColumnName("CONFIRM_VALUE"), data.getParameters().getString("CONFIRM_VALUE") );
        // Save it back to the database.
        user.saveToStorage();

        String[] roles = data.getParameters().getStrings("roleSetBox", null);
        updateRoles(roles, user.getId());

        // Show the updated message!
        data.setMessage("The user has been updated!");
        setTemplate ( data, "admin,user,SelectUser.wm" );
    }

    /**
     * This method is for updating the Roles in the database.  First
     * it deletes all the old roles and then adds in the new
     * roles. Not transaction safe, but oh well...use a database that
     * doesn't fail. :-)
     *
     * @param roles An array of strings with the roles to update.
     * @param id Id of visitor for whom the roles will be updated.
     * @exception Exception, a generic exception.
     */
    static final void updateRoles ( String[] roles,
                                    int id )
        throws Exception
    {
        if ( roles != null )
        {
            // Delete the old roles for this user.
            VisitorRolePeer.deleteAll(VisitorRolePeer.getTableName(),
                                      VisitorRolePeer.VISITOR_ID, id );
            // Re-add the new roles.
            Criteria crit = new Criteria();
            crit.add ( VisitorRolePeer.VISITOR_ID, id );
            String tmp = "";
            for ( int i=0; i<roles.length;i++ )
            {
                crit.add ( VisitorRolePeer.ROLE_ID, roles[i] );
                VisitorRolePeer.doInsert(crit);
            }
        }
    }
}
