package org.apache.turbine.admin.screens.admin.user;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine/Village/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.modules.screens.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.pool.*;
import org.apache.turbine.util.webmacro.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.ecs.*;
import org.apache.ecs.html.*;
import org.apache.ecs.filter.*;

// Webmacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;

/**
 * Add a user to Turbine.
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: AddUser.java,v 1.4 2000/08/29 16:23:40 sean Exp $
 */
public class AddUser extends UserScreen
{
    /**
     * Build the screen context.
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doBuildTemplate( RunData data,
                                 WebContext context )
        throws Exception
    {
        // Get an array of the selected roles.
        String[] selectedRoles = data.getParameters().getStrings("roleSetBox", null);
        // Get an array of all the roles in the system.
        Role[] allRoles = ((RoleSet)RolePeer.retrieveSet()).getRolesArray();
        Object[] names = new Object[allRoles.length]; // id numbers
        Object[] values = new Object[allRoles.length]; // role name string
        // Go over all the roles and put them into array's.
        for ( int i=0;i<allRoles.length; i++ )
        {
            names[i] = new Integer(allRoles[i].getId()).toString();
            values[i] = allRoles[i].getName();
        }
        // Make the selector box with all the names and values.
        SelectorBox sb = new SelectorBox("roleSetBox", names, values);
        if ( selectedRoles != null )
            // compare against the integers
            sb.buildBooleans(selectedRoles, names);
        context.put("roleSetBox", sb);

        context.put("LOGINID", StringUtils.makeString(data.getParameters().getString("LOGINID")));
        context.put("PASSWORD_VALUE", StringUtils.makeString(data.getParameters().getString("PASSWORD_VALUE")));
        context.put("FIRST_NAME", StringUtils.makeString(data.getParameters().getString("FIRST_NAME")));
        context.put("LAST_NAME", StringUtils.makeString(data.getParameters().getString("LAST_NAME")));
        context.put("ADDRESS1", StringUtils.makeString(data.getParameters().getString("ADDRESS1")));
        context.put("ADDRESS2", StringUtils.makeString(data.getParameters().getString("ADDRESS2")));
        context.put("CITY", StringUtils.makeString(data.getParameters().getString("CITY")));
        context.put("STATE", StringUtils.makeString(data.getParameters().getString("STATE")));
        context.put("POSTALCODE", StringUtils.makeString(data.getParameters().getString("POSTALCODE")));
        context.put("COUNTRY", StringUtils.makeString(data.getParameters().getString("COUNTRY")));
        context.put("CITIZENSHIP", StringUtils.makeString(data.getParameters().getString("CITIZENSHIP")));
        context.put("PHONE", StringUtils.makeString(data.getParameters().getString("PHONE")));
        context.put("ALTPHONE", StringUtils.makeString(data.getParameters().getString("ALTPHONE")));
        context.put("FAX", StringUtils.makeString(data.getParameters().getString("FAX")));
        context.put("CELL", StringUtils.makeString(data.getParameters().getString("CELL")));
        context.put("PAGER", StringUtils.makeString(data.getParameters().getString("PAGER")));
        context.put("EMAIL", StringUtils.makeString(data.getParameters().getString("EMAIL")));
        context.put("CONFIRM_VALUE", StringUtils.makeString(data.getParameters().getString("CONFIRM_VALUE")));
    }
}
