package org.apache.turbine.admin.screens.admin.user;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.sql.*;

// JDK Imports
import java.util.Vector;

// Turbine/Village/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.modules.screens.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.pool.*;
import org.apache.turbine.util.webmacro.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.ecs.*;
import org.apache.ecs.html.*;
import org.apache.ecs.filter.*;

// Webmacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;

// Village Stuff
import com.workingdogs.village.*;

/**
 * This class displays a single user record for editing. Its
 * corresponding Action is UpdateUser.
 *
 * @author <a href="mailto:sean@somacity.com">Sean Legassick</a>
 * @version $Id: EditUser.java,v 1.4 2000/08/29 16:23:40 sean Exp $
 */
public class EditUser extends UserScreen
{
    /**
     * Build the screen context.
     *
     * @param data Turbine information.
     * @param context Context for web pages.
     * @exception Exception, a generic exception.
     */
    public void doBuildTemplate( RunData data,
                                 WebContext context )
        throws Exception
    {
        // If the user_id is not set, then send the person back to the
        // users.SelectUser screen.
        Integer user_id = (Integer) data.getUser().getTemp("user_id",
            data.getParameters().getInteger("user_id", -1 ) );
        if ( user_id == null || user_id.intValue() == -1 )
        {
            data.setMessage("You must first select a user!");
            // FIX ME!!!!
            data.setScreen("users.SelectUser");
            return;
        }

        context = getContext(data);

        // Get the user from the database.
        Criteria crit = new Criteria(1);
        crit.add(TurbineUserPeer.VISITOR_ID, user_id);
        Vector visitors = TurbineUserPeer.doSelect(crit);
    
        String loginid = null;
        if ( visitors.size() == 1 )
        {
            User visitor = (User)visitors.elementAt(0);
            loginid = StringUtils.makeString(visitor.getPerm("LOGINID").toString());
            context.put("VISITORID", StringUtils.makeString(visitor.getPerm("VISITORID").toString()));
            context.put("LOGINID", loginid );
            context.put("PASSWORD_VALUE", StringUtils.makeString(visitor.getPerm("PASSWORD_VALUE").toString()));
            context.put("FIRST_NAME", StringUtils.makeString(visitor.getPerm("FIRST_NAME").toString()));
            context.put("LAST_NAME", StringUtils.makeString(visitor.getPerm("LAST_NAME").toString()));
            context.put("ADDRESS1", StringUtils.makeString(visitor.getPerm("ADDRESS1").toString()));
            context.put("ADDRESS2", StringUtils.makeString(visitor.getPerm("ADDRESS2").toString()));
            context.put("CITY", StringUtils.makeString(visitor.getPerm("CITY").toString()));
            context.put("STATE", StringUtils.makeString(visitor.getPerm("STATE").toString()));
            context.put("POSTALCODE", StringUtils.makeString(visitor.getPerm("POSTALCODE").toString()));
            context.put("COUNTRY", StringUtils.makeString(visitor.getPerm("COUNTRY").toString()));
            context.put("CITIZENSHIP", StringUtils.makeString(visitor.getPerm("CITIZENSHIP").toString()));
            context.put("PHONE", StringUtils.makeString(visitor.getPerm("PHONE").toString()));
            context.put("ALTPHONE", StringUtils.makeString(visitor.getPerm("ALTPHONE").toString()));
            context.put("FAX", StringUtils.makeString(visitor.getPerm("FAX").toString()));
            context.put("CELL", StringUtils.makeString(visitor.getPerm("CELL").toString()));
            context.put("PAGER", StringUtils.makeString( visitor.getPerm("PAGER").toString()));
            context.put("EMAIL", StringUtils.makeString(visitor.getPerm("EMAIL").toString()));
            context.put("CONFIRM_VALUE", StringUtils.makeString(visitor.getPerm("CONFIRM_VALUE").toString()));
        }
        else
        {
            // Return to original screen if user was not found.
            data.setMessage("No users were found matching your search criteria.");
            data.getTemplateInfo().setScreenTemplate("admin,users,SelectUser.wm");
            return;
        }
        // Get the roles for a user.
        RoleSet userRoles = new DefaultAccessControl().getRoles(loginid, null);
        if ( userRoles != null )
       {
            // Get an array of the users roles.
            Role[] usersRoles = userRoles.getRolesArray();
            // Get an array of all the roles in the system.
            Role[] allRoles = ((RoleSet)RolePeer.retrieveSet()).getRolesArray();
            
            Object[] names = new Object[allRoles.length]; // id numbers
            Object[] values = new Object[allRoles.length]; // role name string
            // Go over all the roles and put them into array's.
            for ( int i=0;i<allRoles.length; i++ )
            {
                names[i] = new Integer(allRoles[i].getId()).toString();
                values[i] = allRoles[i].getName();
            }
            // Make the selector box with all the names and values.
            SelectorBox sb = new SelectorBox("roleSetBox", names, values);
            sb.buildBooleans(usersRoles, allRoles);
            context.put("roleSetBox", sb);
        }
    }
}
