package org.apache.turbine.modules;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * The purpose of this class is to allow one to load and execute
 * Layout modules.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: LayoutLoader.java,v 1.2 2000/08/25 18:05:42 gonzalo Exp $
 */
public class LayoutLoader extends GenericLoader
{
    /** The single instance of this class. */
    private static LayoutLoader instance = null;

    /** Packages to search for actions. */
    private Vector packages;

    /**
     * These ctor's are private to force clients to use getInstance()
     * to access this class.
     */
    private LayoutLoader()
    {
        super();
    }
    private LayoutLoader(int i)
    {
        super(i);
    }


    /**
     * Adds an instance of an object into the hashtable.
     *
     * @param name Name of object.
     * @param layout Layout to be associated with name.
     */
    private void addInstance ( String name,
                               Layout layout )
    {
        if ( cache() )
            this.put ( name, ( Layout ) layout );
    }

    /**
     * Attempts to load and execute the external layout.
     *
     * @param data Turbine information.
     * @param name Name of object that will execute the layout.
     * @exception Exception, a generic exception.
     */
    public void exec ( RunData data,
                       String name )
        throws Exception
    {
        // Execute layout
        getInstance(name).build(data);
    }

    /**
     * Pulls out an instance of the object by name.  Name is just the
     * single name of the object.
     *
     * @param name Name of object instance.
     * @return A Layout with the specified name, or null.
     * @exception Exception, a generic exception.
     */
    public Layout getInstance (String name)
        throws Exception
    {
        Layout layout = null;
        try
        {
            if (  cache() && this.containsKey ( name ) )
            {
                layout = ( Layout ) this.get ( name );
            }
            else
            {
                for (int i=0; i<instance.packages.size(); i++)
                {
                    String className = ((String) instance.packages.elementAt(i) +
                                        ".layouts." +
                                        name);
                    try
                    {
                        Class servClass = Class.forName( className );
                        layout = ( Layout ) servClass.newInstance();
                        addInstance ( name, layout );
                        return layout;
                    }
                    catch (ClassNotFoundException cnfe)
                    {
                        // Do this so we loop through all the packages.
                    }
                    catch (NoClassDefFoundError ncdfe)
                    {
                        // Do this so we loop through all the packages.
                    }
                    catch (ClassCastException cce)
                    {
                        // Do this so we know what class is having problems.
                        throw new ClassCastException( className );
                    }
                }
                // If we got here the class is really not found.
                throw new ClassNotFoundException();
            }
        }
        catch ( ClassNotFoundException e )
        {
            throw new Exception ( "\n\n\tRequested Layout not found: " +
                                    name + "\n" +
                                   "\tTurbine looked in the following modules.packages path: \n\t" +
                                   instance.packages.toString() + "\n");
        }
        return layout;
    }

    /**
     * The method through which this class is accessed.
     *
     * @return The single instance of this class.
     */
    public static LayoutLoader getInstance()
    {
        if (instance == null)
        {
            synchronized (LayoutLoader.class)
            {
                if (instance == null)
                {
                    int size = TurbineResources.getInt("layout.cache.size", 10);
                    instance = new LayoutLoader(size);
                    instance.packages = TurbineResources.getVector("module.packages");
                    instance.packages.addElement( GenericLoader.getBasePackage() );
                }
            }
        }
        return instance;
    }
}
