package org.apache.turbine.modules;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.TurbineResources;

// WebMacro Stuff
import org.apache.turbine.services.webmacro.*;
import org.webmacro.servlet.*;
import org.webmacro.*;

/**
 * The purpose of this class is to allow one to parse and load
 * WebMacroScreen and Layout modules. It also performs caching if
 * turned on in the properties file.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: WebMacroScreenLayoutLoader.java,v 1.3 2000/08/25 18:05:44 gonzalo Exp $
 */
public class WebMacroScreenLayoutLoader extends GenericLoader
{
    /** The single instance of this class. */
    private static WebMacroScreenLayoutLoader instance = null;

    /**
     * Constructor.
     */
    private WebMacroScreenLayoutLoader()
    {
        super();
    }

    /**
     * Constructor.
     *
     * @param int the size of the cache.
     */
    private WebMacroScreenLayoutLoader(int i)
    {
        super(i);
    }

    /**
     *  Adds the TemplateHolder into the hashtable.
     *
     * @param name Name of object.
     * @param holder TemplateHolder to be associated with name.
     */
    private void addInstance ( String name,
                               TemplateHolder holder )
    {
        if ( cache() )
            this.put ( name, ( TemplateHolder ) holder );
    }

    /**
     * Method called by WebMacroSitePage.
     *
     * @param String the name of the template to parse and find.
     * @return The TemplateHolder containing the names of the
     * Screen/Layout to use.
     */
    public TemplateHolder eval ( String name )
        throws Exception
    {
        return  getInstance( name );
    }

    /**
     * Not needed for this loader.
     *
     * @param data Turbine information.
     * @param name Name of object that will execute the navigation.
     * @exception Exception, a generic exception.
     */
    public void exec ( RunData data,
                       String name )
        throws Exception
    {
        // NOT IMPLEMENTED - Not needed
    }

    /**
     * Locate and return the Loaded TemplatHolder.
     *
     * @param String the name of the template.
     * @return TemplateHolder.
     */
    public TemplateHolder getInstance (String name)
        throws Exception
    {
        TemplateHolder holder = null;

        if (  cache() && this.containsKey ( name ) )
        {
            holder = ( TemplateHolder ) this.get ( name );
        }
        else
        {
            holder = parseTemplatePath( name );
            addInstance ( name, holder );
            return holder;
        }
        return holder;
    }

    /**
     * This method takes the template parameter and parses it, so that
     * relevant Screen/Layout-template information can be extracted.
     * Possible layout paths and Screen packagese placed in a
     * TemplateHolder.
     *
     * @param String the template name.
     * @return TemplateHolder.
     */
    private TemplateHolder parseTemplatePath( String template )
    {
        StringTokenizer st = new StringTokenizer(template, "/");
        Vector tokens = new Vector(st.countTokens());
        while(st.hasMoreTokens())
        {
            String token = st.nextToken();
            if (!token.equals(""))
            {
                tokens.addElement(token);
            }
        }
        String fileName = (String)tokens.lastElement();
        tokens.setSize(tokens.size()-1);
        int dot = fileName.lastIndexOf('.');
        String className = null;
        if (dot>0)
        {
            className = fileName.substring(0, dot);
        }
        else
        {
            className = fileName;
        }
        String firstChar = String.valueOf(className.charAt(0));
        firstChar = firstChar.toUpperCase();
        className = firstChar + className.substring(1);

        String[] paths = new String[tokens.size() + 2];
        String[] pkgs = new String[tokens.size() + 2];
        int arrayIndex = 0;
        for (int i=tokens.size(); i>=0;  i--)
        {
            StringBuffer path = new StringBuffer();
            StringBuffer pkg = new StringBuffer();
            for (int j=0; j<i; j++)
            {
                path.append("/").append((String)tokens.elementAt(j));
                pkg.append((String)tokens.elementAt(j)).append('.');
            }
            if ( i == tokens.size() )
            {
                StringBuffer distinctPath = new StringBuffer(path.toString());
                StringBuffer distinctPkg = new StringBuffer(pkg.toString());
                paths[arrayIndex] = distinctPath.append('/').append(fileName).toString();
                pkgs[arrayIndex] = distinctPkg.append(className).toString();
                arrayIndex++;
            }
            paths[arrayIndex] = path.append("/default.wm").toString();
            pkgs[arrayIndex] = pkg.append("Default").toString();
            arrayIndex++;
        }

        String layout_template = getLayoutTemplateName(paths);
        String screen_name = getScreenName(pkgs);

        TemplateHolder holder = new TemplateHolder();
        holder.setLayoutName(layout_template);
        holder.setScreenName(screen_name);

        return holder;
    }

    /**
     * Extract possible layouts paths.
     *
     * @param String[] possible paths to search.
     * @return String name of the layout template.
     */
    private String getLayoutTemplateName(String[] possiblePaths)
    {
        WebMacroService wm = (WebMacroService)TurbineServices.getInstance().getService(WebMacroService.SERVICE_NAME);
        String layout = null;

        for (int i=0; i<possiblePaths.length; i++)
        {
            try
            {
                Template template = wm.getTemplate("layouts" + possiblePaths[i]);
                return possiblePaths[i];
            }
            catch( NotFoundException e ){}
        }
        return null;
    }

    /**
     * Extract a possible Screen from the packages.
     *
     * @param String[] possible paths to search
     * @return String name of the Screen class to use
     */
    private String getScreenName( String[] possibleScreens)
    {
        for (int i=0; i<possibleScreens.length; i++)
        {
            try
            {
                ScreenLoader.getInstance().getInstance(possibleScreens[i]);
                return possibleScreens[i];
            }
            catch (Exception e)
            {
                // Log.error ("Error in WebMacroSitePage: ", e);
            }
        }
        return null;
    }


    /**
     * The method through which this class is accessed.
     *
     * @return The single instance of this class.
     */
    public static WebMacroScreenLayoutLoader getInstance()
    {
        if (instance == null)
        {
            synchronized (WebMacroScreenLayoutLoader.class)
            {
                if (instance == null)
                {
                    int size = TurbineResources.getInt("wmlayoutscreenloader.cache.size",50);
                    instance = new WebMacroScreenLayoutLoader(size);
                }
            }
        }
        return instance;
    }
}
