package org.apache.turbine.modules.navigations;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine Modules
import org.apache.turbine.modules.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.security.*;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/**
 * This is a sample navigation module.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: DefaultBottomNavigation.java,v 1.2 2000/08/28 18:02:04 gonzalo Exp $
 */
public class DefaultBottomNavigation extends Navigation
{
    private static final boolean DEBUG = false;

    /**
     * Build the Navigation.
     *
     * @param data Turbine information.
     * @return A ConcreteElement.
     * @exception Exception, a generic exception.
     */
    public ConcreteElement doBuild( RunData data )
        throws Exception
    {
        ElementContainer body = new ElementContainer()
            .addElement(new HR().setSize(1).setNoShade(true))
            .addElement(new B().addElement(
               new Font().setColor(HtmlColor.green).setSize(2)
                   .addElement("Turbine - A Servlet Framework for building Secure Dynamic Websites.")))
            .addElement( new Form( new DynamicURI ( data, "DefaultScreen", "LogoutUser", true ).toString(),
                         Form.POST ).addElement(new Input("SUBMIT", "Logout", "Logout") ) );

        if (DEBUG  &&  data.user != null)
        {
            TD perm = new TD().setVAlign(AlignType.TOP);
            TD temp = new TD().setVAlign(AlignType.TOP);
            java.util.Enumeration ePerm = data.getUser().getPermStorage().keys();
            java.util.Enumeration eTemp = ((TurbineUser)data.user).getTempStorage().keys();

            perm.addElement("Perm values:").addElement( new BR() );
            temp.addElement("Temp values:").addElement( new BR() );
            while ( ePerm.hasMoreElements() )
            {
                String key = (String) ePerm.nextElement();
                String value = data.getUser().getPerm(key).toString();
                perm.addElement( key + "=" + value )
                    .addElement( new BR() );
            }
            while ( eTemp.hasMoreElements() )
            {
                String key = (String) eTemp.nextElement();
                String value = data.getUser().getTemp(key).toString();
                temp.addElement( key + "=" + value )
                    .addElement( new BR() );
            }
            body.addElement(new BR()).addElement(new BR())
                .addElement(new Table().setBorder(2).setCellPadding(10)
                .addElement(new TR()
                    .addElement(perm).addElement(temp) ));
        }
        if (DEBUG)
        {
            // If there are GET/POST/PATH_INFO variables put them into
            // a <PRE></PRE> tag so that they can be displayed on the
            // page. This is of course only for example purposes.
            PRE pre = new PRE();
            Enumeration keys = data.parameters.keys();
            while ( keys.hasMoreElements() )
            {
                String key = (String) keys.nextElement();
                String[] values = data.parameters.getStrings(key);
                if (values.length == 1)
                    pre.addElement(key + " = " + values[0] + "\n");
                else
                {
                    pre.addElement(key + " = ");
                    for (int i=0; i<values.length; i++)
                        pre.addElement(values[i] + " ");
                    pre.addElement("\n");
                }
            }
            body.addElement( new B("Query/PathInfo Parameters") )
                .addElement( new BR() )
                .addElement(pre);

            Table table2 = new Table().setBorder(0);
            keys = data.varDebug.keys();
            boolean hasValues2 = false;
            while ( keys.hasMoreElements() )
            {
                String key = (String) keys.nextElement();
                String value = data.varDebug.get(key).toString();
                TR tr = new TR()
                    .addElement ( new TD().addElement(new B(key)) )
                    .addElement ( new TD().addElement(" = " + value ) );
                table2.addElement(tr);
                hasValues2 = true;
            }
            if ( hasValues2 )
            {
                body.addElement (new H4().addElement("Debugging Data:"));
                body.addElement (table2);
            }
        }

        if (DEBUG && data.acl != null)
        {
            // Print out user's permissions.
            PRE pre = new PRE();
            RoleSet rs = data.getACL().getRoles();
            while ( rs.hasMoreElements() )
            {
                String roleName =  rs.next().getName();
                pre.addElement(roleName + "\n");
            }
            body
                .addElement( new BR() )
                .addElement( new B("ROLES") )
                .addElement( new BR() )
                .addElement(pre);

            pre = new PRE();
            PermissionSet ps = data.getACL().getPermissions();
            while ( ps.hasMoreElements() )
            {
                String permissionName =  ps.next().getName();
                pre.addElement(permissionName + "\n");
            }
            body
                .addElement( new BR() )
                .addElement( new B("PERMISSIONS") )
                .addElement( new BR() )
                .addElement(pre);

        }

        return body;
    }
}
