package org.apache.turbine.modules.screens;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine Modules
import org.apache.turbine.modules.*;

// Turbine Utility Classes
import org.apache.turbine.util.RunData;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/**
 * This is a sample Error Screen module.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: Error.java,v 1.2 2000/08/28 20:17:57 gonzalo Exp $
 */
public class Error extends Screen
{
    /**
     * Build screen.
     *
     * @param data Turbine information.
     * @exception Exception, a generic exception.
     */
    public ConcreteElement doBuild( RunData data )
        throws Exception
    {
        data.setTitle("There has been an error!");

        Table table = new Table().setBorder(0);
        boolean hasValues = false;
        Enumeration keys = data.getParameters().keys();
        while ( keys.hasMoreElements() )
        {
            String key = (String) keys.nextElement();
            String value = data.getParameters().getString(key);
            TR tr = new TR()
                .addElement ( new TD().addElement(new B(key)) )
                .addElement ( new TD().addElement(" = " + value ) );
            table.addElement(tr);
            hasValues = true;
        }

        Table table2 = new Table().setBorder(0);
        keys = data.varDebug.keys();
        boolean hasValues2 = false;
        while ( keys.hasMoreElements() )
        {
            String key = (String) keys.nextElement();
            String value = data.varDebug.get(key).toString();
            TR tr = new TR()
                .addElement ( new TD().addElement(new B(key)) )
                .addElement ( new TD().addElement(" = " + value ) );
            table2.addElement(tr);
            hasValues2 = true;
        }

        data.getPage().getBody().addElement (new H3(data.getTitle() +
          " Please review the exception below for more information."));

        if ( hasValues )
        {
            data.getPage().getBody().addElement (new H4().addElement("Get/Post Data:"));
            data.getPage().getBody().addElement (table);
        }

        if ( hasValues2 )
        {
            data.getPage().getBody().addElement (new H4().addElement("Debugging Data:"));
            data.getPage().getBody().addElement (table2);
        }

        if ( data.stackTrace != null )
        {
            data.getPage().getBody()
                .addElement(new H4().addElement("The exception is:"))
                .addElement(new PRE(data.stackTrace))
                .addElement(new PRE(data.stackTraceException.toString()));
        }
        return null;
    }
}
