package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;

/**
 * This class represents a set of Roles.  It makes it easy to build a
 * UI that would allow someone to add a group of Roles to a User.  It
 * wraps a TreeSet object to enforce that only Role objects are
 * allowed in the set and only relevant methods are available.
 * TreeSet's contain only unique Objects (no duplicates).
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: RoleSet.java,v 1.2 2000/08/29 15:39:52 gonzalo Exp $
 */
public class RoleSet implements Serializable
{
    /* jdk 1.2 stuff

       // Set to hold the Role Set
       private TreeSet set = new TreeSet();

       // Iterator for Set
       private Iterator iterator = null;

    */

    /** Vector to hold the Role set. */
    private Vector v = new Vector();

    /** Enumeration for the Role set. */
    private Enumeration enum;

    /**
     * Adds a Role to this RoleSet.
     *
     * @param role The Role.
     * @return True if Role was added; false if RoleSet already
     * contained the Role.
     */
    public boolean add(Role role)
    {
        int index;
        for ( index=0; index<v.size(); index++ )
        {
            int compareResult = role.getName().compareTo( ((Role)v.elementAt(index)).getName() );
            if ( compareResult < 0 ) break;
            if ( compareResult == 0 ) return false;
        }
        v.insertElementAt( role, index );
        return true;
    }

    /**
     * Adds the Roles in another RoleSet to this RoleSet.
     *
     * @param roleSet The RoleSet.
     * @return True if this RoleSet changed as a result; false if no
     * change to this RoleSet occured (this RoleSet already contained
     * all members of the added RoleSet).
     */
    public boolean add(RoleSet roleSet)
    {
        boolean changed = false;
        while ( roleSet.hasMoreElements() )
        {
            if ( add( (Role) roleSet.next() ) )
                changed = true;
        }
        return changed;
    }

    /**
     * Removes a Role from this RoleSet.
     *
     * @param role The Role.
     * @return True if this RoleSet contained the Role before it was
     * removed.
     */
    public boolean remove(Role role)
    {
        for (int index=0; index<v.size(); index++ )
        {
            boolean compareResult = role.getName().equals( ((Role)v.elementAt(index)).getName() );
            if (compareResult)
            {
                v.removeElementAt( index );
                return true;
            }
        }
        return false;
    }

    /**
     * Removes all Roles from this RoleSet.
     */
    public void clear()
    {
        v.removeAllElements();
    }

    /**
     * Compares a Role with the Roles contained in this RoleSet
     *
     * @param role The Role.
     * @return True if argument matched a Role in this RoleSet; false
     * if no match.
     */
    public boolean contains(Role role)
    {
        return v.contains( (Object)role );
    }

    /**
     * Compares by name a Role with the Roles contained in this
     * RoleSet.
     *
     * @param roleName A String with the role's name.
     * @return True if argument matched a Role in this RoleSet; false
     * if no match.
     */
    public boolean contains(String roleName)
    {
        for (Enumeration e = v.elements() ; e.hasMoreElements() ;)
        {
            Role role = (Role) e.nextElement();
            if ( roleName != null  &&  roleName.equals( role.getName() ) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Compares by id a Role with the Roles contained in this RoleSet.
     *
     * @param roleId An int with the role's id.
     * @return True if argument matched a Role in this RoleSet; false
     * if no match.
     */
    public boolean contains(int roleId)
    {
        for (Enumeration e = v.elements() ; e.hasMoreElements() ;)
        {
            Role role = (Role) e.nextElement();
            if ( roleId == role.getId() )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns a Role with the given name, if it is contained in this
     * RoleSet.
     *
     * @param roleName A String with the role's name.
     * @return A Role.
     */
    public Role getRole(String roleName)
    {
        for (Enumeration e = v.elements() ; e.hasMoreElements() ;)
        {
            Role role = (Role) e.nextElement();
            if ( roleName != null  &&  roleName.equals( role.getName() ) )
            {
                return role;
            }
        }
        return null;
    }

    /**
     * Returns the next Role in this RoleSet
     *
     * @return A Role.
     */
    public Role next()
    {
        if (enum == null )
        {
            enum = v.elements();
        }
        return (Role) enum.nextElement();
    }

    /**
     * Checks to see if any more Roles exist after the current
     * position of the iterator/enumeration.
     *
     * @return True if next() would return another Role.
     */
    public boolean hasMoreElements()
    {
        if (enum == null )
        {
            enum = v.elements();
        }
        return enum.hasMoreElements();
    }

    /**
     * Resets the iterator/enumeration to beginning.
     */
    public void resetIterator()
    {
        enum = v.elements();
    }

    /**
     * Returns size (cardinality) of this set.
     *
     * @return The cardinality of the set.
     */
    public int size()
    {
        return v.size();
    }

    /**
     * Returns an Roles[] of Roles in this RoleSet.
     *
     * @return A Role[].
     */
    public Role[] getRolesArray()
    {
        Role[] rles = new Role[this.size()];
        int count = 0;
        this.resetIterator();
        while ( this.hasMoreElements() )
        {
            rles[count] = this.next();
            count++;
        }
        return rles;
    }

    /**
     * Override the writeObject to only write out the Vector
     *
     * @param stream The ObjectOutputStream.
     * @exception IOException, an I/O problem.
     */
    private void writeObject(ObjectOutputStream stream)
        throws IOException
    {
        stream.writeObject(v);
    }

    /**
     * Override the readObject to only read the Vector.
     *
     * @param stream The ObjectInputStream.
     * @exception ClassNotFoundException, an unknown object type was
     * read.
     * @exception IOException, an I/O problem.
     */
    private void readObject(ObjectInputStream stream)
        throws ClassNotFoundException,
               IOException
    {
        v = (Vector) stream.readObject();
    }


    /* For jdk 1.2:


    //
    // Adds a Role to this RoleSet.
    //
    // @param role A Role.
    // @return True if Role was added; false if RoleSet already
    // contained the Role.
    //
    public boolean add(Role role)
    {
        return set.add( (Object)role );
    }

    //
    // Adds the Roles in another RoleSet to this RoleSet.
    //
    // @param roleSet A RoleSet.
    // @return True if this RoleSet changed as a result; false if no
    // change to this RoleSet occured (this RoleSet already contained
    // all members of the added RoleSet).
    //
    public boolean add(RoleSet roleSet)
    {
        return set.addAll(  (Collection) roleSet.set );
    }

    //
    // Removes a Role from this RoleSet.
    //
    // @param role A Role.
    // @return True if this RoleSet contained the Role before it was
    // removed.
    //
    public boolean remove(Role role)
    {
        return set.remove( (Object)role );
    }

    //
    // Removes all Roles from this RoleSet.
    //
    public void clear()
    {
        set.clear();
    }

    //
    // Checks whether this RoleSet contains a Role.
    //
    // @param role A Role.
    // @return True if this RoleSet contains the Role, false
    // otherwise.
    //
    public boolean contains(Role role)
    {
        return set.contains( (Object)role );
    }

    //
    // Compares by name a Role with the Roles contained in this
    // RoleSet.
    //
    // @param roleName Name of Role.
    // @return True if argument matched a Role in this RoleSet; false
    // if no match.
    //
    public boolean contains(String roleName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Role role = (Role)iter.next();
            if ( roleName != null  &&
                 roleName.equals( role.getName() ) )
            {
                return true;
            }
        }
        return false;
    }

    //
    // Returns a Role with the given name, if it is contained in this
    // RoleSet.
    //
    // @param roleName Name of Role.
    // @return Role if argument matched a Role in this RoleSet; null
    // if no match.
    //
    public Role getRole(String roleName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Role role = (Role)iter.next();
            if ( roleName != null  &&
                 roleName.equals( role.getName() ) )
            {
                return role;
            }
        }
        return null;
    }

    //
    // Returns the next Role in this RoleSet.
    //
    // @return A Role.
    //
    public Role next()
    {
        if (iterator == null )
        {
            iterator = set.iterator();
        }
        return (Role)(iterator.next());
    }

    //
    // Checks to see if any more Roles exist after the current
    // position of the iterator/enumeration.
    //
    // @return True if next() would return another Role.
    //
    public boolean hasMoreElements()
    {
        if (iterator == null )
        {
            iterator = set.iterator();
        }
        return iterator.hasNext();
    }

    //
    // Resets the iterator to beginning.
    //
    public void resetIterator()
    {
        iterator = set.iterator();
    }

    //
    // Returns size (cardinality) of this set.
    //
    // @return The cardinality of this RoleSet.
    //
    public int size()
    {
        return set.size();
    }


    For jdk 1.2 */
}
