package org.apache.turbine.om.security.peer;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Utilities
import org.apache.turbine.om.peer.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.map.*;
import org.apache.turbine.om.peer.*;
import org.apache.turbine.services.resources.TurbineResources;

// Village Database Classes
import com.workingdogs.village.*;

// Java stuff.
import java.util.*;
import java.sql.*;

/**
 * This class handles all the database access for the PERMISSION
 * table.  This table contains all the permissions that are used in
 * the system.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: PermissionPeer.java,v 1.3 2000/08/29 20:00:19 gonzalo Exp $
 */
public class PermissionPeer extends BasePeer
{
    private static final TurbineMapBuilder mapBuilder = (TurbineMapBuilder) getMapBuilder();

    /** The table name for this peer. */
    private static final String TABLE_NAME = mapBuilder.getTablePermission();

    /** The column name for the permission id field. */
    public static final String PERMISSION_ID = mapBuilder.getPermission_PermissionId();

    /** The column name for the name field. */
    public static final String NAME = mapBuilder.getPermission_Name();

    /** The Oracle sequence name for this peer. */
    private static final String SEQUENCE_NAME = mapBuilder.getSequencePermission();


    /**
     * Retrieves/assembles a PermissionSet
     *
     * @param criteria The criteria to use.
     * @return A PermissionSet.
     * @exception Exception, a generic exception.
     */
    public static PermissionSet retrieveSet(Criteria criteria)
        throws Exception
    {
        Vector results = PermissionPeer.doSelect(criteria);
        PermissionSet ps = new PermissionSet();
        for (int i=0; i<results.size(); i++)
        {
            ps.add( (Permission)results.elementAt(i) );
        }
        return ps;
    }

    /**
     * Issues a select based on a criteria.
     *
     * @param criteria Object containing data that is used to create
     * the SELECT statement.
     * @return Vector containing Permission objects.
     * @exception Exception, a generic exception.
     */
    public static Vector doSelect(Criteria criteria)
        throws Exception
    {
        criteria.addSelectColumn(PERMISSION_ID)
                .addSelectColumn(NAME);

        if (criteria.getOrderByColumns() == null ||
            criteria.getOrderByColumns().size() == 0)
        {
            criteria.addOrderByColumn(NAME);
        }

        // Place any checks here to intercept criteria which require
        // custom SQL.  For example:
        // if ( criteria.containsKey("SomeTable.SomeColumn") )
        // {
        //     String whereSql = "SomeTable.SomeColumn IN (Select ...";
        //     criteria.add("SomeTable.SomeColumn",
        //                  whereSQL, criteria.CUSTOM);
        // }

        // BasePeer returns a Vector of Value (Village) arrays.  The
        // array order follows the order columns were placed in the
        // Select clause.
        Vector rows = BasePeer.doSelect(criteria);
        Vector results = new Vector();

        // Populate the object(s).
        for ( int i=0; i<rows.size(); i++ )
        {
            Permission obj = new Permission();
            Record row = (Record)rows.elementAt(i);
            obj.setId( row.getValue(1).asInt() );
            obj.setName( row.getValue(2).asString() );
            results.addElement( obj );
        }

        return results;
    }

    /**
     * Issues an update based on a criteria.
     *
     * @param criteria Object containing data that is used to create
     * the UPDATE statement.
     * @exception Exception, a generic exception.
     */
    public static void doUpdate(Criteria criteria)
        throws Exception
    {
        Criteria selectCriteria = new Criteria(2);
        selectCriteria.put( PERMISSION_ID,
                            criteria.remove(PERMISSION_ID) );
        BasePeer.doUpdate( selectCriteria, criteria );
    }

    /**
     * Adds a permission into the system.
     *
     * @param name A String describing the inserted permission.
     * @return An Object.
     * @exception Exception, a generic exception.
     */
    public static Object insertPermission (String name )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( NAME, name );
        return doInsert(crit);
    }

    /**
     * Updates a permission into the system.
     *
     * @param id An int with the permission id.
     * @param name A String describing the updated permission.
     * @exception Exception, a generic exception.
     */
    public static void updatePermission ( int id,
                                          String name )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( PERMISSION_ID, id );
        crit.add ( NAME, name );
        PermissionPeer.doUpdate(crit);
    }

    /**
     * Delete a permission by id number.
     *
     * @param id An int with the permission id.
     * @exception Exception, a generic exception.
     */
    public static void deletePermission (int id )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( PERMISSION_ID, id );
        doDelete(crit);
    }

    /**
     * Returns a vector of Permission objects.
     *
     * @return A Vector with the Permission objects.
     * @exception Exception, a generic exception.
     */
    public static Vector selectPermissions()
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.addOrderByColumn(PermissionPeer.NAME);
        return PermissionPeer.doSelect(criteria);
    }

    /**
     * Returns a single Permission object given its id.
     *
     * @param id An int with the permission id.
     * @return A Permission, null if no Permission object could be
     * found with the given id.
     * @exception Exception, a generic exception.
     */
    public static Permission selectOnePermission(int id)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( PERMISSION_ID, id );
        Vector vec = PermissionPeer.doSelect(criteria);
        if (vec.size() == 1)
            return (Permission) vec.elementAt(0);
        return null;
    }

    /**
     * Returns a Vector of Permission's in the specified Role.  This
     * does a join.
     *
     * @param role_id An int with the role id.
     * @return A Vector with the Permission objects.
     * @exception Exception, a generic exception.
     */
    public static final Vector getPermissions(int role_id)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( RolePermissionPeer.ROLE_ID, role_id );
        criteria.addSelectColumn(PermissionPeer.PERMISSION_ID)
                .addSelectColumn(PermissionPeer.NAME);
        return doSelect(criteria);
    }

    /**
     * Pass in two Vector's of Permission Objects.  It will return a
     * new Vector with the difference of the two Vectors: C = (A - B).
     *
     * @param some Vector B in C = (A - B).
     * @param all Vector A in C = (A - B).
     * @return Vector C in C = (A - B).
     */
    public static final Vector getDifference(Vector some,
                                             Vector all)
    {
        Vector clone = (Vector)all.clone();
        for (Enumeration e = some.elements() ; e.hasMoreElements() ;)
        {
            Permission tmp = (Permission) e.nextElement();
            for (Enumeration f = clone.elements() ; f.hasMoreElements() ;)
            {
                Permission tmp2 = (Permission) f.nextElement();
                if (tmp.getId() == tmp2.getId())
                {
                    clone.removeElement(tmp2);
                    break;
                }
            }
        }
        return clone;
    }

    /**
     * Removes a Permission from a Role.
     *
     * @param perm_id An int with the permission id.
     * @param role_id An int with the role id.
     * @exception Exception, a generic exception.
     */
    public static final void deletePermissionFromRole(int perm_id,
                                                      int role_id)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( RolePermissionPeer.ROLE_ID, role_id );
        criteria.add ( RolePermissionPeer.PERMISSION_ID, perm_id );
        BasePeer.doDelete(criteria);
    }

    /**
     * Removes all Permissions from a Role.
     *
     * @param role_id An int with the role id.
     * @exception Exception, a generic exception.
     */
    public static final void deleteAllPermissionsFromRole(int role_id)
        throws Exception
    {
        BasePeer.deleteAll(RolePermissionPeer.TABLE_NAME,
                           RolePermissionPeer.ROLE_ID,
                           role_id );
    }

    /**
     * Adds a Permission to a Role.
     *
     * @param perm_id An int with the permission id.
     * @param role_id An int with the role id.
     * @exception Exception, a generic exception.
     */
    public static final void addPermissionToRole(int perm_id,
                                                 int role_id)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( RolePermissionPeer.ROLE_ID, role_id );
        criteria.add ( RolePermissionPeer.PERMISSION_ID, perm_id );
        BasePeer.doInsert(criteria);
    }

    /**
     * Adds all Permissions to a Role.
     *
     * @param role_id An int with the role id.
     * @exception Exception, a generic exception.
     */
    public static final void addAllPermissionsToRole(int role_id)
        throws Exception
    {
        deleteAllPermissionsFromRole(role_id);
        Vector allPerms = PermissionPeer.selectPermissions();
        for (Enumeration e = allPerms.elements() ; e.hasMoreElements() ;)
        {
            Permission perm = (Permission) e.nextElement();
            Criteria criteria = new Criteria();
            criteria.add ( RolePermissionPeer.ROLE_ID, role_id );
            criteria.add ( RolePermissionPeer.PERMISSION_ID, perm.getId() );
            BasePeer.doInsert(criteria);
        }
    }
}
