package org.apache.turbine.om.security.peer;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Utility Classes
import org.apache.turbine.om.peer.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.map.*;
import org.apache.turbine.services.resources.TurbineResources;

// Village Database Classes
import com.workingdogs.village.*;

// Java stuff.
import java.util.*;
import java.sql.*;

/**
 * This class handles all the database access for the ROLE table.
 * This table contains all the roles that a given member can play.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: RolePeer.java,v 1.2 2000/08/29 17:36:10 gonzalo Exp $
 */
public class RolePeer extends BasePeer
{
    private static final TurbineMapBuilder mapBuilder = (TurbineMapBuilder) getMapBuilder();

    /** The table name for this peer. */
    private static final String TABLE_NAME = mapBuilder.getTableRole();

    /** The column name for the role id field. */
    public static final String ROLE_ID = mapBuilder.getRole_RoleId();

    /** The column name for the name field. */
    public static final String NAME = mapBuilder.getRole_Name();

    /** The Oracle sequence name for this peer. */
    private static final String SEQUENCE_NAME = mapBuilder.getSequenceRole();


    /**
     * Retrieves/assembles a RoleSet of all of the Roles.
     *
     * @param criteria The criteria to use.
     * @return A RoleSet.
     * @exception Exception, a generic exception.
     */
    public static RoleSet retrieveSet()
        throws Exception
    {
        return retrieveSet(new Criteria());
    }

    /**
    * Retrieves/assembles a RoleSet based on the Criteria passed in
    */
    public static RoleSet retrieveSet(Criteria criteria) throws Exception
    {
        Vector results = RolePeer.doSelect(criteria);
        RoleSet rs = new RoleSet();
        for (int i=0; i<results.size(); i++)
        {
            rs.add( (Role)results.elementAt(i) );
        }
        return rs;
    }

    /**
     * Issues a select based on a criteria.
     *
     * @param Criteria object containing data that is used to create
     * the SELECT statement.
     * @return Vector containing Role objects.
     * @exception Exception, a generic exception.
     */
    public static Vector doSelect(Criteria criteria)
        throws Exception
    {
        criteria.addSelectColumn(ROLE_ID)
                .addSelectColumn(NAME);

        if (criteria.getOrderByColumns() == null ||
            criteria.getOrderByColumns().size() == 0)
        {
            criteria.addOrderByColumn(NAME);
        }

        // Place any checks here to intercept criteria which require
        // custom SQL.  For example:
        // if ( criteria.containsKey("SomeTable.SomeColumn") )
        // {
        //     String whereSql = "SomeTable.SomeColumn IN (Select ...";
        //     criteria.add("SomeTable.SomeColumn",
        //                  whereSQL, criteria.CUSTOM);
        // }

        // BasePeer returns a Vector of Value (Village) arrays.  The
        // array order follows the order columns were placed in the
        // Select clause.
        Vector rows = BasePeer.doSelect(criteria);
        Vector results = new Vector();

        // Populate the object(s).
        for ( int i=0; i<rows.size(); i++ )
        {
            Role obj = new Role();
            Record row = (Record)rows.elementAt(i);
            obj.setId( row.getValue(1).asInt() );
            obj.setName( row.getValue(2).asString() );
            results.addElement( obj );
        }

        return results;
    }

    /**
     * Issues an update based on a criteria.
     *
     * @param Criteria object containing data that is used to create
     * the UPDATE statement.
     * @exception Exception, a generic exception.
     */
    public static void doUpdate(Criteria criteria)
        throws Exception
    {
        Criteria selectCriteria = new Criteria(2);
        selectCriteria.put( ROLE_ID, criteria.remove(ROLE_ID) );
        BasePeer.doUpdate( selectCriteria, criteria );
    }

    /**
     * Get the name of this table.
     *
     * @return A String with the name of the table.
     */
    public static String getTableName()
    {
        return TABLE_NAME;
    }

    /**
     * Returns the full name of a column.
     *
     * @return A String with the full name of the column.
     */
    public static String getColumnName (String name)
    {
        StringBuffer sb = new StringBuffer();
        sb.append (TABLE_NAME);
        sb.append (".");
        sb.append (name);
        return sb.toString();
    }

    /**
     * Adds a role into the system.
     *
     * @param name A String describing the inserted role.
     * @return An Object.
     * @exception Exception, a generic exception.
     */
    public static Object insertRole (String name )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( NAME, name );
        return RolePeer.doInsert(crit);
    }

    /**
     * Updates a role into the system.
     *
     * @param id An int with the role id.
     * @param name A String describing the updated role.
     * @exception Exception, a generic exception.
     */
    public static void updateRole ( int id,
                                    String name )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( ROLE_ID, id );
        crit.add ( NAME, name );
        RolePeer.doUpdate(crit);
    }

    /**
     * Delete a role by id number.
     *
     * @param id An int with the role id.
     * @exception Exception, a generic exception.
     */
    public static void deleteRole (int id )
        throws Exception
    {
        Criteria crit = new Criteria();
        crit.add ( ROLE_ID, id );
        doDelete(crit);
    }

    /**
     * Returns a vector of Role objects.
     *
     * @return A Vector with the Role objects.
     * @exception Exception, a generic exception.
     */
    public static Vector selectRoles()
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.addOrderByColumn(RolePeer.NAME);
        return RolePeer.doSelect(criteria);
    }

    /**
     * Returns a single Role object.
     *
     * @param id An int with the role id.
     * @return A Role, null if no Role object could be found.
     * @exception Exception, a generic exception.
     */
    public static Role selectOneRole(int id)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( ROLE_ID, id );
        Vector vec = RolePeer.doSelect(criteria);
        if (vec.size() == 1)
            return (Role) vec.elementAt(0);
        return null;
    }
}
