package org.apache.turbine.om.user.peer;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.sql.*;
import java.io.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.map.*;
import org.apache.turbine.om.*;
import org.apache.turbine.om.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.turbine.services.resources.TurbineResources;

// Village Database Classes
import com.workingdogs.village.*;

/**
 * This class handles all the database access for the User/Visitor
 * table.  This table contains all the information for a given user.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: TurbineUserPeer.java,v 1.3 2000/09/01 16:30:54 mcnally Exp $
 */
public class TurbineUserPeer extends BasePeer
{
    /** The mapBuilder for this Peer. */
    private static final TurbineMapBuilder mapBuilder = (TurbineMapBuilder) getMapBuilder();

    /** The column name for the visitor id field. */
    private static final String VISITOR_ID_COLUMN = mapBuilder.getVisitor_VisitorId();

    /** The column name for the visitor id field. */
    private static final String OBJECT_DATA_COLUMN = "OBJECTDATA";

    /** The key name for the first name field. */
    public static final String FIRST_NAME_COLUMN = "FIRST_NAME";

    /** The key name for the last name field. */
    public static final String LAST_NAME_COLUMN = "LAST_NAME";

    /** The column name for the modified field. */
    private static final String MODIFIED_COLUMN = "MODIFIED";

    /** The column name for the created field. */
    private static final String CREATED_COLUMN = "CREATED";

    /** The column name for the last_login field. */
    private static final String LAST_LOGIN_COLUMN = "LASTLOGIN";

    /** The column name for the email field. */
    private static final String EMAIL_COLUMN = "EMAIL";

    /** The column name for the confirm_value field. */
    private static final String CONFIRM_VALUE_COLUMN = "CONFIRM_VALUE";

    /*
     * This is the value that is stored in the database for confirmed
     * users.
     */
    public static final String CONFIRM_DATA = "CONFIRMED";

    /** The table name for this peer. */
    private static final String TABLE_NAME = mapBuilder.getTableVisitor();

    // Criteria Keys
    /** The key name for the visitor id field. */
    public static final String VISITOR_ID = mapBuilder.getVisitor_VisitorId();

    /** The key name for the first name field. */
    public static final String FIRST_NAME = TABLE_NAME + "." + FIRST_NAME_COLUMN;

    /** The key name for the last name field. */
    public static final String LAST_NAME = TABLE_NAME + "." + LAST_NAME_COLUMN;

    /** The key name for the username field. */
    public static final String USERNAME = mapBuilder.getVisitor_Username();

    /** The key name for the password field. */
    public static final String PASSWORD = mapBuilder.getVisitor_Password();

    /** The key name for the modified field. */
    public static final String MODIFIED = TABLE_NAME + "." + MODIFIED_COLUMN;

    /** The key name for the created field. */
    public static final String CREATED = TABLE_NAME + "." + CREATED_COLUMN;

    /** The key name for the last_login field. */
    public static final String LAST_LOGIN = mapBuilder.getVisitor_LastLogin();

    /** The key name for the email field. */
    public static final String EMAIL = TABLE_NAME + "." + EMAIL_COLUMN;

    /** The key name for the confirm_value field. */
    public static final String CONFIRM_VALUE = TABLE_NAME + "." + CONFIRM_VALUE_COLUMN;

    /** The key name for the object_data field. */
    public static final String OBJECT_DATA = mapBuilder.getVisitor_ObjectData();

    /** The Oracle sequence name for this peer. */
    private static final String SEQUENCE_NAME = mapBuilder.getSequenceVisitor();

    /** The schema. */
    private static Schema schema = initTableSchema(TABLE_NAME);

    /** The columns. */
    private static com.workingdogs.village.Column[] columns = initTableColumns(schema);

    /** The names of the columns. */
    public static String[] columnNames = initColumnNames(columns);

    /** The keys for the criteria. */
    public static String[] criteriaKeys = initCriteriaKeys(TABLE_NAME, columnNames);


    /**
     * Get the name of this table.
     *
     * @return A String with the name of the table.
     */
    public static String getTableName()
    {
        return TABLE_NAME;
    }

    /**
     * Returns the full name of a column.
     *
     * @return A String with the full name of the column.
     */
    public static String getColumnName (String name)
    {
        StringBuffer sb = new StringBuffer();
        sb.append (TABLE_NAME);
        sb.append (".");
        sb.append (name);
        return sb.toString();
    }


    /**
     * Issues a select based on a criteria.
     *
     * @param criteria Object containing data that is used to create
     * the SELECT statement.
     * @return Vector containing TurbineUser objects.
     * @exception Exception, a generic exception.
     */
    public static Vector doSelect(Criteria criteria)
        throws Exception
    {
        return doSelect(criteria, (User)null);
    }

    /**
     * Issues a select based on a criteria.
     *
     * @param criteria Object containing data that is used to create
     * the SELECT statement.
     * @param current User object that is to be used as part of the
     * results - if not passed, then a new one is created.
     * @return Vector containing TurbineUser objects.
     * @exception Exception, a generic exception.
     */
    public static Vector doSelect(Criteria criteria,
                                  User current)
        throws Exception
    {
        // Set values are where columns are expected.  They are not
        // required to be in these positions, as we set the positions
        // while populating the select columns.
        int idPosition = 1;
        int objectDataPosition = columnNames.length;

        for( int i=0; i<columnNames.length; i++ )
        {
            criteria.addSelectColumn(new StringBuffer()
                .append(TABLE_NAME)
                .append(".")
                .append(columnNames[i]).toString() );
            if (columnNames[i].equals(VISITOR_ID_COLUMN))
                idPosition = i+1;
            if (columnNames[i].equals(OBJECT_DATA_COLUMN))
                objectDataPosition = i+1;
        }

        if (criteria.getOrderByColumns() == null)
        {
            criteria.addOrderByColumn(LAST_NAME);
        }

        // Place any checks here to intercept criteria which require
        // custom SQL.  For example:
        // if ( criteria.containsKey("SomeTable.SomeColumn") )
        // {
        //     String whereSql = "SomeTable.SomeColumn IN (Select ...";
        //     criteria.add("SomeTable.SomeColumn",
        //                  whereSQL, criteria.CUSTOM);
        // }

        // BasePeer returns a Vector of Record (Village) objects.  The
        // array order follows the order columns were placed in the
        // Select clause.
        Vector rows = BasePeer.doSelect(criteria);
        Vector results = new Vector();

        // Populate the object(s).
        for ( int i=0; i<rows.size(); i++ )
        {
            User obj = null;
            if (i==0 && current != null)
                obj = current;
            else
                obj = UserFactory.getUser();
            Record row = (Record)rows.elementAt(i);

            obj.setId( row.getValue(idPosition).asBigDecimal() );

            // Restore the Permanent Storage Hashtable.  First the
            // Hashtable is restored, then any explicit table columns
            // which should be included in the Hashtable are added.
            Hashtable tempHash = null;
            byte[] objectData = (byte[])row.getValue(objectDataPosition).asBytes();

            if (objectData != null)
            {
                // These streams are closed in finally.
                ObjectInputStream in = null;
                ByteArrayInputStream bin =
                    new ByteArrayInputStream(objectData);
                BufferedInputStream bufin =
                    new BufferedInputStream(bin);
                try
                {
                    in = new ObjectInputStream(bufin);

                    // If object_data has not been initialized, and
                    // exception will occur.
                    tempHash = (Hashtable) in.readObject();
                }
                catch (Exception e)
                {
                }
                finally
                {
                    if (bin != null) bin.close();
                    if (bufin != null) bufin.close();
                    if (in != null) in.close();
                }
            }

            if (tempHash == null)
            {
                tempHash = new Hashtable(10);
            }
            for( int j=0; j<columnNames.length; j++ )
            {
                if ( ! ( columnNames[j].equalsIgnoreCase( VISITOR_ID_COLUMN )
                         ||  columnNames[j].equalsIgnoreCase( OBJECT_DATA_COLUMN ) ))
                {
                    Object obj2 = null;
                    Value value = row.getValue(j+1);
                    if (value.isByte()) obj2 = new Byte(value.asByte());
                    if (value.isBigDecimal()) obj2 = value.asBigDecimal();
                    if (value.isBytes()) obj2 = value.asBytes();
                    if (value.isDate()) obj2 = value.asDate();
                    if (value.isShort()) obj2 = new Short(value.asShort());
                    if (value.isInt()) obj2 = new Integer(value.asInt());
                    if (value.isLong()) obj2 = new Long(value.asLong());
                    if (value.isDouble()) obj2 = new Double(value.asDouble());
                    if (value.isFloat()) obj2 = new Float(value.asFloat());
                    if (value.isBoolean()) obj2 = new Boolean(value.asBoolean());
                    if (value.isString()) obj2 = value.asString();
                    if (value.isTime()) obj2 = value.asTime();
                    if (value.isTimestamp()) obj2 = value.asTimestamp();
                    if (value.isUtilDate()) obj2 = value.asUtilDate();
                    if ( obj2 != null )
                        tempHash.put( criteriaKeys[j], obj2 );
                }
            }
            obj.setPermStorage( tempHash );
            // Add User to the return Vector.
            results.addElement( obj );
        }
        return results;
    }

    /**
     * Issues an update based on a criteria.
     * The criteria only uses VISITOR_ID.
     *
     * @param criteria Object containing data that is used to create
     * the UPDATE statement.
     * @exception Exception, a generic exception.
     */
    public static void doUpdate(Criteria criteria)
        throws Exception
    {
        Criteria selectCriteria = new Criteria(2);
        selectCriteria.put( VISITOR_ID, criteria.remove(VISITOR_ID) );
        BasePeer.doUpdate( selectCriteria, criteria );
    }

    /**
     * This method reports whether or not the user has been confirmed
     * in the system by checking the TurbineUserPeer.CONFIRM_VALUE
     * column to see if it is equal to CONFIRM_DATA.
     *
     * @param user The User object.
     * @return True if the user has been confirmed.
     * @exception Exception, a generic exception.
     */
    public static boolean isConfirmed(User user)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.addSelectColumn(VISITOR_ID);
        criteria.add(CONFIRM_VALUE, CONFIRM_DATA);
        criteria.add(VISITOR_ID,user.getId());
        Vector info = TurbineUserPeer.doSelect(criteria);
        if (info.size()>0)
            return true;
        return false;
    }

    /**
     * This method checks to see if the USERNAME already exists in the
     * system or not.
     *
     * @param name The User name.
     * @return True if the user already exists.
     * @exception Exception, a generic exception.
     */
    public static boolean checkExists(String name)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add(USERNAME, name);
        Vector info = TurbineUserPeer.doSelect(criteria);
        if (info.size()>0)
            return true;
        return false;
    }

    /**
     * Returns a vector of all User objects.
     *
     * @return A Vector with all users in the system.
     * @exception Exception, a generic exception.
     */
    public static Vector selectAllUsers()
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.addOrderByColumn(TurbineUserPeer.LAST_NAME);
        criteria.addOrderByColumn(TurbineUserPeer.FIRST_NAME);
        criteria.setIgnoreCase(true);
        return TurbineUserPeer.doSelect(criteria);
    }

    /**
     * Returns a vector of all confirmed User objects.
     *
     * @return A Vector with all confirmed users in the system.
     * @exception Exception, a generic exception.
     */
    public static Vector selectAllConfirmedUsers()
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add ( TurbineUserPeer.CONFIRM_VALUE, CONFIRM_DATA );
        criteria.addOrderByColumn(TurbineUserPeer.LAST_NAME);
        criteria.addOrderByColumn(TurbineUserPeer.FIRST_NAME);
        criteria.setIgnoreCase(true);
        return TurbineUserPeer.doSelect(criteria);
    }
}