package org.apache.turbine.samples.screens;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine Modules
import org.apache.turbine.modules.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/**
 * This is a sample Screen module.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: DefaultScreen.java,v 1.2 2000/08/29 20:45:45 gonzalo Exp $
 */
public class DefaultScreen
    extends Screen
{
    /**
     * Build the Screen.
     *
     * @param data Turbine information.
     * @return A ConcreteElement.
     * @exception Exception, a generic exception.
     */
    public ConcreteElement doBuild( RunData data )
        throws Exception
    {
        // Set the Title tag of the page.
        data.setTitle("Welcome to the DefaultScreen!");

        // If there are GET/POST/PATH_INFO variables, put them into a
        // <PRE></PRE> tag so that they can be displayed on the
        // page. This is of course only for example purposes.
        PRE pre = new PRE();
        boolean hasValues = false;
        Enumeration keys = data.getParameters().keys();
        while ( keys.hasMoreElements() )
        {
            String key = (String) keys.nextElement();
            String value = data.getParameters().getString(key);
            pre.addElement(key + " = " + value + "\n");
            hasValues = true;
        }

        // Show Roles and Permissions assigned to this User.
        RoleSet rs = data.getACL().getRoles();
        rs.resetIterator();
        PermissionSet ps = data.getACL().getPermissions();
        ps.resetIterator();
        String roleName = null;
        String permissionName = null;
        P p = new P();
        p.addElement( new H3().addElement("Roles") );
        while ( rs != null  &&  rs.hasMoreElements() )
        {
            roleName = rs.next().getName();
            p.addElement( roleName ).addElement( new BR() );
        }
        p.addElement( new H3().addElement("Permissions") );
        while ( ps != null  &&  ps.hasMoreElements() )
        {
            permissionName = ps.next().getName();
            p.addElement( permissionName ).addElement( new BR() );
        }
        p.addElement( new BR() );

        // Do a couple checks.
        String roleCheckTrue = "Not Set";
        String roleCheckFalse = "Not Set";
        String permCheckTrue = "Not Set";
        String permCheckFalse = "Not Set";
        if ( data.getACL().hasRole( roleName ) )
        {
            roleCheckTrue = "True";
        }
        else
        {
           roleCheckTrue = "False";
        }
        if ( data.getACL().hasRole( "zx78sk9k--[]dd" ) )
        {
            roleCheckFalse = "True";
        }
        else
        {
            roleCheckFalse = "False";
        }
        if ( data.getACL().hasPermission( permissionName ) )
        {
            permCheckTrue = "True";
        }
        else
        {
            permCheckTrue = "False";
        }
        if ( data.getACL().hasPermission( "zx78sk9k--[]dd" ) )
        {
            permCheckFalse = "True";
        }
        else
        {
            permCheckFalse = "False";
        }
        p.addElement("Role: This value should be True: " +
                     roleCheckTrue).addElement( new BR() );
        p.addElement("Role: This value should be False: " +
                     roleCheckFalse).addElement( new BR() );
        p.addElement("Perm: This value should be True: " +
                     permCheckTrue).addElement( new BR() );
        p.addElement("Perm: This value should be False: " +
                     permCheckFalse).addElement( new BR() );

        // Check whether BasePeer is working.
        String[] columns = TurbineUserPeer.columnNames;
        p.addElement( new BR() );
        p.addElement( new H4("User columns") );
        if ( columns == null)
        {
            p.addElement( "Columns are null ????" ).addElement( new BR() );
        }
        else
        {
            for (int i=0; i<columns.length; i++)
            {
                p.addElement( columns[i] ).addElement( new BR() );
            }
        }

        // Testing log file.
        Log.note("This is a note.");
        Log.warn("This is a warning!");
        Log.error("This is an error!!");
        try
        {
            throw new Exception("I'm a test Exception");
        }
        catch(Exception e)
        {
            Log.error(e);
        }

        // Start adding things to the page (screen).
        StringElement body = new StringElement();

        // Show the Title at the top of the page in <H1> tags.
        body.addElement (new H1().addElement(data.getTitle()));

        // Create an example URL that can be clicked on.
        A a = new A();
        a.setHref(
                  new DynamicURI(data, "DefaultScreen").addPathInfo("here","there").toString()
                      ).addElement("An example link");
        body.addElement (new H4().addElement(a));

        // Show that the userName that the user logged in with is
        // cached.
        body.addElement ( "Welcome: " + data.getUser().getUserName() );

        body.addElement ( new P().addElement( "If you loaded the default_roles_permission.sql script (see docs dir). " +
                                              " and logged in as turbine, you should see the User Administrator link below. " +
                                              " This is just an example of access control." ) );

        // Added for a sample.  Run default_roles_pemissions.sql
        // script for turbine_root.
        if ( data.getACL().hasRole("turbine_root") )
        {
            body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.FindUser" ).toString()).addElement("User Administrator")));
        }

        // Add the various Admin pages so they can be accessed fast.
        body.addElement( new P().addElement("Admin pages: ") );
        body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.FindUser" ).toString()).addElement("User Administrator")));

        /*
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.AddUser" ).toString()).addElement("AddUser")));
          //body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.EditUser" ).toString()).addElement("EditUser")));
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.ChooseUser" ).toString()).addElement("ChooseUser")));
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.DisplayUserList" ).toString()).addElement("DisplayUserList")));
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.FindUser" ).toString()).addElement("FindUser")));
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.PermissionEditor" ).toString()).addElement("PermissionEditor")));
          body.addElement( new P().addElement( new A().setHref(new DynamicURI ( data, "admin.RoleEditor" ).toString()).addElement("RoleEditor")));
        */

        body.addElement ( new P() );

        // Also show how a user can be tracked through a site really
        // easily.
        body.addElement ( "Number of times this user has accessed this" +
            " site: " + data.getUser().getAccessCounter() ).addElement( new BR() );
        body.addElement ( "Number of times this user has accessed this" +
            " site during this login session: " + data.getUser().getAccessCounterForSession() );

        body.addElement ( new P() );
        body.addElement ( "StackTrace A: " );
        body.addElement ( ((TurbineUser)data.user).stackTrace );
        body.addElement ( new P() );
        body.addElement ( "Turbine StackTrace B no longer exists: " );

        /*
          data.getPage().getBody().addElement ( new P() );
          data.getPage().getBody().addElement ( "StackTrace B: " );
          data.getPage().getBody().addElement ( new PRE ( Turbine.stackTrace ) );
          data.getPage().getBody().addElement ( new P() );
        */

        // Add the <PRE> tag information to the bottom of the page.
        if ( hasValues )
            body.addElement (pre);

        body.addElement(p);
        return body;
    }
}
