package org.apache.turbine.samples.screens;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Imports
import java.io.*;
import java.util.*;
import java.net.*;

// Turbine/ECS Imports
import org.apache.turbine.modules.*;
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/**
 * Sample screen to show how it is possible to serve images directly
 * through Turbine.
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @version $Id: ImageServer.java,v 1.2 2000/08/29 20:45:46 gonzalo Exp $
 */
public class ImageServer
    extends Screen
{
    private static String root =
        TurbineResources.getString("screen.ImageServer.root", "d:/MMMMMMMMM");
    private static final int GIF = 1;
    private static final int JPEG = 2;
    private static final int PNG = 3;

    private static Hashtable imageCache =
        new Hashtable(TurbineResources.getInt("imageserver.cache.initialsize", 10));

    /**
     * Set the Layout to null. This causes the DefaultPage to try to
     * execute just the Screen without a Layout.
     *
     * @param data Turbine information.
     * @return A null String.
     */
    public String getLayout(RunData data)
    {
        return null;
    }

    /**
     * Build the Screen.
     *
     * @param data Turbine information.
     * @return A ConcreteElement.
     * @exception Exception, a generic exception.
     */
    public ConcreteElement doBuild( RunData data )
        throws Exception
    {
        byte[] buffer = null;

        String image = data.getParameters().getString("image","").trim();

        if ( image == null )
        {
            // Should do more logging here.
            Log.error( "ImageServer - image is null!" );
            return null;
        }

        // SECURITY!!!  Prevent people from hacking the path.
        if ( image.indexOf("..") != -1 )
        {
            // Should do more logging here.
            Log.error( new Exception("ImageServer - security violation!  " + image) );
            return null;
        }

        File file = new File(root + image);
        if ( ! file.exists() )
        {
            // Should do more logging here.
            Log.error( new FileNotFoundException(image) );
            return null;
        }

        AnImage aimage = null;
        // checkCache(file);
        if ( aimage == null )
        {
            FileInputStream fis = new FileInputStream(file);
            buffer = new byte[(int) file.length()];
            fis.read(buffer);
            fis.close();
            // cacheImage(file, buffer, type);
            this.setContentType(data, file.getName());
        }
        else
        {
            buffer = aimage.data;
            this.setContentType(data,aimage.type);
        }

        data.getResponse().setDateHeader("Last-Modified", file.lastModified());
        data.getResponse().setHeader("Accept-Ranges","bytes");
        data.getResponse().setStatus ( data.getStatusCode() );
        data.getResponse().setContentType( data.getContentType() );
        data.getResponse().setContentLength(buffer.length);
        data.getResponse().getOutputStream().write ( buffer );
        return null;
    }

    /**
     * Set the content type.
     *
     * @param data Turbine information.
     * @param filename Name of file.
     */
    private void setContentType(RunData data,
                                String filename)
    {
        if (filename.endsWith(".jpg") || filename.endsWith(".jpeg"))
        {
            setContentType(data, JPEG);
        }
        else if (filename.endsWith(".gif"))
        {
            setContentType(data, GIF);
        }
        else if (filename.endsWith(".png"))
        {
            setContentType(data, PNG);
        }
    }

    /**
     * Set the content type.
     *
     * @param data Turbine information.
     * @param type Type of file.
     */
    private void setContentType(RunData data, int type)
    {
        switch (type)
        {
        case JPEG:
            data.setContentType("image/jpeg");
            break;
        case GIF:
            data.setContentType("image/gif");
            break;
        case PNG:
            data.setContentType("image/png");
            break;
        default:
            data.setContentType("image/jpeg");
            break;
        }
    }

    /* Image cache stuff

    //
    // Cache an image.
    //
    // @param file The File with the image.
    // @param buffer The image in memory.
    // @param type The image type.
    // @exception MalformedURLException, the URL is invalid.
    //
    private void cacheImage(File file,
                            byte[] buffer,
                            int type)
        throws MalformedURLException
    {
        AnImage ai = new AnImage();
        ai.data = buffer;
        ai.type = type;
        imageCache.put ( file.toURL().toString(), ai );
    }

    //
    // Check the cache for an image.
    //
    // @param file The File with the image.
    // @return AnImage.
    // @exception MalformedURLException, the URL is invalid.
    //
    private AnImage checkCache(File file)
        throws MalformedURLException
    {
        try
        {
            AnImage ai = (AnImage) imageCache.get(file.toURL().toString());
            return ai;
        }
        catch ( Exception e )
        {
        }
        return null;
    }

    */

    private class AnImage
    {
        byte[] data = null;
        int type = -1;
    }
}
