package org.apache.turbine.services.freemarker;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.pool.*;
import org.apache.ecs.html.*;

// Village Database Classes
import com.workingdogs.village.*;

// FreeMarker Classes
import freemarker.template.*;

/**
 * Creates a TEXTAREA html tag.  The rows, columns, and wrap attribute
 * can be passed in through a list.  Extension of FreeMarker.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: DynamicURIModel.java,v 1.2 2000/08/29 23:19:39 gonzalo Exp $
 */
public class DynamicURIModel
    implements TemplateMethodModel
{
    private RunData data;

    /**
     * Constructor.
     *
     * @param data Turbine information.
     */
    public DynamicURIModel(RunData data)
    {
        this.data = data;
    }

    /**
     * Method called by FreeMarker during template parsing.  A list of
     * strings are passed in from the template.  The first string can
     * specify https or http or be the only required String which sets
     * the template parameter.  Any additional String pairs are taken
     * to be pathInfo data.  An "&" String can be used to specify
     * query data which should then be followed by pairs of Strings.
     * E.g (in a template where the DynamicURIModel has been stored
     * under the key "links":
     * ${links("http", "/subdir/directions.html",
     *         "from", "LA", "&", "to", "SF")}
     *
     * @param args A List of Strings passed from the template.
     * @return A TemplateModel with a String representation of the
     * DynamicURI.
     * @exception TemplateModelException.
     */
    public TemplateModel exec(ArgumentList args)
        throws TemplateModelException
    {
        DynamicURI uri = new DynamicURI(data);
        String firstArg = (String)args.get(0);
        int start=1;
        if (firstArg.startsWith("http"))
        {
            int colonPosition = firstArg.indexOf(':');
            if (colonPosition == -1)
            {
                uri.setServerScheme(firstArg);
            }
            else
            {
                uri.setServerScheme(firstArg.substring(0,colonPosition));
                int secondColonPosition = firstArg.indexOf(':', colonPosition+1);
                if (secondColonPosition == -1)
                {
                    uri.setServerName( firstArg.substring(colonPosition+3) );
                }
                else
                {
                    uri.setServerName(firstArg.substring(colonPosition+3, secondColonPosition));
                    uri.setServerPort(Integer.parseInt(firstArg.substring(secondColonPosition+1)));
                }
            }
            uri.addPathInfo("template", (String)args.get(1));
            start=2;
        }
        else
        {
            uri.addPathInfo("template", firstArg.replace('/', ','));
        }
        boolean firstOfPair = true;
        boolean queryData = false;
        String first = null;
        for (int i=start; i<args.size(); i++)
        {
            String tempArg = (String)args.get(i);
            if (firstOfPair)
            {
                if (tempArg.equals("?"))
                {
                    queryData = true;
                }
                else
                {
                    first = tempArg;
                    firstOfPair = false;
                }
            }
            else
            {
                if (queryData)
                {
                    uri.addQueryData(first, tempArg);
                }
                else
                {
                    uri.addPathInfo(first, tempArg);
                }
                firstOfPair = true;
            }
        }
        return new SimpleScalar( uri.toString() );
    }

    /**
     * Required method in TemplateMethodModel, not implemented.
     *
     * @return Always false.
     */
    public boolean isEmpty()
    {
        return false;
    }
}
