package org.apache.turbine.services.freemarker;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.ecs.html.*;

// FreeMarker Classes
import freemarker.template.*;

/**
 * Creates a TEXTAREA html tag.  The rows, columns, and wrap attribute
 * can be passed in through a list. This object is intended to be used
 * by applications, so that the name and value attributes will not
 * accidently be altered by designers.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: TextAreaModel.java,v 1.2 2000/08/29 23:19:40 gonzalo Exp $
 */
public class TextAreaModel
    extends TextArea
    implements TemplateMethodModel
{
    /**
     * Creates a textarea html tag.
     *
     * @param name The input name attribute.
     * @param value The text between the <textarea> and </textarea>
     * tags.
     */
    public TextAreaModel(String name,
                         String value)
    {
        super();
        setName(name);
        addElement(value);
    }

    /**
     * Method called by FreeMarker during template parsing.
     *
     * @param args A List of Strings passed from the template to set,
     * in order, the row and column attributes, and optionally the
     * wrap attribute.
     * @return <textarea name="name" rows="args[0]" columns="args[1]"
     * wrap="args[2]">value</textarea>
     * @exception TemplateModelException.
     */
    public TemplateModel exec(ArgumentList args)
        throws TemplateModelException
    {
        if (args.size() > 1)
        {
            String rows = (String)args.get(0);
            String columns = (String)args.get(1);
            try
            {
                Integer.parseInt(rows);
                Integer.parseInt(columns);
                setRows(rows);
                setCols(columns);
            }
            catch (Exception e) {}
            if (args.size() > 2)
            {
                setWrap((String)args.get(2));
            }
        }
        return new SimpleScalar( toString() );
    }

    /**
     * Required method in TemplateMethodModel, not implemented.
     *
     * @return Always false.
     */
    public boolean isEmpty()
    {
        return false;
    }
}
