package org.apache.turbine.services.naming;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import javax.naming.*;

// Turbine stuff.
import org.apache.turbine.util.*;
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.*;

/**
 * This class is the default implementation of NamingService, which
 * provides JNDI naming contexts.
 *
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @version $Id: TurbineNamingService.java,v 1.3 2000/08/30 20:57:07 gonzalo Exp $
 */
public class TurbineNamingService
    extends TurbineBaseService
    implements NamingService
{
    /**
     * Places the contexts defined in the TurbineResources instance
     * (if any) into the data.contexts Hashtable.
     *
     * @param data The RunData object for the current request.
     * @exception InitializationException, if there was a problem
     * during initialization.
     */
    public void init( RunData data )
        throws InitializationException
    {
        // Context properties are specified in lines in the properties
        // file that begin with "context.contextname.", allowing
        // multiple named contexts to be used.  Everything after the
        // "contextname."  is the name of the property that will be
        // used by the InitialContext class to create a new context
        // instance.

        try
        {
            org.apache.turbine.util.Log.note("Initializing JNDI contexts!");
            Enumeration contextKeys = TurbineResources.getKeys("context.");
            Hashtable contextPropsList = new Hashtable();
            while( contextKeys.hasMoreElements() )
            {
                String key = (String) contextKeys.nextElement();
                int start = key.indexOf(".") + 1;
                int end = key.indexOf(".", start);
                String contextName = key.substring(start, end);
                Properties contextProps = null;
                if( contextPropsList.containsKey(contextName) )
                {
                    contextProps =
                        (Properties) contextPropsList.get(contextName);
                }
                else
                {
                    contextProps = new Properties();
                }
                contextProps.put(key.substring(end + 1),
                                 TurbineResources.getString(key));
                contextPropsList.put(contextName, contextProps);
            }
            Enumeration contextPropsKeys = contextPropsList.keys();
            while( contextPropsKeys.hasMoreElements() )
            {
                String key = (String) contextPropsKeys.nextElement();
                Properties contextProps =
                    (Properties) contextPropsList.get(key);
                InitialContext context = new InitialContext(contextProps);
                data.getJNDIContexts().put(key, context);
            }
            org.apache.turbine.util.Log.note("Initialized JNDI contexts!");
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error("Failed to initialize JDNI contexts!");
            org.apache.turbine.util.Log.error(e);
            throw new InitializationException("Failed to initialize JDNI contexts!");
        }
    }

    /**
     * Return the Context with the specified name.  The Context is
     * constructed using the properties for the context with the
     * specified name; ie. those properties that start with
     * "services.servicename.properties.name.".
     *
     * @param name The name of the context.
     * @return The context with the specified name, or null if no
     * context exists with that name.
     */
    public Context getContext(String name)
    {
        // Get just the properties for the context with the specified
        // name.
        Properties allProps = getProperties();
        Properties contextProps = new Properties();
        Enumeration allKeys = allProps.keys();
        while( allKeys.hasMoreElements() )
        {
            String key = (String) allKeys.nextElement();
            String prefix = name + ".";
            if( key.startsWith(prefix) )
            {
                contextProps.put(key.substring(prefix.length()),
                                 allProps.get(key));
            }
        }

        // Construct a new context with the properties.
        try
        {
            return new InitialContext(contextProps);
        }
        catch( Exception e )
        {
            return null;
        }
    }
}
