package org.apache.turbine.services.resources;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine stuff.
import org.apache.java.util.*;

/**
 * This class encapsulates the functionality of a generic turbine
 * resource bundle.  It basically consists of a group of accessors for
 * configuration information.
 *
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @author <a href="mailto:jeffprickett@mindspring.com">Jeff Prickett</a>
 * @version $Id: GenericResources.java,v 1.3 2000/08/30 20:57:07 gonzalo Exp $
 */
public class GenericResources
{
    /** The configurations that this class holds. */
    private Configurations confs = null;

    /**
     * Constructor.
     */
    public GenericResources()
    {
    }

    /**
     * The constructor used by a sub class to create a resource
     * bundle.
     *
     * @param theConf The new configuration for the resource
     * bundles.
     */
    protected GenericResources(Configurations theConf)
    {
        confs = theConf;
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a boolean value.
     *
     * @param name The resource name.
     * @return The value of the named resource as a boolean.
     */
    public boolean getBoolean(String name)
    {
        return confs.getBoolean(name);
    }

    /**
     * The purppose of this method is to get the configuration
     * resource with the given name as a boolean value, or a default
     * value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the named resource as a boolean.
     */
    public boolean getBoolean(String name,
                              boolean def)
    {
        return confs.getBoolean(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a double.
     *
     * @param name The resource name.
     * @return The value of the named resource as double.
     */
    public double getDouble(String name)
    {
        return confs.getDouble(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a double, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the named resource as a double.
     */
    public double getDouble(String name,
                            double def)
    {
        return confs.getDouble(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a float.
     *
     * @param name The resource name.
     * @return The value of the resource as a float.
     */
    public float getFloat(String name)
    {
        return confs.getFloat(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a float, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a float.
     */
    public float getFloat(String name,
                          float def)
    {
        return confs.getFloat(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an Integer.
     *
     * @param name The resource name.
     * @return The value of the resource as an Integer.
     */
    public Integer getInteger(String name)
    {
        return new Integer(confs.getInteger(name));
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an Integer, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as an Integer.
     */
    public Integer getInteger(String name,
                              int def)
    {
        return new Integer(confs.getInteger(name, def));
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an integer.
     *
     * @param name The resource name.
     * @return The value of the resource as an integer.
     */
    public int getInt(String name)
    {
        return confs.getInteger(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an integer, or a default value.
     *
     * @param name The resource name
     * @param def The default value of the resource.
     * @return The value of the resource as an integer.
     */
    public int getInt(String name,
                      int def)
    {
        return confs.getInteger(name, def);
    }

    /**
     * Get the list of the keys contained in the configuration
     * repository.
     *
     * @return An Enumeration with the keys.
     */
    public Enumeration getKeys()
    {
        return confs.getRepository().keys();
    }

    /**
     * Get the list of the keys contained in the configuration
     * repository that match the specified prefix.
     *
     * @param prefix The String prefix to test against.
     * @return An Enumeration of keys that match the prefix.
     */
    public Enumeration getKeys(String prefix)
    {
        Enumeration keys = this.getKeys();
        Vector matchingKeys = new Vector();
        while( keys.hasMoreElements() )
        {
            Object key = keys.nextElement();
            if( key instanceof String && ((String) key).startsWith(prefix) )
            {
                matchingKeys.addElement(key);
            }
        }
        return matchingKeys.elements();
    }

    /*

    //
    // The purpose of this method is to get an Enumeration of all the
    // keys in the resource bundle.
    //
    // @return An enumeration full of all the keys.
    //
    public Enumeration getKeys()
    {
        return confs.getKeys();
    }


    //
    // The purpose of this method is to get an Enumeration full of all
    // the keys that begin with the given prefix.
    //
    // @param prefix The prefix of the keys to retrieve.
    // @return An enumeration full of keys that begin with the given
    // prefix.
    //
    public Enumeration getKeys(String prefix)
    {
        return confs.getKeys(prefix);
    }

    */

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a long.
     *
     * @param name The resource name.
     * @return The value of the resource as a long.
     */
    public long getLong(String name)
    {
        return confs.getLong(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a long, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a long.
     */
    public long getLong(String name,
                        long def)
    {
        return confs.getLong(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string.
     *
     * @param name The resource name.
     * @return The value of the resource as a string.
     */
    public String getString(String name)
    {
        return confs.getString(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a string.
     */
    public String getString(String name,
                            String def)
    {
        return confs.getString(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string array.
     *
     * @param name The resource name.
     * @return The value of the resource as a string array.
     */
    public String[] getStringArray(String name)
    {
        return confs.getStringArray(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a vector.
     *
     * @param name The resource name.
     * @return The value of the resource as a vector.
     */
    public Vector getVector(String name)
    {
        return confs.getVector(name);
    }

    /**
     * Get the Configurations that were used to define this object.
     *
     * @return The Configurations that were used to define this
     * object.
     */
    public Configurations getConfig()
    {
        return this.confs;
    }
}
