package org.apache.turbine.services.resources;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;

// Turbine stuff.
import org.apache.java.util.*;

/**
 * This class defines a message formatting scheme whereby Exception,
 * Log, and other messages can be formated given parameters.  The
 * message is a string and each message has a parameter list.  The
 * parameters are inserted into the message where there are $number
 * tokens in the message.  For example, $1 would insert the first
 * parameter string into the message instead of outputting $1.  This
 * is modelled after most UNIX shell scripting conventions for command
 * line arguments.
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @author <a href="mailto:jeffprickett@mindspring.com">Jeff Prickett</a>
 * @version $Id: MessageResources.java,v 1.2 2000/08/30 20:57:07 gonzalo Exp $
 */
public class MessageResources
    extends GenericResources
{
    /**
     * Constructor.
     *
     * @param theConf The configuration for the resource bundles.
     */
    public MessageResources(Configurations confs)
    {
        super(confs);
    }

    /**
     * The purpose of this method is output a formatted message.  The
     * message format takes a key which corresponds to a string.  The
     * message string is parsed by this method to add the parameters
     * into the formatted message at the right place.  The message
     * parameters are inserted in the string wherever there are
     * $number combinations.  This means that everywhere in the string
     * where there is a $1 the first parameter, messageParams[0], will
     * be inserted.  This was inspired by UNIX shell scripting and the
     * use of $1, $2, etc., to denote command line arguments.
     *
     * @param messageName The name or key where we can find the
     * message.
     * @param messageParams The parameters to use to format the
     * message.
     * @return The formatted message for our log file or the user's.
     * @exception Exception, a generic exception.
     */
    public String getMessage(String messageName,
                             Object[] messageParams)
        throws Exception
    {
        if ( messageName == null )
            throw new Exception ("MessageResources: messageName is null");
        String rawMessage = getString(messageName);
        if ( rawMessage == null )
            throw new Exception ( "MessageResources: rawMessage is null" );
        StringBuffer messageOut = new StringBuffer();
        StringTokenizer tokens = new StringTokenizer(rawMessage, "$");
        String nextToken = (String) tokens.nextElement();
        messageOut.append(nextToken);
        while(tokens.hasMoreTokens())
        {
            nextToken = (String) tokens.nextElement();
            int i = 0;
            for(i = 0; Character.isDigit(nextToken.charAt(i)); i++)
            {
            }
            int parameterIndex = Integer.valueOf(nextToken.substring(0, i))
                .intValue() - 1;
            messageOut.append(messageParams[parameterIndex].toString());
            messageOut.append(nextToken.substring(i, nextToken.length()));
        }
        return messageOut.toString();
    }
}
