package org.apache.turbine.services.resources;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.IOException;
import java.util.*;

// Turbine stuff.
import org.apache.java.util.*;
import org.apache.turbine.services.*;

/**
 * TODO: Description for this class.
 *
 * @author <a href="mailto:jm@mediaphil.de">Jonas Maurus</a>
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @version $Id: TurbineResourceService.java,v 1.3 2000/08/30 20:57:08 gonzalo Exp $
 */
public class TurbineResourceService
    extends TurbineBaseService
{
    /** The name of this service. */
    public static final String SERVICE_NAME = "TurbineResourceService";

    /** Location of the TurbineResources class. */
    private static TurbineResources turbineResources = null;

    /**
     * This method is called when the Service is initialized.
     */
    public void init()
    {
        turbineResources = new TurbineResources();
        setInit(true);
    }

    /**
     * Returns and instance of the TurbineResources object.
     *
     * @return A TurbineResources.
     */
    public TurbineResources getTurbineResources()
    {
        return turbineResources;
    }

    /**
     * Set the file name of the properties file.  This
     * <strong>MUST</strong> be called before this class is used with
     * the default no argument constructor.
     *
     * @param propertiesFileName The file name.
     * @exception IOException, if there was an I/O problem.
     */
    public static void setPropertiesFileName(String propertiesFileName)
        throws IOException
    {
        if ( propertiesFileName == null ||
             propertiesFileName.length() == 0 )
            throw new IOException ( "TurbineResourceService: properties file is null");
        TurbineResources.setPropertiesFileName(propertiesFileName);

        cacheInstance();
    }

    /**
     * Set the properties member of the Configuration instance.  This
     * <strong>MUST</strong> be called before this class is used with
     * the default no argument constructor.
     *
     * @param properties The Properites object read in by another
     * source.
     * @exception IOException, if there was an I/O problem.
     */
    public static void setProperties(Properties properties)
        throws IOException
    {
        if (properties != null && properties.isEmpty())
            throw new IOException ( "TurbineResourceService: properties object is null");
        TurbineResources.setProperties(properties);

        cacheInstance();
    }

    /**
     * This is here so that we grab an instance of the
     * TurbineResourceService and cache that within the
     * TurbineServices class, so that the instance of myself is never
     * GC'd.
     */
    private static void cacheInstance()
    {
        TurbineServices.getInstance()
            .getService(TurbineResourceService.SERVICE_NAME);
    }
}
