package org.apache.turbine.services.schedule;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.peer.*;
import com.workingdogs.village.*;
import org.apache.turbine.util.db.map.*;

/**
 * Peer class for JobEntry database access.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: JobEntryPeer.java,v 1.2 2000/08/30 21:46:46 gonzalo Exp $
 */
public class JobEntryPeer extends BasePeer
{
    /** Get the MapBuilder. */
    private static final TurbineMapBuilder mapBuilder =
        (TurbineMapBuilder) getMapBuilder();

    /** Name of the table. */
    private static final String  TABLE_NAME = "Jobentry";

    // Table colums.
    public static final String OID_COLUMN = "JOB_ID";
    public static final String MIN_COLUMN = "MINUTE";
    public static final String HOUR_COLUMN = "HOUR";
    public static final String WEEKDAY_COLUMN = "WEEKDAY";
    public static final String DAY_OF_MONTH_COLUMN = "DAY_OF_MONTH";
    public static final String TASK_COLUMN = "TASK";
    public static final String EMAIL_COLUMN = "EMAIL";

    // Table name + column name.
    public static final String OID = TABLE_NAME + "." + OID_COLUMN;
    public static final String MINUTE = TABLE_NAME + "." + MIN_COLUMN;
    public static final String HOUR = TABLE_NAME + "." + HOUR_COLUMN;
    public static final String WEEKDAY = TABLE_NAME + "." + WEEKDAY_COLUMN;
    public static final String DAY_OF_MONTH = TABLE_NAME + "." + DAY_OF_MONTH_COLUMN;
    public static final String TASK = TABLE_NAME + "." + TASK_COLUMN;
    public static final String EMAIL = TABLE_NAME + "." + EMAIL_COLUMN;


    /**
     * Update an existing Job.
     *
     * @param Criteria The information to update.
     * @exception Exception, a generic exception.
     */
    public static void doUpdate(Criteria criteria)
        throws Exception
    {
        Criteria selectCriteria = new Criteria(2);
        selectCriteria.put( OID, criteria.remove(OID) );
        BasePeer.doUpdate( selectCriteria, criteria );
    }

    /**
     * Called from the SchedulerService init() to batch load Jobs into
     * the queue.
     *
     * @param Criteria The information for the where.
     * @return Vector of JobEntries.
     * @exception Exception, a generic exception.
     */
    public static Vector doSelect(Criteria criteria)
        throws Exception
    {
        criteria.addSelectColumn(OID)
            .addSelectColumn(MINUTE)
            .addSelectColumn(HOUR)
            .addSelectColumn(WEEKDAY)
            .addSelectColumn(DAY_OF_MONTH)
            .addSelectColumn(TASK)
            .addSelectColumn(EMAIL);

        Vector rows = BasePeer.doSelect(criteria);
        Vector results = new Vector();

        // Populate the object(s).
        for ( int i=0; i<rows.size(); i++ )
        {
            Record rec = (Record)rows.elementAt(i);
            int oid =  rec.getValue(1).asInt();
            int min =  rec.getValue(2).asInt();
            int hr  =  rec.getValue(3).asInt();
            int wd  =  rec.getValue(4).asInt();
            int d_m =  rec.getValue(5).asInt();
            String task =  rec.getValue(6).asString();
            String email =  rec.getValue(7).asString();

            JobEntry je = new JobEntry(min, hr, wd, d_m, task);
            je.setId(oid);
            je.setEmail(email);

            je.resetModified();

            results.addElement( je );
        }
        return results;
    }

    /**
     * Retrieve a JobEntry based on its id.
     *
     * @param oid The JobEntry int id.
     * @return A JobEntry.
     * @exception Exception, a generic exception.
     */
    public static JobEntry getJob(int oid)
        throws Exception
    {
        JobEntry je = null;

        Criteria c = new Criteria(8);
        c.add(OID,new Integer(oid));

        Vector results = JobEntryPeer.doSelect(c);

        if ( results != null  )
        {
            je = (JobEntry)results.elementAt(0);
        }
        return je;
    }
}
