package org.apache.turbine.services.webmacro;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK Stuff
import java.io.*;

// Turbine Stuff
import org.apache.turbine.services.Service;
import org.apache.turbine.util.*;

// WebMacro Stuff
import org.webmacro.*;
import org.webmacro.util.*;
import org.webmacro.servlet.*;

/**
 * Implementations of the WebMacroService interface.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: WebMacroService.java,v 1.5 2000/08/31 18:26:46 gonzalo Exp $
 */
public interface WebMacroService
    extends Service
{
    public static final String SERVICE_NAME = "TurbineWebMacroService";
    public static final String WEBMACRO_CONTEXT = "WEBMACRO_CONTEXT";

    /**
     * Process the request and fill in the template with the values
     * you set in the WebContext.
     *
     * @param wc A WebContext.
     * @param filename A String with the filename of the template.
     * @return The process template as a String.
     * @exception Exception, a generic exception.
     */
    public String handleRequest(WebContext wc,
                                String template)
        throws Exception;

    /**
     * Process the request and fill in the template with the values
     * you set in the WebContext.
     *
     * @param wc A WebContext.
     * @param filename A String with the filename of the template.
     * @param out A Writer where we will write the process template as
     * a String.
     * @exception Exception, a generic exception.
     */
    public void handleRequest(WebContext wc,
                              String template,
                              Writer out)
        throws Exception;

    /**
     * Create an empty WebContext object.
     *
     * @return An empty WebContext object.
     */
    public WebContext getContext();

    /**
     * Create a WebContext from the RunData object.  Adds a pointer to
     * the RunData object to the WC so that RunData is available in
     * the templates.
     *
     * @param data The Turbine RunData object.
     * @return A clone of the WebContext needed by WebMacro.
     */
    public WebContext getContext(RunData data);

    /**
     * Return a template from WebMacro.
     *
     * @param filename A String with the name of the template.
     * @return A Template.
     * @exception NotFoundException.
     */
    public Template getTemplate(String template)
        throws NotFoundException;
}
