package org.apache.turbine.services.xmlrpc;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes.
import java.util.Vector;
import java.io.IOException;
import java.net.URL;

// Turbine classes.
import org.apache.turbine.services.*;

// Xmlrpc classes.
import helma.xmlrpc.*;

/**
 * This is a service which will make an xml-rpc call to a remote
 * server.
 *
 * Here's an example of how it would be done.
 *
 * XmlRpcService xs =
 *   (XmlRpcService)TurbineServices.getInstance()
 *   .getService(XmlRpcService.XMLRPC_SERVICE_NAME);
 * Vector vec = new Vector();
 * vec.addElement(new Integer(5));
 * String name = (String)xs.executeRpc(new URL("http://betty.userland.com/RPC2","examples.getStateName",vec);
 *
 * @author <a href="mailto:josh@stonecottage.com">Josh Lucas</a>
 * @version $Id: TurbineXmlRpcService.java,v 1.3 2000/08/31 18:36:47 gonzalo Exp $
 */
public class TurbineXmlRpcService
    extends TurbineBaseService
    implements XmlRpcService
{
    /** The xmlrpc client. */
    private XmlRpcClient client = null;

    /**
     * Constructor.
     *
     * @exception Exception, a generic exception.
     */
    public TurbineXmlRpcService()
        throws Exception
    {
    }

    /**
     * Init is called the first time the service is used.
     */
    public void init()
    {
        if ( getInit() )
        {
            return;
        }

        try
        {
            org.apache.turbine.util.Log.note (XmlRpcService.SERVICE_NAME +
                                              " init()...starting!");
            setInit(true);
            org.apache.turbine.util.Log.note (XmlRpcService.SERVICE_NAME +
                                              " init()...finished!");
        }
        catch (Exception ex)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize " +
                                                XmlRpcService.SERVICE_NAME );
            org.apache.turbine.util.Log.error ( ex );
        }
    }

    /**
     * The 'meat' of the class.  Actually we just pass our info to the
     * client.  The return is an Object which you'll need to cast to
     * whatever you are expecting.
     *
     * @param url A URL.
     * @param methodName A String with the method name.
     * @param params A Vector with the parameters.
     * @return An Object.
     * @exception XmlRpcException.
     * @exception IOException.
     */
    public Object executeRpc(URL url,
                             String methodName,
                             Vector params)
        throws XmlRpcException,
               IOException
    {
        try
        {
            XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
            client = new XmlRpcClient ( url );
            return client.execute(methodName, params);
        }
        catch (Exception ex)
        {
            org.apache.turbine.util.Log.error ( "Had error within executeRpc" );
            org.apache.turbine.util.Log.error ( ex );
            return null;
        }
    }
}
