package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes.
import java.io.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.*;

/**
 *  This class is what is used to generate MapBuilders from JDBC
 *  metadata.
 *
 *  <p>This class is used as a task in Ant to generate the map.
 *  Here's an example of it's use:
 *
 *  <ol>
 *  <li> Add the task definition to the init target in the build file.<br>
 *  <code><taskdef name="generatemap" classname="org.apache.turbine.util.db.map.JDBCGenerateMapBuilderTask"/></code>
 *  <li> Add to an appropriate place in the build (don't put a trailing ";" after the packages line!).<br>
 *  <pre>
 *  <target name="makemaps" depends="init">
 *    <generatemap
 *      rootdir="/home/project"
 *      dbUrl="mydatabase"
 *      dbDriver="myvendor.mydriver"
 *      dbPassword="secret"
 *      dbUser="user"
 *      licenseFile="/usr/local/apachelicense.txt"
 *      package="myapp.om.mapbuilder"
 *      extend="MyAppMapBuilder"
 *      prefix="MYAPP_"
 *      suffix="MapBuilder"
 *      mapname="MyAppDatabaseMap"/>
 *  </target>
 *  </pre>
 * </ol>
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:gjritter@codespin.org">Greg Ritter</a>
 * @version $Id: JDBCOMGenerator.java,v 1.3 2000/09/05 14:07:32 gonzalo Exp $
 */
public class JDBCOMGenerator
    extends BaseOMGenerator
{
    private String dbUrl;
    private String dbDriver;
    private String dbPassword="";
    private String dbUser="";

    /**
     * Get the database URL.
     *
     * @return A String.
     */
    public String getDbUrl ()
    {
        return dbUrl;
    }

    /**
     * Get the database driver.
     *
     * @return A String.
     */
    public String getDbDriver ()
    {
        return dbDriver;
    }

    /**
     * Get the password to log in.
     *
     * @return A String.
     */
    public String getDbPassword ()
    {
        return dbPassword;
    }

    /**
     * Get the username for the database.
     *
     * @return A String.
     */
    public String getDbUser ()
    {
        return dbUser;
    }

    /**
     * Set the database URL.
     *
     * @param v A String.
     */
    public void setDbUrl (String v)
    {
        dbUrl = v;
    }

    /**
     * Set the database driver.
     *
     * @param v A String.
     */
    public void setDbDriver (String v)
    {
        dbDriver = v;
    }

    /**
     * Set the password to log in.
     *
     * @param v A String.
     */
    public void setDbPassword (String v)
    {
        dbPassword = v;
    }

    /**
     * Set the username for the database.
     *
     * @param v A String.
     */
    public void setDbUser (String v)
    {
        dbUser = v;
    }

    /**
     * Returns the DatabaseMap created from a MySQL script.
     *
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        // return org.apache.turbine.util.db.map.MySQLSchemaToDatabaseMap.generateMap(getMapname(), new FileReader(getSqlfile()));
        JDBCSchemaToDatabaseMap mapper = new JDBCSchemaToDatabaseMap();
        mapper.setDbUrl (getDbUrl());
        mapper.setDbDriver (getDbDriver());
        mapper.setDbPassword (getDbPassword());
        mapper.setDbUser (getDbUser());
        return mapper.generateMap (getMapname());
    }
}
