package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes.
import java.io.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.DatabaseMap;

/**
 *  This class is what is used to generate MapBuilders from a MySQL
 *  script.
 *
 *  <p>It takes advantage of MySQLSchemaToDatabaseMap to parse the
 *  MySQL schema into memory and represent it as a DatabaseMap.  Then,
 *  this class goes over that DatabaseMap and outputs the MapBuilder
 *  files.
 *
 *  <p>This class is used as a task in Ant to generate the map.
 *  Here's an example of it's use:
 *
 *  <ol>
 *  <li> Add the task definition to the init target in the build file.<br>
 *  <code><taskdef name="generatemap" classname="org.apache.turbine.util.db.map.MySqlGenerateMapBuilderTask"/></code>
 *  <li> Add to an appropriate place in the build (don't put a trailing ";" after the packages line!).<br>
 *  <pre>
 *  <target name="makemaps" depends="init">
 *    <generatemap
 *      rootdir="/home/project"
 *      sqlfile="/home/project/mysql_schema.sql"
 *      licenseFile="/usr/local/apachelicense.txt"
 *      package="myapp.om.mapbuilder"
 *      extend="MyAppMapBuilder"
 *      prefix="MYAPP_"
 *      suffix="MapBuilder"
 *      mapname="MyAppDatabaseMap"/>
 *  </target>
 *  </pre>
 * </ol>
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:gjritter@codespin.org">Greg Ritter</a>
 * @version $Id: MySqlOMGenerator.java,v 1.3 2000/08/31 20:09:37 gonzalo Exp $
 */
public class MySqlOMGenerator
    extends BaseOMGenerator
{
    private String sqlfile;

    /**
     * Get the value of sqlfile.
     *
     * @return A String with the value of sqlfile.
     */
    public String getSqlfile()
    {
        return sqlfile;
    }

    /**
     * Set the value of sqlfile.
     *
     * @param v Value to assign to sqlfile.
     */
    public void setSqlfile(String v)
    {
        this.sqlfile = v;
    }

    /**
     * Returns the DatabaseMap created from an MySQL script.
     *
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        return org.apache.turbine.tools.MySQLSchemaToDatabaseMap.generateMap(getMapname(), new FileReader(getSqlfile()));
    }

    /**
     * For testing purposes only.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main( String[] args )
    {
        MySqlOMGenerator gm = new MySqlOMGenerator();
        gm.setRootdir("/home/regis/project/src");
        gm.setSqlfile(args[0]);
        gm.setLicenseFile(null);
        gm.setPackage("test");
        gm.setExtend("MapBuilder");
        gm.setMapname("TurbineMap");
        gm.setSuffix("MapBuilder");
        gm.execute();
    }
}
