package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java
import java.util.Hashtable;
import java.util.Date;

// Turbine
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.turbine.services.resources.TurbineResourceService;
import org.apache.turbine.util.db.pool.*;
import org.apache.turbine.util.Log;
import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.ColumnMap;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;

// XML
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class is an <b>experimental</b> SAX parser that builds a
 * DatabaseMap from a .xml file.
 *
 * <p>It is experimental in the way that the XML DTD is not finalized.
 * To see how the XML looks like, have a look on TurbineDBSchema under
 * the example directory.
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @author <a href="mailto:rkoenig@chez.com">Regis Koenig </a>
 * @version $Id: SAX2MapParser.java,v 1.2 2000/08/31 20:09:37 gonzalo Exp $
 */
public class SAX2MapParser
    extends DefaultHandler
{
    // Map.
    private DatabaseMap dbMap = null;
    private static final boolean DEBUG = true;

    // Elements.
    private static final String COLUMN = "column";
    private static final String TABLE = "table";
    private static final String DATABASE = "database";

    // Atrributes global.
    private static final String NAME = "name";

    // Attributes database.
    private static final String CONNECTION_POOL = "connectionPool";

    // Attributes column.
    private static final String PRIMARY_KEY = "primaryKey";
    private static final String TYPE = "type";
    private static final String NOTNULL = "notNull";
    private static final String FOREIGN_TABLE_NAME = "foreignPKTable";
    private static final String FOREIGN_COLUMN_NAME = "foreignPKColumn";

    // Attributes column value.
    private static final String PK_AUTOINCREMENT = "autoIncrement";
    private static final String PK_NONE = "none";
    private static final String PK_SEQUENCE = "sequence";
    private static final String PK_IDBROKERTABLE = "idBroker";

    // Types value.
    private static final String TYPE_INTEGER = "integer";
    private static final String TYPE_DATE = "date";
    private static final String TYPE_HASHTABLE = "hashtable";
    private static final String TYPE_STRING = "string";

    // Navigation.
    private String currentTableName = null;


    /* --------------------------------------------------- */
    /*                  SAX Events                         */
    /* --------------------------------------------------- */

    /**
     * Start element.
     *
     * @param uri A String.
     * @param local A String.
     * @param raw A String.
     * @param attrs An Attributes.
     */
    public void startElement(String uri,
                             String local,
                             String raw,
                             Attributes attrs)
        throws SAXException
    {
        try
        {
            if ( raw.equals(DATABASE) )
            {
                processDatabaseTag(attrs);
            }
            else if ( raw.equals(TABLE) )
            {
                processTableTag(attrs);
            }
            else if ( raw.equals(COLUMN) )
            {
                processColumnTag(attrs);
            }
        }
        catch ( Exception e )
        {
            throw new SAXException ( e );
        }
    }

    /* --------------------------------------------------- */
    /*                  Processors                         */
    /* --------------------------------------------------- */

    /**
     * Processes the Database Tag.
     *
     * @param attrs An Attributes.
     * @exception Exception, a generic exception.
     */
    private void processDatabaseTag(Attributes attrs)
        throws Exception
    {
        String name = attrs.getValue(NAME);
        String cp = attrs.getValue(CONNECTION_POOL);
        if ( cp == null )
        {
            cp = "default";
        }

        LOG("Current Database Pool: " + cp );
        dbMap = new DatabaseMap(cp);
    }

    /**
     * Processes the Table Tag.
     *
     * @param attrs An Attributes.
     * @exception Exception, a generic exception.
     */
    private void processTableTag(Attributes attrs)
        throws Exception
    {
        currentTableName = attrs.getValue(NAME);
        dbMap.addTable(currentTableName);
        LOG("\tCurrent Table: " + currentTableName );
    }

    /**
     * Processes the Column Tag.
     *
     * @param attrs An Attributes.
     * @exception Exception, a generic exception.
     */
    private void processColumnTag(Attributes attrs)
        throws Exception
    {
        TableMap tm = dbMap.getTable(currentTableName);

        String primaryKey = attrs.getValue(PRIMARY_KEY);
        String fpkTable = attrs.getValue(FOREIGN_TABLE_NAME);
        String fpkColumn = attrs.getValue(FOREIGN_COLUMN_NAME);
        String notNull = attrs.getValue(NOTNULL);
        String name = attrs.getValue(NAME);
        String type = attrs.getValue(TYPE);
        LOG("\t\tCurrent Column: " + name );
        if ( primaryKey != null && primaryKey.length() > 0 )
        {
            primaryKeyMaker(tm, name, type, primaryKey);
        }
        else if ( fpkTable != null &&
                  fpkTable.length() > 0 &&
                  fpkColumn != null &&
                  fpkColumn.length() > 0)
        {
            foreignPrimaryKeyMaker(tm, name, type, fpkTable, fpkColumn );
        }
        else
        {
            columnMaker ( tm, name, type, notNull );
        }
    }

    /**
     * Adds the appropriate column to a table.  The logic in here
     * could probably be coded much better to avoid duplication, but
     * i'm lazy right now.
     *
     * @param tm A TableMap.
     * @param name A String.
     * @param type A String.
     * @param kind A String.
     */
    private static void primaryKeyMaker(TableMap tm,
                                        String name,
                                        String type,
                                        String kind)
    {
        // Set the PK type.
        if( kind.equals(PK_AUTOINCREMENT) )
        {
            tm.setPrimaryKeyMethod(TableMap.AUTOINCREMENT);
        }
        else if( kind.equals(PK_IDBROKERTABLE) )
        {
            tm.setPrimaryKeyMethod(TableMap.IDBROKERTABLE);
        }
        else if( kind.equals(PK_SEQUENCE) )
        {
            tm.setPrimaryKeyMethod(TableMap.SEQUENCE);
        }
        else
        {
            tm.setPrimaryKeyMethod(TableMap.NONE);
        }

        // Set the primary keys.
        if ( type.equals(TYPE_STRING) )
        {
            tm.addPrimaryKey(name, "");
        }
        else if ( type.equals(TYPE_HASHTABLE) )
        {
            tm.addPrimaryKey(name, new Hashtable(1));
        }
        else if ( type.equals(TYPE_INTEGER) )
        {
            tm.addPrimaryKey(name, new Integer(0));
        }
        else if ( type.equals(TYPE_DATE) )
        {
            tm.addPrimaryKey(name, new Date());
        }
        LOG ("\t\t\tPrimary Key: " + name + " ("+kind+")");
    }

    /**
     * Adds the appropriate column.
     *
     * @param tm A TableMap.
     * @param name A String.
     * @param type A String.
     * @param notNull A String.
     */
    static void columnMaker(TableMap tm,
                            String name,
                            String type,
                            String notNull)
    {
        if ( type.equalsIgnoreCase(TYPE_STRING) ||
             type.indexOf("CHAR") != -1 )
        {
            tm.addColumn(name, "");
        }
        else if ( type.equalsIgnoreCase(TYPE_HASHTABLE) )
        {
            tm.addColumn(name, new Hashtable(1));
        }
        else if ( type.equalsIgnoreCase(TYPE_INTEGER) ||
                  type.indexOf("INTEGER") != -1 ||
                  type.indexOf("NUMBER") != -1 )
        {
            tm.addColumn(name, new Integer(0));
        }
        else if ( type.equals(TYPE_DATE) ||
                  type.indexOf("TIMESTAMP") != -1 ||
                  type.indexOf ("DATETIME") != -1 )
        {
            tm.addColumn(name, new Date());
        }
        else
        {
            tm.addColumn(name, "");
        }
        if( notNull != null && notNull.equalsIgnoreCase("true") )
            tm.getColumn(name).setNotNull(true);
        else
            tm.getColumn(name).setNotNull(false);
        // LOG ("\t\t\tColumn: " + name );
    }

    /**
     * Adds the appropriate foreign key.
     *
     * @param tm A TableMap.
     * @param name A String.
     * @param type A String.
     * @param fTableName A String.
     * @param fColumnName A String.
     */
    private static void foreignPrimaryKeyMaker(TableMap tm,
                                               String name,
                                               String type,
                                               String fTableName,
                                               String fColumnName)
    {
        if ( type.equals(TYPE_STRING) )
        {
            tm.addForeignPrimaryKey(name, "", fTableName, fColumnName);
        }
        else if ( type.equals(TYPE_HASHTABLE) )
        {
            tm.addForeignPrimaryKey(name, new Hashtable(1),
                                    fTableName, fColumnName);
        }
        else if ( type.equals(TYPE_INTEGER) )
        {
            tm.addForeignPrimaryKey(name, new Integer(0),
                                    fTableName, fColumnName);
        }
        else if ( type.equals(TYPE_DATE) )
        {
            tm.addForeignPrimaryKey(name, new Date(),
                                    fTableName, fColumnName);
        }
        LOG ("\t\t\tForeignPrimaryKey: " + name );
    }

    /**
     * Log an event.
     *
     * @param msg A String.
     */
    private static void LOG(String msg)
    {
        if (DEBUG)
            System.out.println ( msg );
        Log.note ( msg );
    }

    /**
     * Get the DatabaseMap object.
     *
     * @return A DatabaseMap.
     */
    DatabaseMap getDatabaseMap()
    {
        return this.dbMap;
    }
}
