package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine stuff.
import org.apache.turbine.util.db.map.DatabaseMap;

// XML
import org.xml.sax.*;

/**
 * Generate a SQL script file for PostgreSQL out from a XML database
 * description.
 *
 * <p>This class is <b>experimental</b> since the XML DTD it takes the
 * database description from is not finalized.  For an example of what
 * a XML description looks like, see the TurbineDBSchema.xml under the
 * example directory.
 *
 * <p>This should be used as an Ant task. Here is an example of what
 * to do:
 *
 * <ol>
 * <li>Add the task definition to the init target in the build file.<br>
 * <code><taskdef name="generatesql" classname="org.apache.turbine.util.db.map.XmlGeneratePostgresSchemaTask"/></code>
 * <li>Add to an appropriate place in the build (don't put a trailing ";" after the packages line!).<br>
 *  <pre>
 *  <target name="makesql" depends="init">
 *    <generatesql
 *      xmlFile="/home/project/myxml_schema.xml"
 *      sqlfile="/home/project/schema.sql""/>
 *  </target>
 *  </pre>
 * </ol>
 *
 * <p>It has also a main() method that takes two command line
 * arguments: the first is the xml filename and the second the output
 * sqlfile.
 *
 * @author <a href="mailto:rkoenig@chez.com">Regis Koenig </a>
 * @version $Id: XmlGeneratePostgresSchemaTask.java,v 1.2 2000/08/31 20:09:37 gonzalo Exp $
 */
public class XmlGeneratePostgresSchemaTask
    extends BaseGeneratePostgresSchemaTask
{
    private static final String
        DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";

    private String xmlFile;

    /**
     * Get the value of sqlfile.
     *
     * @return A String with the value of sqlfile.
     */
    public String getXmlFile()
    {
        return xmlFile;
    }

    /**
     * Set the value of xmlfile.
     *
     * @param v Value to assign to xmlfile.
     */
    public void setXmlFile(String v)
    {
        this.xmlFile = v;
    }

    /**
     * Create the DatabaseMap from the XMl file.
     *
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        SAX2MapParser smp = new SAX2MapParser();
        XMLReader parser = (XMLReader)Class
            .forName(DEFAULT_PARSER_NAME).newInstance();
        parser.setContentHandler(smp);
        parser.setErrorHandler(smp);
        parser.parse( getXmlFile() );
        return smp.getDatabaseMap();
    }

    /**
     * For testing purposes only.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main( String[] args )
    {
        XmlGeneratePostgresSchemaTask gm =
            new XmlGeneratePostgresSchemaTask();
        gm.setXmlFile(args[0]);
        gm.setSqlfile(args[1]);
        gm.execute();
    }
}
