package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import java.lang.*;

// Ant Stuff
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * This class allows you to use Torque from an Ant build file.  Here's
 * an example of its use:
 *
 * <p>
 *1. Add the task definition to the init target in the build file.<br>
 * <taskdef name="torque" classname="org.apache.turbine.tools.torque.TorqueTask"/>
 *
 * <p>
 * 2. Add to an appropriate place in the build.<br>
 * <pre>
 *  <target name="makemaps" depends="init">
 *   <torque
 *    props="config/torque.props"
 *    macros="config/torque.macros"
 *   />
 * </pre>
 *
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:celkins@scardini.com">Christopher Elkins</a>
 * @version $Id: TorqueTask.java,v 1.5 2000/08/31 22:31:40 gonzalo Exp $
 */
public class TorqueTask
    extends Task
{
    /**
     * The Torque properties file.
     */
    private File props;

    /**
     * The Torque macros file.
     */
    private File macros;

    /**
     * Sets the Torque properties file.
     *
     * @param props The Torque properties file.
     */
    public void setProps(File props)
    {
        this.props = props;
    }

    /**
     * Sets the Torque macros file.
     *
     * @param macros The Torque macros file.
     */
    public void setMacros(File macros)
    {
        this.macros = macros;
    }

    /**
     * This is the method executed by Ant.
     *
     * @exception BuildException.
     */
    public void execute()
        throws BuildException
    {
        Torque torque = new Torque(props, macros);
        torque.execute();
    }
}
