package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import java.text.*;

// W3C stuff.
import org.w3c.dom.*;

// XML stuff.
import org.apache.turbine.util.db.map.*;

/**
 * This class takes an XML schema for a database and produces a
 * DatabaseMap that can subsequently be used to produce the Peer
 * object model for a turbine project.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: XMLSchemaToDatabaseMap.java,v 1.5 2000/08/31 22:31:40 gonzalo Exp $
 */
public class XMLSchemaToDatabaseMap
{
    /**
     * Subclass of XMLParser used for parsing the XML database
     * schema.
     */
    Parser parser;

    /** DatabaseMap created from parsing the XML database schema. */
    DatabaseMap databaseMap;

    /** TableMap created by databaseMap. */
    TableMap tableMap;

    /** ColumnMap created by tableMap. */
    ColumnMap columnMap;

    /** Default constructor. */
    public XMLSchemaToDatabaseMap(String url)
    {
        try
        {
            parser = getParser();
            parser.setDocument(url);
        }
        catch (Exception e)
        {
        }
    }

    /**
     * Create DatabaseMap.
     *
     * @param mapName A String with the map name.
     * @return A DatabaseMap.
     */
    public DatabaseMap getDatabaseMap(String mapName)
    {
        databaseMap = new DatabaseMap(mapName);
        parser.parse();

        return databaseMap;
    }

    /**
     * Get the XMLParser that is defined as inner class of this class.
     *
     * @return A Parser.
     */
    public Parser getParser()
    {
        return(new Parser());
    }

    /**
     * Subclass of XMLParser that provides methods for parsing
     * database, table, and column Nodes.
     */
    public class Parser extends XMLParser
    {
        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean databaseParser()
        {
            return(true);
        }

        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean tableParser()
        {
            String tableName;

            tableName = (String) attributes.get("name");

            databaseMap.addTable(tableName);
            tableMap = databaseMap.getTable(tableName);
            return(true);
        }

        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean columnParser()
        {
            String columnName;
            String columnType;
            String foreignKeyTable;
            String foreignKeyColumn;

            columnName = (String) attributes.get("name");
            columnType = (String) attributes.get("type");

            // Put this in a JDBC type to OM map.  I don't like this
            // but I just ripped it out of JDBCShemaToDatabaseMap.

            if (columnType.equals("BIGINT") ||
                columnType.equals("INTEGER") ||
                columnType.equals("SMALLINT") ||
                columnType.equals("TINYINT"))
            {
                tableMap.addColumn(columnName, new Integer(0));
            }

            else if (columnType.equals("CHAR") ||
                     columnType.equals("LONGVARCHAR") ||
                     columnType.equals("VARCHAR"))
            {
                tableMap.addColumn(columnName, "");
            }

            else if (columnType.equals("DATE") ||
                     columnType.equals("TIME") ||
                     columnType.equals("TIMESTAMP"))
            {
                tableMap.addColumn(columnName, new java.util.Date());
            }
            else
            {
                tableMap.addColumn(columnName, "");
            }

            columnMap = tableMap.getColumn(columnName);

            // Primary keys.
            if (attributes.containsKey("primaryKey"))
            {
                columnMap.setPrimaryKey(true);
            }

            // Foreign keys.
            if (attributes.containsKey("foreignKeyTable"))
            {
                foreignKeyTable = (String) attributes.get("foreignKeyTable");
                foreignKeyColumn = (String) attributes.get("foreignKeyColumn");

                columnMap.setForeignKey(foreignKeyTable, foreignKeyColumn);
            }

            return(true);
        }
    }

    /**
     * Main function.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main(String[] args)
    {
        XMLSchemaToDatabaseMap mapper = new XMLSchemaToDatabaseMap(args[0]);
        mapper.getDatabaseMap(args[1]);
    }
}
