package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import java.lang.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.ColumnMap;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;

// Ant Stuff
// import org.apache.tools.ant.BuildException;
// import org.apache.tools.ant.Task;

/**
 * This class generates a Peer-based object model from an XML database
 * schema.  It does so by first creating a DatabaseMap from the XML
 * database schema and subsequently uses the DatabaseMap to generate
 * the required classes for a Peer-based object model for a Turbine
 * project.
 *
 * I just combined this with the BaseOMGenerator.
 *
 * This class is what is used to generate MapBuilders from a
 * DatabaseMap.
 *
 * <p>It should be extended in order to get the DatabaseMap object
 * which will be used for generation.
 *
 * <p>One has to implement the <code>getDatabaseMap()</code> method in
 * order to give the code to the generator the right databasemap.
 * Then, this class goes over that DatabaseMap and outputs the
 * MapBuilder files.
 *
 * <p>The descendants of this class are used as a task in Ant to
 * generate the map.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @author <a href="mailto:jon@collab.net">Jon S. Stevens</a>
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @author <a href="mailto:gjritter@codespin.org">Greg Ritter</a>
 * @version $Id: XMLSchemaToObjectModel.java,v 1.10 2000/09/03 01:50:24 celkins Exp $
 */
public class XMLSchemaToObjectModel
{
    /** The database map. */
    private DatabaseMap dbMap = null;

    /** Used to put data into the map. */
    private static String string = new String("");

    /** Used to put data into the map. */
    private static Integer integer = new Integer(0);

    /** Used to put data into the map. */
    private static java.util.Date date = new Date();

    // Properties set in the XML file.

    /** The directory where the java files will be created. */
    private static String rootdir = "";

    /** The SQL file to use. */
    private static String sqlfile = "";

    /** The package name. */
    private static String packageName = "";

    /** The class extended. */
    private static String extend = "";

    /** The class prefix used. */
    private static String prefix = "";

    /** The class suffix used. */
    private static String suffix = "";

    /** The Mapname. */
    private static String mapname = "";

    /** The License header. */
    private static String licenseFile = "";

    /** Torque properties (torque.props). */
    protected Properties props;

    /** Torque macros (torque.macros). */
    protected Properties macros;

    /** Name of XML database schema to use. */
    protected String xmlSchema;

    /** Name of SQL schema to use. */
    protected String sqlSchema;


    /**
     * Default constructor.
     */
    public XMLSchemaToObjectModel()
    {
        super();
    }

    /**
     * Method invoked to produce the object model for the given XML
     * database schema.
     */
    public void execute()
    {
        setProperties();

        try
        {
            dbMap = getDatabaseMap();
            createDatabaseMapClasses(dbMap);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * TODO: document.
     */
    public void executeSQL()
    {
        setProperties();

        try
        {
            dbMap = SQLToDatabaseMap.generateMap(mapname,
                                                 new FileReader(sqlSchema));
            createDatabaseMapClasses(dbMap);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Set properties for the generation of the OM classes.
     */
    public void setProperties()
    {
        xmlSchema = props.getProperty("xmlSchema");
        sqlSchema = props.getProperty("sqlSchema");
        setRootdir(props.getProperty("outputDir") + "/");
        setLicenseFile(null);
        setExtend(props.getProperty("extend"));
        setMapname(props.getProperty("mapname"));
        setSuffix(props.getProperty("suffix"));
        setPackage(props.getProperty("package"));
    }

    /**
     * Set the torque properties file.
     *
     * @param props A Properties.
     */
    public void setProperties(Properties props)
    {
        this.props = props;
    }

    /**
     * Set the torque macros file.
     *
     * @param macros A Properties.
     */
    public void setMacros(Properties macros)
    {
        this.macros = macros;
    }

    /**
     * Create the DatabaseMap from the XML file.
     *
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        XMLSchemaToDatabaseMap mapper =
            new XMLSchemaToDatabaseMap(xmlSchema);
        return mapper.getDatabaseMap(getMapname());
    }

    /**
     * For testing purposes only.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main(String[] args)
    {
        XMLSchemaToObjectModel mapper = new XMLSchemaToObjectModel();
    }

    /*

    //
    // This method should be implemented to provide the code generator
    // with the right DataBaseMap.
    // @return A DatabaseMap.
    //
    public abstract DatabaseMap getDatabaseMap()
        throws Exception;

    */

    /**
     * Get the value of mapname.
     *
     * @return A String with the value of mapname.
     */
    public String getMapname()
    {
        return mapname;
    }

    /**
     * Set the value of mapname.
     *
     * @param v Value to assign to mapname.
     */
    public void setMapname(String v)
    {
        this.mapname = v;
    }

    /**
     * Get the value of suffix.
     *
     * @return A String with the value of suffix.
     */
    public String getSuffix()
    {
        return suffix;
    }

    /**
     * Set the value of suffix.
     *
     * @param v Value to assign to suffix.
     */
    public void setSuffix(String v)
    {
        this.suffix = v;
    }

    /**
     * Get the value of prefix.
     *
     * @return A String with the value of prefix.
     */
    public String getPrefix()
    {
        return prefix;
    }

    /**
     * Set the value of prefix.
     *
     * @param v Value to assign to prefix.
     */
    public void setPrefix(String v)
    {
        this.prefix = v;
    }

    /**
     * Get the value of extends.
     *
     * @return A String with the value of extends.
     */
    public String getExtend()
    {
        String extend = (this.extend != null ? this.extend.trim() : "");
        if (extend.equals(""))
        {
            return " implements MapBuilder";
        }
        else
        {
            return " extends " + extend;
        }
    }

    /**
     * Set the value of extends.
     *
     * @param v Value to assign to extends.
     */
    public void setExtend(String v)
    {
        this.extend = v;
    }

    /**
     * Get the value of package.
     *
     * @return A String with the value of package.
     */
    public String getPackage()
    {
        return packageName;
    }

    /**
     * Set the value of package.
     *
     * @param v Value to assign to package.
     */
    public void setPackage(String v)
    {
        this.packageName = v;
    }

    /**
     * Get the value of rootdir.
     *
     * @return A String with the value of rootdir.
     */
    public String getRootdir()
    {
        return rootdir;
    }

    /**
     * Set the value of rootdir (which should supply a trailing
     * slash).
     *
     * @param v Value to assign to rootdir.
     */
    public void setRootdir(String v)
    {
        this.rootdir = v;
    }

    /**
     * Get the value of licenseFile.
     *
     * @return A String with the value of licenseFile.
     */
    public String getLicenseFile()
    {
        return licenseFile;
    }

    /**
     * Set the value of licenseFile.
     *
     * @param v Value to assign to licenseFile.
     */
    public void setLicenseFile(String v)
    {
        this.licenseFile = v;
    }

    /**
     * Creates the database map classes.  Not thread safe at all.
     *
     * @param dm A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public void createDatabaseMapClasses(DatabaseMap dm)
        throws Exception
    {
        TableMap[] tMaps = dbMap.getTables();
        System.out.println("createData " + tMaps.length);
        for (int i = 0; i < tMaps.length; i++)
        {
            TableMap tmap = tMaps[i];
            String pth = rootdir + getPackageAsPath();
            System.out.println("Parsed path..." + pth);

            // Need to parse apart path and make each directory
            // recursively.

            // Make top dir.
            new File (pth).mkdirs();

            // Make map dir.
            new File (pth + "map").mkdirs();

            // Make peer dir.
            new File (pth + "peer").mkdirs();

            writeMaps(tmap);
            writePeers(tmap);
            writeDOs(tmap);
        }
    }

    /**
     * Give me just the column name out of a TABLE.COLUMN type of
     * string.
     *
     * @param fullname A String with the full column name.
     * @return A String with the shortened column name.
     */
    private String getColumnName(String fullname)
    {
        return fullname.substring(fullname.indexOf(".") + 1);
    }

    /**
     * Returns the type of object obj.  If it doesn't know what it is,
     * then it makes it "string".
     *
     * @param obj An Object.
     * @return A String with the type of the Object.
     */
    private String getType(Object obj)
    {
        String type = "string";
        if (obj instanceof String)
        {
            type = "string";
        }
        else if (obj instanceof Integer)
        {
            type = "integer";
        }
        else if (obj instanceof Date)
        {
            type = "date";
        }
        return type;
    }

    /**
     * Returns the Village type of object obj.  If it doesn't know
     * what it is, then it makes it "string".
     *
     * @param obj An Object.
     * @return A String with the Village type of the Object.
     */
    private String getVillageType(Object obj)
    {
        String type = "string";
        if (obj instanceof String)
        {
            type = "asString()";
        }
        else if (obj instanceof Integer)
        {
            type = "asInt()";
        }
        else if (obj instanceof Date)
        {
            type = "asDate()";
        }
        return type;
    }

    /**
     * Returns the Java type of object obj.  If it doesn't know what
     * it is, then it makes it "string".
     *
     * @param obj An Object.
     * @return A String with the Java type of the Object.
     */
    private String getJavaType(Object obj)
    {
        String type = "string";
        if (obj instanceof String)
        {
            type = "String";
        }
        else if (obj instanceof Integer)
        {
            type = "int";
        }
        else if (obj instanceof Date)
        {
            type = "Date";
        }
        return type;
    }

    /**
     * Tell me if a String has PREFIX.
     *
     * @param data A String.
     * @return True if the string as the prefix.
     */
    private boolean hasPrefix (String data)
    {
        return (data.indexOf(getPrefix()) != -1);
    }

    /**
     * Removes the PREFIX from a String.
     *
     * @param data A String.
     * @return A new String with the prefix removed.
     */
    private String removePrefix (String data)
    {
        return data.substring(getPrefix().length());
    }

    /**
     * Removes the PREFIX and underscores, and makes first letters
     * uppercase.
     *
     * SCARAB_FOO_BAR becomes FooBar
     *
     * @param data A String.
     * @return A new String with the said processing applied.
     */
    private String removeUnderScores (String data)
    {
        String tmp = null;
        StringBuffer out = new StringBuffer();
        if (hasPrefix(data))
        {
            tmp = removePrefix(data);
        }
        else
        {
            tmp = data;
        }

        StringTokenizer st = new StringTokenizer(tmp, "_");
        while (st.hasMoreTokens())
        {
            String element = (String) st.nextElement();
            out.append (firstLetterCaps(element));
        }
        return out.toString();
    }

    /**
     * Makes the first letter caps and the rest lowercase.
     *
     * @param data A String.
     * @return A new String with the said processing applied.
     */
    private String firstLetterCaps (String data)
    {
        String firstLetter = data.substring(0, 1).toUpperCase();
        String restLetters = data.substring(1).toLowerCase();
        return firstLetter + restLetters;
    }

    /**
     * Turn the package name into an OS-specific directory name.
     *
     * @return The package name as a path name.
     */
    private String getPackageAsPath()
    {
        return (getPackage().replace('.', File.separator.charAt(0)) +
                File.separator);
    }

    /**
     * Write all the classes for Databasemaps.
     *
     * @param tmap A TableMap.
     */
    private void writeMaps(TableMap tmap)
    {
        PrintWriter pw = null;
        try
        {
            String realTableName = tmap.getName();
            String shortTableName = removeUnderScores(realTableName);
            String className = shortTableName + suffix;

            String fullpath = rootdir + getPackageAsPath() + "map" +
                File.separator + className + ".java";

            System.out.println ("Generating " + fullpath + "...");

            pw = new PrintWriter(new FileWriter(fullpath),
                                      true);
            pw.println("package " + getPackage() + ".map;");
            pw.println("");
            writeLicense(pw);
            pw.println("");
            pw.println("// JDK classes");
            pw.println("import java.util.Date;");
            pw.println("");
            pw.println("// Turbine classes");
            pw.println("import org.apache.turbine.util.db.map.MapBuilder;");
            pw.println("import org.apache.turbine.util.db.map.DatabaseMap;");
            pw.println("import org.apache.turbine.util.db.map.TableMap;");
            pw.println("import org.apache.turbine.util.db.pool.DBBroker;");
            pw.println("");
            pw.println (
                             "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                             new Date().toString() + " */");
            pw.println("public class " + className +
                            getExtend());
            pw.println("{");
            pw.println("    /** the name of this class */");
            pw.println(
                            "    public static final String CLASS_NAME = \"" +
                            getPackage() + ".map." + className + "\";");
            pw.println("");
            pw.println("    /** " + realTableName + " */");
            pw.println("    public static String getTable( )");
            pw.println("    {");
            pw.println("        return \"" +
                            realTableName + "\";");
            pw.println("    }");
            pw.println("");

            ColumnMap[] columns = tmap.getColumns();
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                pw.println("    /** " + realTableName + "." +
                                realColumnName + " */");
                pw.println("    public static String get" +
                                shortTableName + "_" + shortColumnName + "()");
                pw.println("    {");
                pw.println("        return getTable() + \"." +
                                realColumnName + "\";");
                pw.println("    }");
                pw.println("");
            }

            // If we are implementing MapBuilder ourselves we need to
            // add the following implementations.  Otherwise we extend
            // a superclass which we assume is implementing these...
            if (getExtend().indexOf ("implements") != -1)
            {
                pw.println("    /**  the database map  */");
                pw.println("    private DatabaseMap dbMap = null;");
                pw.println("");
                pw.println("    /**");
                pw.println("        tells us if this DatabaseMapBuilder is built so that we don't have");
                pw.println("        to re-build it every time");
                pw.println("    */");
                pw.println("    public boolean isBuilt()");
                pw.println("    {");
                pw.println("        if ( dbMap != null )");
                pw.println("            return true;");
                pw.println("        return false;");
                pw.println("    }");
                pw.println("");
                pw.println("    /**  gets the databasemap this map builder built.  */");
                pw.println("    public DatabaseMap getDatabaseMap()");
                pw.println("    {");
                pw.println("        return this.dbMap;");
                pw.println("    }");
            }

            // Top of the doBuild() method.
            pw.println("    /** the doBuild() method builds the DatabaseMap */");
            pw.println("    public void doBuild ( ) throws Exception");
            pw.println("    {");
            pw.println("        String string = new String(\"\");");
            pw.println("        Integer integer = new Integer(0);");
            pw.println("        java.util.Date date = new Date();");

            pw.println("");
            pw.println("        dbMap = DBBroker.getInstance().getDatabaseMap(DBBroker.DEFAULT);");
            pw.println("        dbMap.addTable(getTable());");
            pw.println("        TableMap tMap = dbMap.getTable(getTable());");
            pw.println("");

            // Generate relationships.
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String type = getType(cmap.getType());
                String realColumnName = cmap.getColumnName();
                String shortColumnName = removeUnderScores(realColumnName);
                String shortRelatedTableName =
                    removeUnderScores(cmap.getRelatedTableName());
                String shortRelatedColumnName = removeUnderScores(
                                                                  getColumnName(cmap.getRelatedColumnName()));
                if (cmap.isForeignKey() && cmap.isPrimaryKey())
                {
                    pw.println(
                                    "        tMap.addForeignPrimaryKey ( get" +
                                    shortTableName + "_" + shortColumnName +
                                    "(), " + type + ",");
                    pw.println("            " +
                                    shortRelatedTableName + getSuffix() + ".getTable(),");
                    pw.println("            " +
                                    shortRelatedTableName + getSuffix() +
                                    ".get" + shortRelatedTableName + "_" +
                                    shortRelatedColumnName + "() );");
                }
                else if (cmap.isForeignKey())
                {
                    pw.println( "        tMap.addForeignKey ( get" +
                                     shortTableName + "_" + shortColumnName +
                                     "(), " + type + ",");
                    pw.println("            " +
                                    shortRelatedTableName + getSuffix() + ".getTable(),");
                    pw.println("            " +
                                    shortRelatedTableName + getSuffix() +
                                    ".get" + shortRelatedTableName + "_" +
                                    shortRelatedColumnName + "() );");
                }
                else if (cmap.isPrimaryKey())
                {
                    pw.println( "        tMap.addPrimaryKey ( get" +
                                     shortTableName + "_" + shortColumnName +
                                     "(), " + type + ");");
                }
                else
                {
                    pw.println("        tMap.addColumn ( get" +
                                    shortTableName + "_" + shortColumnName +
                                    "(), " + type + ");");
                }

                pw.println("");
            }
            pw.println("    }");
            pw.println("");

            pw.println("}");
        }
        catch (Exception e)
        {
        }
        finally { if (pw != null)
            pw.close();
        } }

    /**
     * Write the peers
     *
     * @param tmap A TableMap.
     * @exception Exception, a generic exception.
     */
    private void writePeers(TableMap tmap)
        throws Exception
    {
        // Includes  Suffix  - TURBINE_FOOBAR
        String realTableName = tmap.getName();

        // FooBar
        String tableClassname = removeUnderScores(realTableName);

        // FooBarPeer
        String peerclassName = tableClassname + "Peer";

        // FooBarMapBuilder
        String mapclassName = tableClassname + "MapBuilder";

        // org.apache.turbine.peer
        String subPackage = getPackage() + ".peer";

        String primaryKey = "?";
        PrintWriter pw = null;

        String fullpath = (rootdir +
                           getPackageAsPath() +
                           "peer/" +
                           peerclassName +
                           ".java");
        try
        {
            pw = new PrintWriter(new FileWriter(fullpath), true);
            pw.println("package " + subPackage + ";");
            pw.println("");
            writeLicense(pw);
            pw.println("");
            pw.println("import com.workingdogs.village.*;");
            pw.println("import org.apache.turbine.om.peer.*;");
            pw.println("import org.apache.turbine.util.*;");
            pw.println("import org.apache.turbine.util.db.*;");
            pw.println("import org.apache.turbine.util.db.map.*;");
            pw.println("import java.util.*;");
            // Import the package with the DO.
            pw.println("import " + getPackage() + ".map.*;");
            // Import the package with the DO.
            pw.println("import " + getPackage() + ".*;");
            pw.println("");
            pw.println (
                             "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                             new Date().toString() + " */");
            pw.println("public class " + peerclassName + " extends BasePeer");
            pw.println("{");
            pw.println("    /** the mapbuilder for this class */");
            pw.println("    private static final " +
                            mapclassName + " mapBuilder = (" +
                            mapclassName + ")getMapBuilder("+
                            mapclassName + ".CLASS_NAME);");
            pw.println("");
            pw.println("    /** the table name for this class */");
            pw.println("    public static final String _TABLE_NAME = mapBuilder.getTable();");
            pw.println("");

            // Make the static field accessors.
            ColumnMap[] columns = tmap.getColumns();
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                if (cmap.isPrimaryKey())
                {
                    primaryKey = realColumnName;
                }
                pw.println("    /** the column name for the " +
                                realColumnName + " id field */");
                pw.println("    public static final String " +
                                realColumnName + " = mapBuilder.get" +
                                tableClassname + "_" + shortColumnName + "();");

            }
            pw.println("");

            // Write the doInsert method.
            pw.println("    /** Method to do inserts */ ");
            pw.println("    public static Object doInsert( Criteria criteria ) throws Exception");
            pw.println("    {");
            // pw.println( "        criteria.setMapName (_TABLE_NAME);");
            pw.println("        return BasePeer.doInsert( criteria );");
            pw.println("    }");
            pw.println("");

            // Write the doSelect method.
            pw.println("    /** Method to do selects */ ");
            pw.println("    public static Vector doSelect( Criteria criteria ) throws Exception");
            pw.println("    {");
            // pw.println( "        criteria.setMapName (_TABLE_NAME);");
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                pw.println("        criteria.addSelectColumn( " +
                                realColumnName + " );");
            }
            pw.println("");
            pw.println("        // BasePeer returns a Vector of Value (Village) arrays.  The array");
            pw.println("        // order follows the order columns were placed in the Select clause.");
            pw.println("        Vector rows = BasePeer.doSelect(criteria);");
            pw.println("        Vector results = new Vector();");
            pw.println("");

            pw.println("        // populate the object(s)");
            pw.println("        for ( int i=0; i<rows.size(); i++ )");
            pw.println("        {");
            pw.println("           " +tableClassname + " obj = new "+
                            tableClassname + "();");
            pw.println("            Record row = (Record)rows.elementAt(i);");

            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                // String Fieldname = firstLetterCaps( getColumnName(cmap.getColumnName()) );
                String Fieldname = removeUnderScores (
                                                      getColumnName(cmap.getColumnName()));
                String type = getVillageType(cmap.getType());
                pw.println("            obj.set"+Fieldname + "( row.getValue("+
                                (j + 1) + ")."+type + " );");
            }

            pw.println("            results.addElement( obj );");
            pw.println("         }");
            pw.println("         return results;");
            pw.println("    }");

            pw.println("");

            // Write the doUpdate method.
            pw.println("    /**");
            pw.println("     * @param Criteria object containing data that is used to create the UPDATE statement.");
            pw.println("     */");
            pw.println("     public static void doUpdate(Criteria criteria) throws Exception");
            pw.println("     {");
            // pw.println( "         criteria.setMapName (_TABLE_NAME);");
            pw.println("         Criteria selectCriteria = new Criteria(2);");
            // pw.println( "         selectCriteria.setMapName (_TABLE_NAME);" );
            pw.println("         selectCriteria.put( " +
                            primaryKey + ", criteria.remove("+primaryKey + ") );");
            pw.println("         BasePeer.doUpdate( selectCriteria, criteria );");
            pw.println("     }");
            pw.println("");

            // Write the doDelete method.
            pw.println("    /**");
            pw.println("     * @param Criteria object containing data that is used DELETE from database.");
            pw.println("     */");
            pw.println("     public static void doDelete(Criteria criteria) throws Exception");
            pw.println("     {");
            // pw.println( "         criteria.setMapName (_TABLE_NAME);");
            pw.println("         BasePeer.doDelete ( criteria );");
            pw.println("     }");
            pw.println("");


            // Write the doInsert(DO) method.
            pw.println("    /** Method to do inserts */");
            pw.println("    public static Object doInsert( " +
                            tableClassname + " obj ) throws Exception");
            pw.println("    {");
            pw.println("        return " + peerclassName + ".doInsert(buildCriteria(obj));");
            pw.println("    }");
            pw.println("");

            // Write the buildCriteria method.
            pw.println("    /** Build a Criteria object from the data object for this peer */");
            pw.println("    public static Criteria buildCriteria( " +
                            tableClassname + " obj )");
            pw.println("    {");
            pw.println("        Criteria criteria = new Criteria();");
            for (int j = 0; j < columns.length; j++)
            {
                ColumnMap cmap = columns[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String shortColumnName = removeUnderScores(realColumnName);
                // This only supports integer primary keys right now.
                if (cmap.isPrimaryKey())
                {
                    pw.println("        if( obj.get" +
                                    shortColumnName + "() > 0 )");
                    pw.print("    ");
                }
                pw.println("        criteria.add( " +
                                peerclassName + "." + realColumnName +
                                ", obj.get" + shortColumnName + "() );");
            }
            pw.println("        return criteria;");
            pw.println("    }");
            pw.println("");


            pw.println("");

            // Write the doUpdate(DO) method.
            pw.println("    /**");
            pw.println("     * @param obj the data object to update in the database.");
            pw.println("     */");
            pw.println( "    public static void doUpdate(" +
                             tableClassname + " obj) throws Exception");
            pw.println("    {");
            pw.println("        " + peerclassName + ".doUpdate(buildCriteria(obj));");
            pw.println("    }");

            // Write the doDelete(DO) method.
            pw.println("    /**");
            pw.println("     * @param obj the data object to delete in the database.");
            pw.println("     */");
            pw.println( "    public static void doDelete(" +
                             tableClassname + " obj) throws Exception");
            pw.println("    {");
            pw.println("        " + peerclassName + ".doDelete(buildCriteria(obj));");
            pw.println("    }");

            pw.println("");
            pw.println("}");
        }
        catch (Exception e)
        {
        }
        finally
        {
            if (pw != null)
                pw.close();
        }
    }

    /**
     * Write the DataObjects.
     *
     * @param tmap A TableMap.
     */
    private void writeDOs(TableMap tmap)
    {
        // Includes  Suffix  - TURBINE_FOOBAR
        String realTableName = tmap.getName();

        // FooBar
        String tableClassname = removeUnderScores(realTableName);

        // org.apache.turbine
        String subPackage = getPackage();

        PrintWriter pw = null;

        String fullpath = (rootdir +
                           getPackageAsPath() +
                           tableClassname +
                           ".java");
        try
        {
            pw = new PrintWriter(new FileWriter(fullpath), true);
            pw.println("package " + subPackage + ";");
            pw.println("");
            writeLicense(pw);
            pw.println("");
            pw.println("// Turbine");
            pw.println("import org.apache.turbine.om.BaseObject;");
            pw.println("import java.util.*;");
            pw.println("");
            pw.println (
                             "/** This class was autogenerated by XMLSchemaToObjectModel on: " +
                             new Date().toString() + " */");
            pw.println("public class " + tableClassname + " extends BaseObject");
            pw.println("{");

            // Make the fields.
            ColumnMap[] col1 = tmap.getColumns();
            for (int j = 0; j < col1.length; j++)
            {
                ColumnMap cmap = col1[j];
                String type = getJavaType(cmap.getType());

                String realColumnName = getColumnName(cmap.getColumnName());
                String field =
                    removeUnderScores(realColumnName).toLowerCase();

                pw.println("    /** the value for the " + field +
                                " field */");
                pw.println("    private " + type + " " + field + ";");

            }
            pw.println("");

            // Make the accessors.
            ColumnMap[] col2 = tmap.getColumns();
            for (int j = 0; j < col2.length; j++)
            {
                ColumnMap cmap = col2[j];
                String realColumnName = getColumnName(cmap.getColumnName());
                String field = removeUnderScores(realColumnName);
                String type = getJavaType(cmap.getType());
                String fieldlower = field.toLowerCase();

                // Get.
                pw.println("    /**");
                pw.println("     * Get the " + fieldlower);
                pw.println("     * @return " + type);
                pw.println("     */");
                pw.println("     public " + type + " get" +
                                field + "()");
                pw.println("     {");
                pw.println("          return " + fieldlower + ";");
                pw.println("     }");

                pw.println("");

                // Set.
                pw.println("    /**");
                pw.println("     * Set the value of " + fieldlower);
                pw.println("     * @param " + type);
                pw.println("     */");
                pw.println("     public void set" + field + "(" +
                                type + " v )");
                pw.println("     {");
                pw.println("          this." + fieldlower + " = v;");
                pw.println("     }");

                pw.println("");

            }

            pw.println("}");

        }
        catch (Exception e)
        {
        }
        finally
        {
            if (pw != null)
                pw.close();
        }
    }

    /**
     * Write the license header.
     *
     * @param pw A PrintWriter.
     * @exception Exception, a generic exception.
     */
    private void writeLicense(PrintWriter pw)
        throws Exception
    {
        String filename = getLicenseFile();

        if (filename != null && filename.length() > 0)
        {
            BufferedReader br =
                new BufferedReader(new FileReader(getLicenseFile()));

            // The current line.
            String line = null;

            // Loop over every line in the file.
            while ((line = br.readLine()) != null)
            {
                pw.println(line);
            }
        }
        else
        {
            pw.println("");
        }
    }
}
