package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;

// W3C stuff.
import org.w3c.dom.*;

// TODO:
//
// o Foreign keys.
// o Indices.
// o Insertion of default values.
// o Generation/templates of scripts to run the generated SQL.
// o Ant task(s) to integrate with TDK.

/**
 * This class takes an XML database schema and generates the
 * corresponding SQL for the target database in question.  There are a
 * set of templates for each target database system.  This class is
 * called by Torque to generate the SQL for an XML database schema,
 * this class isn't used directly.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: XMLSchemaToSQL.java,v 1.11 2000/08/31 22:31:41 gonzalo Exp $
 */
public class XMLSchemaToSQL
{
    /** Torque properties file (torque.props). */
    protected Properties props;

    /** Torque properties file (torque.macros). */
    protected Properties macros;

    /** Properties for the target database. */
    protected Properties dbProps;

    /** Configuration as specified in torque.props. */
    protected String configDir;

    /** XML database schema as specified in torque.props. */
    protected String xmlSchema;

    /** Output directory as specified in torque.props. */
    protected String outputDir;

    /** Name of the target db as specified in torque.props. */
    protected String db;

    /** Path to target database configuration files. */
    protected String path;

    /** Path to comment templates for target database. */
    protected String commentPath;

    /**
     * Name of the SQL file to produce as specified in
     * torque.props.
     */
    protected String sqlTargetFile;

    /** Drop template for target database. */
    protected String dropTemplate;

    /** Table template for target database. */
    protected String tableTemplate;

    /** Comment template for target database. */
    protected String commentTemplate;

    /** Primary key template for target database. */
    protected String primarykeyTemplate;

    /** Unique template for target database. */
    protected String uniqueTemplate;

    /** Index template for target database. */
    protected String indexTemplate;

    /** PrintWriter used to output the resultant SQL. */
    protected PrintWriter printWriter;

    /**
     * Subclass of XMLParser used to parse/process the XML database
     * schema.
     */
    protected Parser parser;


    /**
     * Default constructor.
     */
    public XMLSchemaToSQL()
    {
    }

    /**
     * Method invoked to produce SQL for target database based on the
     * XML database schema.
     */
    public void execute()
    {
        db = props.getProperty("db");
        xmlSchema = props.getProperty("xmlSchema");
        configDir = props.getProperty("configDir");
        sqlTargetFile = props.getProperty("sqlTargetFile");
        path = configDir + "/db/" + db.toLowerCase() + "/";
        commentPath = configDir + "/comment/";
        outputDir = props.getProperty("outputDir");

        // Get the db.props file for the db specified in torque.props.
        dbProps = Utils.loadProps(path + "db.props");

        dropTemplate = Utils.fileContentsToString(path + "drop");
        tableTemplate = Utils.fileContentsToString(path + "table");
        commentTemplate = Utils.fileContentsToString(path + "comment");

        primarykeyTemplate =
                Utils.fileContentsToString(path + "primarykey").trim();
        uniqueTemplate = Utils.fileContentsToString(path + "unique").trim();
        indexTemplate = Utils.fileContentsToString(path + "index").trim();

        try
        {
            printWriter = new PrintWriter(
                    new FileOutputStream(outputDir + "/" + sqlTargetFile));
            parser = getParser();
            parser.setDocument(xmlSchema);
            parser.parse();
            printWriter.close();
        }
        catch (Exception e)
        {
            System.out.println ("Error: " + e.toString() );
        }
    }

    /**
     * Set torque properties file.
     *
     * @param props A Properties.
     */
    public void setProperties(Properties props)
    {
        this.props = props;
    }

    /**
     * Set torque macros files.
     *
     * @param macros A Properties.
     */
    public void setMacros(Properties macros)
    {
        this.macros = macros;
    }

    /**
     * Get our subclass of XMLParser used to parse/process the XML
     * database schema.
     *
     * @return a Parser.
     */
    public Parser getParser()
    {
        return(new Parser());
    }

    /**
     * Subclass of XMLParser used to parse/process the XML database
     * schema.
     */
    public class Parser
        extends XMLParser
    {
        // This needs to be cleaned up.
        String indent = "    ";
        StringBuffer sql = new StringBuffer();
        StringBuffer table = new StringBuffer();
        StringBuffer columns = new StringBuffer();
        String tableName;
        String DBType;
        boolean tableCreationInProcess = false;

        Hashtable substitutions;
        Hashtable primaryKeySubstitutions;
        Hashtable indexSubstitutions;
        Hashtable uniqueSubstitutions;

        StringBuffer primaryKeys = new StringBuffer();
        StringBuffer indices = new StringBuffer();
        StringBuffer unique = new StringBuffer();

        int chop;

        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean databaseParser()
        {
            // The comments that indicate creation should be
            // configurable in the db section.  Or in a general
            // sections.  So they can be localized without changing
            // this program.
            printWriter.print(
                    Utils.fileContentsToCommentString(commentPath + "database",
                    commentTemplate, macros));
            return(true);
        }

        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean tableParser()
        {
            if (! tableCreationInProcess)
            {
                tableCreationInProcess = true;
            }
            else
            {
                // Make appropriate substitutions:
                // $table => tableName
                // $columns => columns
                substitutions = new Hashtable();
                substitutions.put("table", tableName);

                // Unique.
                if (unique.length() > 0)
                {
                    uniqueSubstitutions = new Hashtable();
                    uniqueSubstitutions.put("columns",
                            Utils.chop(unique.toString(), 1));
                    unique = Utils.stringSubstitution(uniqueTemplate,
                            uniqueSubstitutions);
                    substitutions.put("unique", indent + unique.toString());
                }

                // Indices.
                if (indices.length() > 0)
                {
                    String indexIndent = "";

                    indexSubstitutions = new Hashtable();
                    indexSubstitutions.put("columns",
                            Utils.chop(indices.toString(), 1));
                    indexSubstitutions.put("tableIndex", tableName + "_INDEX");
                    indexSubstitutions.put("table", tableName);
                    indices = Utils.stringSubstitution(indexTemplate,
                            indexSubstitutions);

                    if (dbProps.getProperty( "indexInsideTableDirective").
                            equalsIgnoreCase("true"))
                        indexIndent = indent;

                    if (unique.length() > 0)
                        substitutions.put("index",
                                indexIndent + indices.toString() + ",");
                    else
                        substitutions.put("index",
                                indexIndent + indices.toString());
                }

                // Primary key.
                if (primaryKeys.length() > 0)
                {
                    String primaryKeyIndent = "";

                    primaryKeySubstitutions = new Hashtable();
                    primaryKeySubstitutions.put("columns",
                            Utils.chop(primaryKeys.toString(), 1));
                    primaryKeySubstitutions.put("table", tableName);
                    primaryKeySubstitutions.put("primaryKey",
                            tableName + "_PK");

                    primaryKeys = Utils.stringSubstitution(
                            primarykeyTemplate, primaryKeySubstitutions);

                    if (dbProps.getProperty(
                            "primaryKeyInsideTableDirective").
                            equalsIgnoreCase("true"))
                        primaryKeyIndent = indent;

                    if (unique.length() > 0 || (indices.length() > 0 &&
                            dbProps.getProperty(
                            "indexInsideTableDirective").
                            equalsIgnoreCase("true") && dbProps.getProperty(
                            "primaryKeyInsideTableDirective").
                            equalsIgnoreCase("true")))
                    {
                        substitutions.put("primarykey",
                                primaryKeyIndent +
                                primaryKeys.toString() + ",");
                    }
                    else
                    {
                        substitutions.put("primarykey",
                                primaryKeyIndent + primaryKeys.toString());
                    }
                }

                // Remove the trailing ",\n" from the columns.
                if (primaryKeys.length() > 0 && dbProps.getProperty(
                        "primaryKeyInsideTableDirective").
                        equalsIgnoreCase("true"))
                    chop = 1;
                else
                    chop = 2;

                // Place the columns in the table directive.
                substitutions.put("columns",
                        Utils.chop(columns.toString(), chop));
                table = Utils.stringSubstitution(table.toString(),
                        substitutions);

                // Collapse multiple newlines with the table directive.
                // printWriter.print(Utils.collapseNewlines(table.toString()) +
                //                   "\n");

                sql.append(Utils.collapseNewlines(table.toString()) + "\n");

                table = new StringBuffer();
                columns = new StringBuffer();
                primaryKeys = new StringBuffer();
                indices = new StringBuffer();
                unique = new StringBuffer();
            }

            tableName = (String) attributes.get("name");
            table.append(dropTemplate);
            table.append("\n");
            table.append(tableTemplate);
            table.append("\n");

            return(true);
        }

        /**
         * TODO: document.
         *
         * @return True.
         */
        public boolean columnParser()
        {
            String columnName = (String) attributes.get("name");
            String JDBCType = (String) attributes.get("type");
            String typeSize = dbProps.getProperty(JDBCType);
            String dbAutoIncrement = dbProps.getProperty("AUTOINCREMENT");
            String primaryKey = " ";
            String defaultValue = " ";
            String autoIncrement = " ";
            String nullable = "";

            // Collect a list of columns that are primary keys.
            if (attributes.containsKey("primaryKey"))
            {
                primaryKeys.append(columnName + ",");
            }

            // Collect a list of columns that are indices.
            if (attributes.containsKey("index"))
            {
                indices.append(columnName + ",");
            }

            // Collect a list of columns that are unique.
            if (attributes.containsKey("unique"))
            {
                unique.append(columnName + ",");
            }

            // Build the column up.

            if (attributes.containsKey("null"))
            {
                nullable = dbProps.getProperty("NOTNULL") + " ";
            }

            if (attributes.containsKey("autoIncrement"))
            {
                autoIncrement = dbAutoIncrement + " ";
            }

            if (attributes.containsKey("size"))
            {
                typeSize = typeSize + "(" +
                        (String) attributes.get("size") + ") ";
            }
            else
            {
                typeSize = typeSize + " ";
            }

            if (attributes.containsKey("autoIncrement") &&
                dbProps.getProperty( "removeNotNullWithAutoIncrement")
                .equalsIgnoreCase("true"))
            {
                nullable = "";
                typeSize = "";
            }

            if (attributes.containsKey("default") &&
                dbProps.getProperty( "suppressDefaults")
                .equalsIgnoreCase("false"))
            {
                defaultValue = ("default " +
                                (String) attributes.get("default") +
                                " ");
            }

            // I can get rid of all this padding logic now that I have
            // Utils.collapseSpaces. Fix.

            // The order of the different components of a column are
            // as follows:
            //
            // columnName typeSize [defaultValue N] [autoIncrement] [nullable]

            columns.append(indent +
                           Utils.replace(Utils.collapseSpaces(columnName +
                                                              " " +
                                                              typeSize +
                                                              defaultValue +
                                                              nullable +
                                                              autoIncrement +
                                                              ",\n"),
                                         " ,", ","));

            return(true);
        }

        /**
         * TODO: document.
         *
         * @param string A String.
         */
        private void print(String string)
        {
            System.out.println(string);
        }

        /**
         * TODO: document.
         */
        private void print()
        {
            System.out.println("");
        }

        /**
         * Optional command to execute when parsing is complete.
         */
        public void finalCommand()
        {
            // Complete the processing of the last table in the XML
            // schema file.
            tableParser();
            printWriter.print(sql);
        }
    }

    /**
     * Main function.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main(String[] args)
    {
        XMLSchemaToSQL xmlSchemaToSQL = new XMLSchemaToSQL();
    }
}
