##
##ER/Studio 4.0 SQL Code Generation
## Company :      Collab.Net
## Project :      Scarab
## Author :       Jon S. Stevens + Scarab Project Members
##
## Date Created : Saturday, September 02, 2000 16:08:51
## Target DBMS : Oracle 8
##


CREATE TABLE SCARAB_ACTIVITY(
    ISSUE_ID          INTEGER     NOT NULL,
    ATTRIBUTE_ID      INTEGER     NOT NULL,
    TRANSACTION_ID    INTEGER     NOT NULL,
    OLD_VALUE         VARCHAR(255),
    NEW_VALUE         VARCHAR(255),
    PRIMARY KEY (ISSUE_ID,ATTRIBUTE_ID,TRANSACTION_ID), 
    FOREIGN KEY (ISSUE_ID)
    REFERENCES SCARAB_ISSUE(ISSUE_ID),
    FOREIGN KEY (ATTRIBUTE_ID)
    REFERENCES SCARAB_ATTRIBUTE(ATTRIBUTE_ID),
    FOREIGN KEY (TRANSACTION_ID)
    REFERENCES SCARAB_TRANSACTION(TRANSACTION_ID)
) 
;


CREATE TABLE SCARAB_ATTACHMENT(
    ATTACHMENT_ID           INTEGER     NOT NULL,
    ISSUE_ID                INTEGER     NOT NULL,
    ATTACHMENT_TYPE_ID      INTEGER     NOT NULL,
    ATTACHMENT_NAME         VARCHAR(255)    NOT NULL,
    ATTACHMENT_DATA         BLOB,
    ATTACHMENT_FILE_PATH    VARCHAR(255),
    ATTACHMENT_MIME_TYPE    VARCHAR(25),
    MODIFIED_BY             INTEGER     NOT NULL,
    CREATED_BY              INTEGER     NOT NULL,
    MODIFIED_DATE           DATE             NOT NULL,
    CREATED_DATE            DATE             NOT NULL,
    DELETED                 VARCHAR(1),
    PRIMARY KEY (ATTACHMENT_ID), 
    FOREIGN KEY (ISSUE_ID)
    REFERENCES SCARAB_ISSUE(ISSUE_ID),
    FOREIGN KEY (ATTACHMENT_TYPE_ID)
    REFERENCES SCARAB_ATTACHMENT_TYPE(ATTACHMENT_TYPE_ID)
) 
;


CREATE TABLE SCARAB_ATTACHMENT_TYPE(
    ATTACHMENT_TYPE_ID      INTEGER     NOT NULL,
    ATTACHMENT_TYPE_NAME    INTEGER     NOT NULL,
    PRIMARY KEY (ATTACHMENT_TYPE_ID)
) 
;


CREATE TABLE SCARAB_ATTRIBUTE(
    ATTRIBUTE_ID         INTEGER     NOT NULL,
    STRING_ID            INTEGER     NOT NULL,
    ATTRIBUTE_TYPE_ID    INTEGER     NOT NULL,
    DELETED              VARCHAR(1),
    PRIMARY KEY (ATTRIBUTE_ID), 
    FOREIGN KEY (STRING_ID)
    REFERENCES SCARAB_STRING(STRING_ID),
    FOREIGN KEY (ATTRIBUTE_TYPE_ID)
    REFERENCES SCARAB_ATTRIBUTE_TYPE(ATTRIBUTE_TYPE_ID)
) 
;


CREATE TABLE SCARAB_ATTRIBUTE_CLASS(
    ATTRIBUTE_CLASS_ID      INTEGER     NOT NULL,
    ATTRIBUTE_CLASS_NAME    VARCHAR(255)    NOT NULL,
    ATTRIBUTE_CLASS_DESC    VARCHAR(255)    NOT NULL,
    PRIMARY KEY (ATTRIBUTE_CLASS_ID)
) 
;


CREATE TABLE SCARAB_ATTRIBUTE_OPTION(
    OPTION_ID        INTEGER     NOT NULL,
    ATTRIBUTE_ID     INTEGER     NOT NULL,
    STRING_ID        INTEGER     NOT NULL,
    NUMERIC_VALUE    INTEGER     NOT NULL,
    PRIMARY KEY (OPTION_ID), 
    FOREIGN KEY (ATTRIBUTE_ID)
    REFERENCES SCARAB_ATTRIBUTE(ATTRIBUTE_ID),
    FOREIGN KEY (STRING_ID)
    REFERENCES SCARAB_STRING(STRING_ID)
) 
;


CREATE TABLE SCARAB_ATTRIBUTE_TYPE(
    ATTRIBUTE_TYPE_ID      INTEGER     NOT NULL,
    ATTRIBUTE_CLASS_ID     INTEGER     NOT NULL,
    ATTRIBUTE_TYPE_NAME    VARCHAR(255)    NOT NULL,
    PRIMARY KEY (ATTRIBUTE_TYPE_ID), 
    FOREIGN KEY (ATTRIBUTE_CLASS_ID)
    REFERENCES SCARAB_ATTRIBUTE_CLASS(ATTRIBUTE_CLASS_ID)
) 
;


CREATE TABLE SCARAB_DEPEND(
    OBSERVED_ID       INTEGER     NOT NULL,
    OBSERVER_ID       INTEGER     NOT NULL,
    DEPEND_TYPE_ID    INTEGER     NOT NULL,
    DELETED           VARCHAR(1),
    PRIMARY KEY (OBSERVED_ID,OBSERVER_ID), 
    FOREIGN KEY (OBSERVED_ID)
    REFERENCES SCARAB_ISSUE(ISSUE_ID),
    FOREIGN KEY (OBSERVER_ID)
    REFERENCES SCARAB_ISSUE(ISSUE_ID),
    FOREIGN KEY (DEPEND_TYPE_ID)
    REFERENCES SCARAB_DEPEND_TYPE(DEPEND_TYPE_ID)
) 
;


CREATE TABLE SCARAB_DEPEND_TYPE(
    DEPEND_TYPE_ID      INTEGER     NOT NULL,
    DEPEND_TYPE_NAME    VARCHAR(100)    NOT NULL,
    PRIMARY KEY (DEPEND_TYPE_ID)
) 
;


CREATE TABLE SCARAB_ISSUE(
    ISSUE_ID         INTEGER     NOT NULL,
    MODULE_ID        INTEGER     NOT NULL,
    MODIFIED_BY      INTEGER     NOT NULL,
    CREATED_BY       INTEGER     NOT NULL,
    MODIFIED_DATE    DATE             NOT NULL,
    CREATED_DATE     DATE             NOT NULL,
    DELETED          VARCHAR(1),
    PRIMARY KEY (ISSUE_ID), 
    FOREIGN KEY (MODULE_ID)
    REFERENCES SCARAB_MODULE(MODULE_ID)
) 
;


CREATE TABLE SCARAB_ISSUE_ATTRIBUTE_VALUE(
    ISSUE_ID        INTEGER     NOT NULL,
    ATTRIBUTE_ID    INTEGER     NOT NULL,
    OPTION_ID       INTEGER ,
    VISITOR_ID      INTEGER     NOT NULL,
    VALUE           VARCHAR(255),
    DELETED         VARCHAR(1),
    PRIMARY KEY (ISSUE_ID,ATTRIBUTE_ID), 
    FOREIGN KEY (ISSUE_ID)
    REFERENCES SCARAB_ISSUE(ISSUE_ID),
    FOREIGN KEY (ATTRIBUTE_ID)
    REFERENCES SCARAB_ATTRIBUTE(ATTRIBUTE_ID),
    FOREIGN KEY (OPTION_ID)
    REFERENCES SCARAB_ATTRIBUTE_OPTION(OPTION_ID)
) 
;


CREATE TABLE SCARAB_LANGUAGE(
    LANGUAGE_ID      INTEGER     NOT NULL,
    LANGUAGE_NAME    VARCHAR(255)    NOT NULL,
    DELETED          VARCHAR(1),
    PRIMARY KEY (LANGUAGE_ID)
) 
;


CREATE TABLE SCARAB_MODIFICATION(
    TABLE_ID         INTEGER     NOT NULL,
    COLUMN_ID        INTEGER     NOT NULL,
    MODIFIED_BY      INTEGER     NOT NULL,
    CREATED_BY       INTEGER     NOT NULL,
    MODIFIED_DATE    DATE             NOT NULL,
    CREATED_DATE     DATE             NOT NULL,
    PRIMARY KEY (TABLE_ID,COLUMN_ID)
) 
;


CREATE TABLE SCARAB_MODULE(
    MODULE_ID             INTEGER     NOT NULL,
    MODULE_NAME           VARCHAR(255)    NOT NULL,
    MODULE_DESCRIPTION    VARCHAR(255)    NOT NULL,
    MODULE_URL            VARCHAR(255)    NOT NULL,
    OWNER_ID              INTEGER     NOT NULL,
    QA_CONTACT_ID         INTEGER ,
    DELETED               VARCHAR(1),
    PRIMARY KEY (MODULE_ID)
) 
;


CREATE TABLE SCARAB_R_MODULE_ATTRIBUTE(
    ATTRIBUTE_ID    INTEGER     NOT NULL,
    MODULE_ID       INTEGER     NOT NULL,
    DELETED         VARCHAR(1),
    PRIMARY KEY (ATTRIBUTE_ID,MODULE_ID), 
    FOREIGN KEY (ATTRIBUTE_ID)
    REFERENCES SCARAB_ATTRIBUTE(ATTRIBUTE_ID),
    FOREIGN KEY (MODULE_ID)
    REFERENCES SCARAB_MODULE(MODULE_ID)
) 
;


CREATE TABLE SCARAB_R_MODULE_VISITOR(
    VISITOR_ID    INTEGER     NOT NULL,
    MODULE_ID     INTEGER     NOT NULL,
    DELETED       VARCHAR(1),
    PRIMARY KEY (VISITOR_ID), 
    FOREIGN KEY (MODULE_ID)
    REFERENCES SCARAB_MODULE(MODULE_ID)
) 
;


CREATE TABLE SCARAB_R_MODULE_VISITOR_ROLE(
    MODULE_ID     INTEGER     NOT NULL,
    VISITOR_ID    INTEGER     NOT NULL,
    ROLE_ID       INTEGER     NOT NULL,
    DELETED       VARCHAR(1),
    PRIMARY KEY (MODULE_ID,VISITOR_ID,ROLE_ID), 
    FOREIGN KEY (MODULE_ID)
    REFERENCES SCARAB_MODULE(MODULE_ID)
) 
;


CREATE TABLE SCARAB_STRING(
    STRING_ID      INTEGER     NOT NULL,
    STRING_NAME    VARCHAR(255)    NOT NULL,
    STRING_DESC    VARCHAR(255)    NOT NULL,
    PRIMARY KEY (STRING_ID)
) 
;


CREATE TABLE SCARAB_STRING_LANGUAGE_VALUE(
    STRING_ID       INTEGER     NOT NULL,
    LANGUAGE_ID     INTEGER     NOT NULL,
    STRING_VALUE    TEXT             NOT NULL,
    PRIMARY KEY (STRING_ID,LANGUAGE_ID), 
    FOREIGN KEY (STRING_ID)
    REFERENCES SCARAB_STRING(STRING_ID),
    FOREIGN KEY (LANGUAGE_ID)
    REFERENCES SCARAB_LANGUAGE(LANGUAGE_ID)
) 
;


CREATE TABLE SCARAB_TRANSACTION(
    TRANSACTION_ID    INTEGER     NOT NULL,
    CREATED_BY        INTEGER     NOT NULL,
    CREATED_DATE      DATE             NOT NULL,
    PRIMARY KEY (TRANSACTION_ID)
) 
;


CREATE TABLE SCRAB_ISSUE_ATTRIBUTE_VOTE(
    ISSUE_ID        INTEGER     NOT NULL,
    ATTRIBUTE_ID    INTEGER     NOT NULL,
    VISITOR_ID      INTEGER     NOT NULL,
    OPTION_ID       INTEGER     NOT NULL,
    PRIMARY KEY (ISSUE_ID,ATTRIBUTE_ID,VISITOR_ID), 
    FOREIGN KEY (OPTION_ID)
    REFERENCES SCARAB_ATTRIBUTE_OPTION(OPTION_ID),
    FOREIGN KEY (ISSUE_ID,ATTRIBUTE_ID)
    REFERENCES SCARAB_ISSUE_ATTRIBUTE_VALUE(ISSUE_ID,ATTRIBUTE_ID)
) 
;


