package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// JDK
import java.io.IOException;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Creates instances of RunData for use within Turbine or 3rd party
 * applications.
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: RunDataFactory.java,v 1.3 2000/09/01 22:11:28 gonzalo Exp $
 */
public class RunDataFactory
{
    /**
     * Deprecated.
     *
     * Open way to get RunData information across Turbine.  This is an
     * old method.  It does not pass the servlet's config to RunData.
     * It is here in the event that someone is using it.  Turbine now
     * uses getRunData(req, res, config).
     *
     * @param req An HttpServletRequest.
     * @param res An HttpServletResponse.
     * @exception IOException.
     */
    public static RunData getRunData( HttpServletRequest req,
                                      HttpServletResponse res )
        throws IOException
    {
        return getRunData(req, res, null);
    }

    /**
     * Open way to get RunData information across Turbine..
     *
     * @param req An HttpServletRequest.
     * @param res An HttpServletResponse.
     * @param config A ServletConfig.
     * @exception IOException.
     */
    public static RunData getRunData( HttpServletRequest req,
                                      HttpServletResponse res,
                                      ServletConfig config )
        throws IOException,
               IllegalArgumentException
    {
        /*
         * NOTE: getRunData( HttpServletRequest req,
         * HttpServletResponse res ) has been deprecated 3-3-2000.
         * Wait a couple months (before Turbine 1.0) and remove this
         * method.  Also don't allow null for req, res, or config as
         * these are now required by Turbine.  Uncomment the below as
         * this should include the necessary functionality when we are
         * ready.
         */
        if ( req == null ||
             res == null ||
             config == null )
        {
            throw new IllegalArgumentException( "no values may be null" );
        }

        // Create a new RunData object.  This object caches all the
        // information that is needed for the execution lifetime of a
        // single request.  A new RunData object is created for each
        // and every request and is passed to each and every module.
        // Since each thread has its own RunData object, it is not
        // necessary to perform syncronization for the data within
        // this object.
        RunData data = new RunData();

        // Cache some information that will be used elsewhere.
        data.setRequest(req);
        data.setResponse(res);
        data.setMessages( new FormMessages() );

        // data.context = this.getServletContext();

        // Don't set this because if we want to output via
        // res.getOutputStream() then we will get an
        // IllegalStateException (already called getWriter()).  The
        // solution is to only do this if data.getOut() is called and
        // data.out is null. -jss

        // data.setOut(data.getResponse().getWriter());

        // Allow Turbine to work with both 2.2 (and 2.1) and 2.0
        // Servlet API.
        String scriptName = null;
        Class jsdkClass = HttpServletRequest.class;
        try
        {
            java.lang.reflect.Method meth =
                jsdkClass.getDeclaredMethod("getContextPath", null);
            scriptName = (String)meth.invoke(req, null);
        }
        catch (Exception ex)
        {
            // Ignore a NoSuchMethodException because it means we are
            // using Servlet API 2.0.  Make sure scriptName is not
            // null.
            scriptName = "";
        }
        scriptName = scriptName + data.getRequest().getServletPath();

        // Contains all of the GET/POST parameters.
        data.setParameters ( new ParameterParser ( data.getRequest() ) );

        // Get the HttpSession object.
        data.setSession ( data.getRequest().getSession(true) );

        // Set the servlet configuration in RunData for use in loading
        // other servlets.
        data.setServletConfig(config);

        // Now set the ServerData.
        data.setServerData( new ServerData( data.getRequest().getServerName(),
                                data.getRequest().getServerPort(),
                                data.getRequest().getScheme(),
                                scriptName ) );
        return data;
    }
}
