package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff
import java.util.Stack;

/*
 * This class implements a Stack for String objects.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: StringStackBuffer.java,v 1.2 2000/09/01 22:11:28 gonzalo Exp $
 */
public class StringStackBuffer
{
    /** The stack. */
    private Stack stk = null;

    /**
     * Constructor.
     */
    public StringStackBuffer()
    {
        stk = new Stack();
    }

    /**
     * Adds the String to the collection if it does not already
     * contain it.
     *
     * @param s A String.
     * @return A StringStackBuffer.
     */
    public StringStackBuffer add( String s )
    {
        if ( s != null  &&  !contains(s) )
             stk.push(s);
        return this;
    }

    /**
     * Does the Stack contain this String?
     *
     * @param s A String.
     * @return True if the Stack contains this String.
     */
    public boolean contains( String s )
    {
        return ( stk.search(s) != -1 );
    }

    /**
     * Is the Stack empty?
     * @return True if the Stack is empty.
     */
    public boolean empty()
    {
        return stk.empty();
    }

    /**
     * Get a String off the Stack at a certain position.
     *
     * @param i An int with the position.
     * @return A String.
     */
    public String get(int i)
    {
        return (String) stk.elementAt(i);
    }

    /**
     * What is the size of the Stack?
     *
     * @return An int, the size of the Stack.
     */
    public int size()
    {
        return stk.size();
    }

    /**
     * Converts the Stack to a single String with no separator.
     *
     * @return A String.
     */
    public String toString()
    {
        return toString("");
    }

    /**
     * Converts the Stack to a single String with a separator.
     *
     * @param separator A String.
     * @return A String.
     */
    public String toString( String separator )
    {
        if ( size() == 0 ) return "";
        if ( separator == null ) separator = "";
        StringBuffer sb = new StringBuffer().append( get(0) );
        for (int i=1; i<stk.size(); i++)
        {
            sb.append(separator).append(get(i));
        }
        return sb.toString();
    }
}
