package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff
import java.util.*;
import java.io.*;

/**
 * Used for wrapping system errors (exceptions) that may occur in the
 * application.
 *
 * @author <a href="mailto:neeme@one.lv">Neeme Praks</a>
 * @version $Id: SystemError.java,v 1.2 2000/09/01 22:11:29 gonzalo Exp $
 */
public class SystemError
{
    private Exception e;
    private String message;

    /**
     * Constructor.
     *
     * @param e An Exception.
     */
    public SystemError( Exception e)
    {
        setMessage(e.getMessage());
        setException(e);
    }

    /**
     * Constructor.
     *
     * @param e An Exception.
     * @param message A String.
     */
    public SystemError( Exception e,
                        String message )
    {
        this(e);
        setMessage(message);
    }

    /**
     * Constructor.
     *
     * @param e An Exception.
     * @param returncode A long.
     */
    public SystemError( Exception e,
                        long returncode )
    {
        this( e, new Long(returncode).toString() );
    }

    /**
     * Get the message.
     *
     * @return A String.
     */
    public String getMessage()
    {
        return message;
    }

    /**
     * Set the message.
     *
     * @param message A String.
     */
    public void setMessage(String message)
    {
        this.message = message;
    }

    /**
     * Get the exception.
     *
     * @return An Exception.
     */
    public Exception getException()
    {
        return e;
    }

    /**
     * Set the exception.
     *
     * @param e An Exception.
     */
    public void setException(Exception e)
    {
        this.e = e;
    }

    /**
     * Get the stack trace.
     *
     * @return A String.
     */
    public String getStackTrace()
    {
        if (e != null)
        {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            e.printStackTrace( new PrintWriter(out,true) );
            return out.toString();
        }
        else
            return "";
    }
}
