package org.apache.turbine.util.db.map;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.util.*;

// Turbine stuff.
import org.apache.turbine.util.db.pool.*;

/**
 * Default Builder for Database/Table/Column Maps within the Turbine
 * System.  If you decide to use your own table schema, then you
 * probably will want to implement this class on your own.  It is then
 * defined within the TurbineResources.properties file.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: TurbineMapBuilder.java,v 1.3 2000/09/05 19:36:58 gonzalo Exp $
 */
public class TurbineMapBuilder implements MapBuilder
{
    /**
     * Get the Visitor table.
     *
     * @return A String.
     */
    public String getTableVisitor()
    {
        return "Visitor";
    }

    /**
     * Get the UserRole table.
     *
     * @return A String.
     */
    public String getTableRole()
    {
        return "UserRole";
    }

    /**
     * Get the Permission table.
     *
     * @return A String.
     */
    public String getTablePermission()
    {
        return "Permission";
    }

    /**
     * Get the VisitorRole table.
     *
     * @return A String.
     */
    public String getTableVisitorRole()
    {
        return "VisitorRole";
    }

    /**
     * Get the RolePermission table.
     *
     * @return A String.
     */
    public String getTableRolePermission()
    {
        return "RolePermission";
    }

    /**
     * Internal Unique key to the visitor table.  Override this if
     * using your custom table.
     *
     * @return A String.
     */
    public String getVisitorId()
    {
        return "VISITORID";
    }

    /**
     * Fully qualified Unique key to the visitor table.  Shouldn't
     * need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getVisitor_VisitorId()
    {
        return getTableVisitor()+"."+getVisitorId();
    }

    /**
     * Column used to record the last login time for visitor.
     * Override this if using your custom table.
     *
     * @return A String.
     */
    public String getLastLogin()
    {
        return "LASTLOGIN";
    }

    /**
     * Fully qualified column used to record the last login time for
     * visitor.  Shouldn't need to override this as it uses the above
     * methods.
     *
     * @return A String.
     */
    public String getVisitor_LastLogin()
    {
        return getTableVisitor()+"."+getLastLogin();
    }

    /**
     * Column used to record the users username.  Override this if
     * using your custom table.
     *
     * @return A String.
     */
    public String getUsername()
    {
        return "LOGINID";
    }

    /**
     * Fully qualified column used to record the visitors username.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getVisitor_Username()
    {
        return getTableVisitor()+"."+getUsername();
    }

    /**
     * Column used to record the users password.  Override this if
     * using your custom table.
     *
     * @return A String.
     */
    public String getPassword()
    {
        return "PASSWORD_VALUE";
    }

    /**
     * Fully qualified column used to record the visitors password.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getVisitor_Password()
    {
        return getTableVisitor()+"."+getPassword();
    }

    /**
     * Column used to record general visitor data from a hashmap.
     * Override this if using your custom table.
     *
     * @return A String.
     */
    public String getObjectData()
    {
        return "OBJECTDATA";
    }

    /**
     * Fully qualified column used to record general visitor data from
     * a hashmap.  Shouldn't need to override this as it uses the
     * above methods.
     *
     * @return A String.
     */
    public String getVisitor_ObjectData()
    {
        return getTableVisitor()+"."+getObjectData();
    }

    /**
     * Column used for the unique id to a UserRole.  Override this if
     * using your custom table
     *
     * @return A String.
     */
    public String getRoleId()
    {
        return "ROLEID";
    }

    /**
     * Fully qualified column name for UserRole unique key.  Shouldn't
     * need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getRole_RoleId()
    {
        return getTableRole()+"."+getRoleId();
    }

    /**
     * Column used for the name of a UserRole.  Override this if using
     * your custom table.
     *
     * @return A String.
     */
    public String getRoleName()
    {
        return "ROLENAME";
    }

    /**
     * Fully qualified column name for UserRole name.  Shouldn't need
     * to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getRole_Name()
    {
        return getTableRole()+"."+getRoleName();
    }

    /**
     * Column used for the id of the Permission table.  Override this
     * if using your custom table.
     *
     * @return A String.
     */
    public String getPermissionId()
    {
        return "PERMISSIONID";
    }
    /**
     * Fully qualified column name for Permission table unique key.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getPermission_PermissionId()
    {
        return getTablePermission()+"."+getPermissionId();
    }

    /**
     * Column used for the name of a Permission.  Override this if
     * using your custom table.
     *
     * @return A String.
     */
    public String getPermissionName()
    {
        return "PERMISSION";
    }

    /**
     * Fully qualified column name for Permission table name of the
     * permission.  Shouldn't need to override this as it uses the
     * above methods.
     *
     * @return A String.
     */
    public String getPermission_Name()
    {
        return getTablePermission()+"."+getPermissionName();
    }

    /**
     * Fully qualified column name for VisitorRole visitor id.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getVisitorRole_VisitorId()
    {
        return getTableVisitorRole()+"."+getVisitorId();
    }

    /**
     * Fully qualified column name for VisitorRole role id.  Shouldn't
     * need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getVisitorRole_RoleId()
    {
        return getTableVisitorRole()+"."+getRoleId();
    }

    /**
     * Fully qualified column name for RolePermission permission id.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getRolePermission_PermissionId()
    {
        return getTableRolePermission()+"."+getPermissionId();
    }

    /**
     * Fully qualified column name for RolePermission role id.
     * Shouldn't need to override this as it uses the above methods.
     *
     * @return A String.
     */
    public String getRolePermission_RoleId()
    {
        return getTableRolePermission()+"."+getRoleId();
    }

    /**
     * PERMISSION_SEQUENCE.
     *
     * @return A String.
     */
    public String getSequencePermission()
    {
        return "PERMISSION_SEQUENCE";
    }

    /**
     * ROLE_SEQUENCE.
     *
     * @return A String.
     */
    public String getSequenceRole()
    {
        return "ROLE_SEQUENCE";
    }

    /**
     * VISITOR_SEQUENCE.
     *
     * @return A String.
     */
    public String getSequenceVisitor()
    {
        return "VISITOR_SEQUENCE";
    }

    /** The database map. */
    protected DatabaseMap dbMap = null;

    /**
     * Tells us if this DatabaseMapBuilder is built so that we don't
     * have to re-build it every time.
     *
     * @return True if DatabaseMapBuilder is built.
     */
    public boolean isBuilt()
    {
        if ( dbMap != null )
            return true;
        return false;
    }

    /**
     * Gets the databasemap this map builder built.
     *
     * @return A DatabaseMap.
     */
    public DatabaseMap getDatabaseMap()
    {
        return this.dbMap;
    }

    /**
     * Build up the databasemapping.  It should probably be modified
     * to read a .xml file representation of the database to build
     * this.
     *
     * @exception Exception, a generic exception.
     */
    public void doBuild()
        throws Exception
    {
        // Make some objects.
        String string = new String("");
        Integer integer = new Integer(0);
        java.util.Date date = new Date();

        // Get default map.
        dbMap = DBBroker.getInstance().getDatabaseMap(DBBroker.DEFAULT);

        // Add tables.
        dbMap.addTable(getTableVisitor());
        dbMap.addTable(getTableRole());
        dbMap.addTable(getTablePermission());
        dbMap.addTable(getTableVisitorRole());
        dbMap.addTable(getTableRolePermission());
        dbMap.addTable("Jobentry");

        dbMap.setIdTable("ID_TABLE");
        TableMap tMap = dbMap.getIdTable();
        tMap.addPrimaryKey("ID_TABLE_ID", integer);
        tMap.addColumn("TABLE_NAME", string);
        tMap.addColumn("NEXT_ID", integer);
        tMap.addColumn("QUANTITY", integer);

        // Add Visitor columns.
        tMap = dbMap.getTable(getTableVisitor());
        tMap.setPrimaryKeyMethod(TableMap.IDBROKERTABLE);
        tMap.addPrimaryKey(getVisitorId(), integer);
        tMap.addColumn("FIRST_NAME", string);
        tMap.addColumn("LAST_NAME", string);
        tMap.addColumn("CITIZENSHIP", string);
        tMap.addColumn(getUsername(), string);
        tMap.addColumn(getPassword(), string);
        tMap.addColumn("EMAIL", string);
        tMap.addColumn("CONFIRM_VALUE", string);
        tMap.addColumn("ADDRESS1", string);
        tMap.addColumn("ADDRESS2", string);
        tMap.addColumn("CITY", string);
        tMap.addColumn("STATE", string);
        tMap.addColumn("POSTALCODE", string);
        tMap.addColumn("COUNTRY", string);
        tMap.addColumn("PHONE", string);
        tMap.addColumn("ALTPHONE", string);
        tMap.addColumn("FAX", string);
        tMap.addColumn("CELL", string);
        tMap.addColumn("PAGER", string);
        tMap.addColumn("CREATED", date);
        tMap.addColumn("MODIFIED", date);
        tMap.addColumn(getLastLogin(), date);
        tMap.addColumn("OBJECTDATA", new Hashtable(1) );

        // Add Role columns.
        tMap = dbMap.getTable(getTableRole());
        tMap.setPrimaryKeyMethod(TableMap.IDBROKERTABLE);
        tMap.addPrimaryKey(getRoleId(), integer);
        tMap.addColumn(getRoleName(), string);

        // Add Permission columns.
        tMap = dbMap.getTable(getTablePermission());
        tMap.setPrimaryKeyMethod(TableMap.AUTOINCREMENT);
        tMap.addPrimaryKey(getPermissionId(), integer);
        tMap.addColumn(getPermissionName(), string);

        // Add RolePermission columns.
        tMap = dbMap.getTable(getTableRolePermission());
        tMap.addForeignPrimaryKey(getPermissionId(),
                                  integer,
                                  getTablePermission(),
                                  getPermissionId());
        tMap.addForeignPrimaryKey(getRoleId(),
                                  integer,
                                  getTableRole(),
                                  getRoleId());

        // Add MemberRole columns.
        tMap = dbMap.getTable(getTableVisitorRole());
        tMap.addForeignPrimaryKey(getVisitorId(),
                                  integer,
                                  getTableVisitor(),
                                  getVisitorId());
        tMap.addForeignPrimaryKey(getRoleId(),
                                  integer,
                                  getTableRole(),
                                  getRoleId());

         // Add Jobentry columns.
        tMap = dbMap.getTable("Jobentry");
        tMap.setPrimaryKeyMethod(TableMap.IDBROKERTABLE);
        tMap.addPrimaryKey("JOB_ID", integer);
        tMap.addColumn("MINUTE", integer);
        tMap.addColumn("HOUR", integer);
        tMap.addColumn("WEEKDAY", integer);
        tMap.addColumn("DAY_OF_MONTH", integer);
        tMap.addColumn("TASK", string);
        tMap.addColumn("EMAIL", string);
    }
}
