package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Clases
import java.util.*;

// Turbine stuff.
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class creates different DB objects based on the database
 * driver that is provided.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:ralf@reswi.ruhr.de">Ralf Stranzenbach</a>
 * @version $Id: DBFactory.java,v 1.2 2000/09/05 21:09:35 gonzalo Exp $
 */
public class DBFactory
{
    // List of registered drivers.
    private static Hashtable drivers = null;

    // This static code creates the list of possible drivers and adds
    // the "NO DATABASE" adaptor to this list.  After all the
    // TurbineResources is queried to get a list of JDBC drivers and
    // their associated adaptors.
    static
    {
        drivers = new Hashtable();

        // Add the null db driver.
        registerDriver("", DBNone.class);

        Enumeration e =
            TurbineResources.getVector("database.adaptor").elements();
        while (e.hasMoreElements())
        {
            String adaptor = (String)e.nextElement();
            String driver =
                TurbineResources.getString("database.adaptor." + adaptor);

            Class c = null;
            try
            {
                c = Class.forName("org.apache.turbine.util.db.pool." +
                                  adaptor);
            }
            catch (ClassNotFoundException ign1)
            {
                try
                {
                    c = Class.forName(adaptor);
                }
                catch (ClassNotFoundException ign2)
                {
                    Log.error(ign2);
                }
            }
            if ((c != null) && (driver != null))
            {
                registerDriver(driver, c);
            }
        }
    }

    /**
     * Try to register the class of a database driver at the factory.
     * This concept allows for dynamically adding new database drivers
     * using the configuration files instead of changing the codebase.
     *
     * @param driver A String with the name of the driver.
     * @param dc A Class.
     */
    private static void registerDriver(String driver,
                                       Class dc)
    {
        if (!drivers.containsKey(driver))
            // Add this new driver class to the list of known drivers.
            drivers.put(driver, dc);
    }

    /**
     * TODO: Document.
     */
    protected static DB create(String driver)
        throws InstantiationException
    {
        Class dc = (Class)drivers.get(driver);

        if (dc == null)
            throw new InstantiationException("Database type " + driver +
                                             " not implemented.");

        try
        {
            // Create an instantiation of the driver.
            DB retVal = (DB)dc.newInstance();
            retVal.setJDBCDriver(driver);
            return retVal;
        }
        catch (IllegalAccessException e)
        {
            throw new InstantiationException("Driver " + driver +
                                             " not instantiated.");
        }
    }
}
