-- Turbine, a Servlet Framework for building Dynamic Websites
--
-- For licensing and use, please see the LICENSE.txt in this directory.
--
-- This is a schema for IBM DB2 for
-- the concept of Users/Roles/Permissions
--
-- Please see the Html documentation for more information
-- on this SQL. Equivalent contributions of this for other
-- databases are welcome.
--
-- Commented Sequence Creation / Added ID_TABLE and JOBENTRY Tables
-- 
-- @author Jon S. Stevens
-- @author Brett McLaughlin
-- @author Martin Poeschl

DROP TABLE Permission
;
CREATE TABLE Permission (
  PERMISSIONID  INT            NOT NULL,
  PERMISSION    VARCHAR(99)   NOT NULL )
;
ALTER TABLE Permission
 ADD CONSTRAINT Permission_PK PRIMARY KEY (PERMISSIONID)
;
CREATE UNIQUE INDEX Permission_NAME ON PERMISSION(PERMISSION)
;
DROP TABLE UserRole
;
CREATE TABLE UserRole  (
    ROLEID      INT     	   NOT NULL,
    ROLENAME    VARCHAR(99)     NOT NULL
)
;
ALTER TABLE UserRole
 ADD CONSTRAINT UserRole_PK PRIMARY KEY (ROLEID)
;
CREATE UNIQUE INDEX UserRole_NAME ON UserRole(ROLENAME)
;
DROP TABLE RolePermission
;
CREATE TABLE RolePermission  (
    ROLEID       INT           NOT NULL,
    PERMISSIONID INT           NOT NULL
)
;
ALTER TABLE RolePermission
  ADD CONSTRAINT RolePermission_PK PRIMARY KEY (ROLEID,PERMISSIONID)
;
DROP TABLE RolePermission
;
CREATE TABLE Visitor  (
    VISITORID   INT          NOT NULL,
    LOGINID     VARCHAR(32) NOT NULL,
    PASSWORD_VALUE    VARCHAR(32) NOT NULL,
    FIRST_NAME  VARCHAR(99) NOT NULL,
    LAST_NAME   VARCHAR(99) NOT NULL,
    ADDRESS1    VARCHAR(32),
    ADDRESS2    VARCHAR(32),
    CITY        VARCHAR(32),
    STATE       VARCHAR(32),
    POSTALCODE  VARCHAR(32),
    COUNTRY     VARCHAR(32),
    CITIZENSHIP VARCHAR(32),
    PHONE       VARCHAR(32),
    ALTPHONE    VARCHAR(32),
    FAX         VARCHAR(32),
    CELL        VARCHAR(32),
    PAGER       VARCHAR(32),
    EMAIL       VARCHAR(99),
    MODIFIED    DATE,
    CREATED     DATE,
    LASTLOGIN   DATE,
    OBJECTDATA  VARCHAR(255) ) --DBCLOB)
;
alter table Visitor add CONFIRM_VALUE VARCHAR(99)
;
ALTER TABLE RolePermission
  ADD CONSTRAINT Visitor_PK PRIMARY KEY (VISITORID)
;
CREATE UNIQUE INDEX Visitor_LOGINID ON VISITOR(LOGINID)
;
DROP TABLE VisitorRole
;
CREATE TABLE VisitorRole  (
    VISITORID   INT         NOT NULL,
    ROLEID      INT         NOT NULL
)
;
ALTER TABLE VisitorRole
 ADD CONSTRAINT VisitorRole_PK PRIMARY KEY (VISITORID, ROLEID)
;
DROP TABLE Jobentry 
;
CREATE TABLE Jobentry  (
    JOB_ID	          INT 		  NOT NULL,
    MINUTE        INT  default -1 NOT NULL,
    HOUR          INT  default -1 NOT NULL,
    WEEKDAY       INT  default -1 NOT NULL,
    DAY_OF_MONTH  INT  default -1 NOT NULL,
    TASK          VARCHAR(99) NOT NULL,
    EMAIL         VARCHAR(99)
)
;
ALTER TABLE Jobentry
 ADD CONSTRAINT Jobentry_PK PRIMARY KEY (JOB_ID)
;
CREATE UNIQUE INDEX Jobentry_NAME ON Jobentry(TASK)
;
