; This is a schema for InstantDB for the concept of Users/Roles/Permissions
; (Note: These instructions are written to conform with v. 3.14 Beta 7.)
;
; Note: The process for creating a database in InstantDB is a tad
; convoluted. These instructions are written to match. ;)
;
; The following steps use the ScriptTool program/test harness contained in 
; the InstantDB distribution. 
;
; 1. Create a directory to store the database files (e.g., C:\TurbineDB).
; 2. Create an InstantDB properties file in the new directory called 
;    'Turbine.prp'. Base this on the sample props file from the 
;    InstantDB distribution (i.e., IDB_HOME\Examples\sample.prp).
; 3. Copy this file that you're reading into the database directory and 
;    rename it 'sql1.txt'.
; 4. Add the following to your CLASSPATH environment variable:
;      - IDB_HOME\Classes\idb.jar
;      - IDB_HOME\Classes\idbexmpl.jar
;      - IDB_HOME\Classes\idbf.jar
; 5. Change to your database directory and enter the following command:
;      java org.enhydra.instantdb.ScriptTool
; 6. If all goes well, this will create further sub-directories and create
;    database tables in those sub-directories.
; 7. Set the following in TurbineResources.properties:
;      database.default.driver=org.enhydra.instantdb.jdbc.idbDriver
;      database.default.url=C:/TurbineDB/Turbine.prp
;      database.default.username=
;      database.default.password=
; 8. Make sure IDB_HOME\Classes\idb.jar is in the CLASSPATH of your servlet
;    engine.

; First load the JDBC driver and open a database.
d org.enhydra.instantdb.jdbc.idbDriver;
o jdbc:idb=Turbine.prp;

e drop table Permission;

e create table Permission  (
    PERMISSIONID        integer AUTO INCREMENT PRIMARY KEY,
    PERMISSION          char (99) NOT NULL,
    UNIQUE (PERMISSION)
);

e drop table UserRole;

e create table UserRole  (
    ROLEID      integer AUTO INCREMENT PRIMARY KEY,
    ROLENAME    char (99) NOT NULL,
    UNIQUE (ROLENAME)
);

e drop table RolePermission;

e create table RolePermission  (
    ROLEID              integer NOT NULL,
    PERMISSIONID        integer NOT NULL,
    PRIMARY KEY (ROLEID, PERMISSIONID)
);

e create unique index roleID_permissionID_index ON RolePermission ( ROLEID, PERMISSIONID );

e drop table Visitor;

e create table Visitor(
    VISITORID   integer AUTO INCREMENT PRIMARY KEY,
    LOGINID     char (32) NOT NULL,
    PASSWORD_VALUE    char (32) NOT NULL,
    FIRST_NAME  char (99) NOT NULL,
    LAST_NAME   char (99) NOT NULL,
    ADDRESS1    char (255),
    ADDRESS2    char (255),
    CITY        char (255),
    STATE       char (32),
    POSTALCODE  char (32),
    COUNTRY     char (99),
    CITIZENSHIP char (32),
    PHONE       char (32),
    ALTPHONE    char (32),
    FAX         char (32),
    CELL        char (32),
    PAGER       char (32),
    EMAIL       char (99),
    MODIFIED    date,
    CREATED     date,
    LASTLOGIN   date,
    OBJECTDATA  binary,
    UNIQUE (LOGINID)
);

e alter table Visitor add PREFIX_NAME char (16);
e alter table Visitor add MIDDLE_NAME char (99);
e alter table Visitor add SUFFIX_NAME char (16);
e alter table Visitor add COMPANY     char (255);

e drop table VisitorRole;

e create table VisitorRole  (
    VISITORID   integer NOT NULL,
    ROLEID      integer NOT NULL,
    PRIMARY KEY (VISITORID, ROLEID)
);

e create unique index visitorID_roleID_index ON VisitorRole ( VISITORID, ROLEID );

e drop table Jobentry;

; FIXME: IDB doesn't like -1 as a default value
;e create table Jobentry  (
;    JOB_ID	          long NOT NULL PRIMARY KEY,
;    MINUTE        integer default -1 NOT NULL,
;    HOUR          integer default -1 NOT NULL,
;    WEEKDAY       integer default -1 NOT NULL,
;    DAY_OF_MONTH  integer default -1 NOT NULL,
;    TASK          char(99) NOT NULL,
;    EMAIL         char(99)
;);

; 
; This table provides the primary keys for all
; other tables in the system.  Should be used
; with util.db.IDBroker.
;
e drop table ID_TABLE;

e CREATE TABLE ID_TABLE 
(
  ID_TABLE_ID integer NOT NULL,
  TABLE_NAME char(255) NOT NULL,
  NEXT_ID integer,
  QUANTITY integer,
  PRIMARY KEY(ID_TABLE_ID),
  UNIQUE (TABLE_NAME)
);

e insert into ID_TABLE (id_table_id, table_name, next_id, quantity)
 values (1, 'Permission', 20, 10);
e insert into ID_TABLE (id_table_id, table_name, next_id, quantity)
 values (2, 'UserRole', 20, 10);
e insert into ID_TABLE (id_table_id, table_name, next_id, quantity)
 values (3, 'Visitor', 20, 10);
e insert into ID_TABLE (id_table_id, table_name, next_id, quantity)
 values (4, 'Jobentry', 20, 10);

c close;
