/* Turbine, a Servlet Framework for building Dynamic Websites
** Copyright (C) 1999  Jon S. Stevens, Brett McLaughlin, Fedor
Karpelevitch
**
** For licensing and use, please see the LICENSE.txt in this directory.
**
** This is a schema for Interbase for
** the concept of Users/Roles/Permissions
**
*/

/********************************************************************/
/* 
** do not execute (comment-out) the drop part when performing 
** the initial setup or it will break. The drop part is needed for
re-build only.
*/

alter table RolePermission
    drop constraint FK_ROLEPERM_PERM;

alter table RolePermission
    drop constraint FK_ROLEPERM_USERROLE;

alter table VisitorRole
    drop constraint FK_VISROLE_USERROLE;

alter table VisitorRole
    drop constraint FK_VISROLE_VISITOR;



drop table Permission;
drop table UserRole;
drop table RolePermission;
drop table Visitor;
drop table VisitorRole;
drop table Jobentry;

DELETE FROM RDB$GENERATORS
	WHERE RDB$GENERATOR_NAME in ('PERMISSIONID_GEN              
','ROLEID_GEN                     ', 'VISITORID_GEN                  ');

commit work;

create table Permission
(
    PERMISSIONID        integer not null,
    PERMISSION          varchar (99) NOT NULL,
    PRIMARY KEY (PERMISSIONID)
);

create unique index Permission_permission on Permission (PERMISSION);

create generator permissionid_gen;

SET TERM ^ ;

CREATE TRIGGER SET_permissionid FOR Permission
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
	new.permissionid = gen_id(permissionid_gen, 1);
END ^
set term ;^

/********************************************************************/

create table UserRole
(
    ROLEID		integer not null,
    ROLENAME		varchar (99) NOT NULL,
    PRIMARY KEY (ROLEID)
);

create unique index UserRole_rolename on UserRole (ROLENAME);

create generator ROLEID_gen;

SET TERM ^ ;

CREATE TRIGGER SET_ROLEID FOR UserRole
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
	new.ROLEID = gen_id(ROLEID_gen, 1);
END ^
set term ;^


/********************************************************************/

create table RolePermission
(
    ROLEID              integer NOT NULL,
    PERMISSIONID        integer NOT NULL,
    PRIMARY KEY (ROLEID, PERMISSIONID)
);

create index RoPerm_rid_permid_index on RolePermission ( ROLEID,
PERMISSIONID );

/********************************************************************/

create table Visitor
(
    VISITORID		INTEGER NOT NULL,
    LOGINID		varchar (32) NOT NULL,
    PASSWORD_VALUE	varchar (32) NOT NULL,
    FIRST_NAME		varchar (99) NOT NULL,
    LAST_NAME		varchar (99) NOT NULL,
    ADDRESS1		varchar (255),
    ADDRESS2		varchar (255),
    CITY		varchar (255),
    STATE		varchar (32),
    POSTALCODE		varchar (32),
    COUNTRY		varchar (99),
    CITIZENSHIP		varchar (32),
    PHONE		varchar (32),
    ALTPHONE		varchar (32),
    FAX			varchar (32),
    CELL		varchar (32),
    PAGER		varchar (32),
    EMAIL		varchar (99),
    MODIFIED		timestamp,
    CREATED		timestamp,
    LASTLOGIN		timestamp,
    OBJECTDATA		BLOB,
    PREFIX_NAME varchar (16),
    MIDDLE_NAME varchar (99),
    SUFFIX_NAME varchar (16),
    COMPANY     varchar (255),
    CONFIRM_VALUE varchar (99),
    PRIMARY KEY (VISITORID)
);

create unique index Visitor_loginid on Visitor (LOGINID);

create generator VISITORID_gen;

SET TERM ^ ;

CREATE TRIGGER SET_VISITORID FOR Visitor
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
	new.VISITORID = gen_id(VISITORID_gen, 1);
END ^
set term ;^


/********************************************************************/

create table VisitorRole
(
    VISITORID		integer NOT NULL,
    ROLEID		integer NOT NULL,
    PRIMARY KEY (VISITORID, ROLEID)
);

create unique index VisRo_visid_roid_index on VisitorRole ( VISITORID,
ROLEID );

alter table RolePermission
    add constraint FK_ROLEPERM_PERM foreign key  (PERMISSIONID)
       references Permission (PERMISSIONID);

alter table RolePermission
    add constraint FK_ROLEPERM_USERROLE foreign key  (ROLEID)
       references UserRole (ROLEID);

alter table VisitorRole
    add constraint FK_VISROLE_USERROLE foreign key  (ROLEID)
       references UserRole (ROLEID);

alter table VisitorRole
    add constraint FK_VISROLE_VISITOR foreign key  (VISITORID)
       references Visitor (VISITORID);


/********************************************************************/
 
create table Jobentry  (
    JOB_ID        integer NOT NULL,
    "MINUTE"        integer default -1 NOT NULL,
    "HOUR"          integer default -1 NOT NULL,
    "WEEKDAY"       integer default -1 NOT NULL,
    DAY_OF_MONTH  integer default -1 NOT NULL,
    TASK          varchar(99) NOT NULL,
    EMAIL         varchar(99),
    PRIMARY KEY (JOB_ID)
);
