/* ============================================================ */
/*   Database name:  TURBINE                                    */
/*   DBMS name:      Sybase AS Enterprise 11.5-11.9             */
/*   Created on:     5/25/00  1:26 PM                           */
/* ============================================================ */

/* ============================================================ */
/*  Drop constraints                                            */
/* ============================================================ */
if exists (select 1 from sysobjects where type ="RI" and name='FK_ROLEPERM_REF_83_PERMISSI')
    alter table RolePermission drop constraint FK_ROLEPERM_REF_83_PERMISSI
go

if exists (select 1 from sysobjects where type ="RI" and name='FK_ROLEPERM_REF_86_USERROLE')
    alter table RolePermission drop constraint FK_ROLEPERM_REF_86_USERROLE
go

if exists (select 1 from sysobjects where type ="RI" and name='FK_VISITORR_REF_89_USERROLE')
    alter table VisitorRole drop constraint FK_VISITORR_REF_89_USERROLE
go

if exists (select 1 from sysobjects where type ="RI" and name='FK_VISITORR_REF_92_VISITOR')
    alter table VisitorRole drop constraint FK_VISITORR_REF_92_VISITOR
go


/* ============================================================ */
/*   Table: Jobentry                                            */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "Jobentry")
begin
	drop table Jobentry
end
go

create table Jobentry
(
    JOB_ID          int                    not null,
    MINUTE          int                    not null,
    HOUR            int                    not null,
    WEEKDAY         int                    not null,
    DAY_OF_MONTH    int                    not null,
    TASK            varchar(99)            not null,
    EMAIL           varchar(99)            null    ,
    constraint Jobentry_PK primary key (JOB_ID)
)
go

/* ============================================================ */
/*   Table: Permission                                          */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "Permission")
begin
	drop table Permission
end
go

create table Permission
(
    PERMISSIONID    int                    not null,
    PERMISSION      varchar(99)            not null,
    constraint Permission_PK primary key (PERMISSIONID)
)
go

/* ============================================================ */
/*   Index: PERMISSION                                          */
/* ============================================================ */
create unique index PERMISSION on Permission (PERMISSION)
go

/* ============================================================ */
/*   Table: UserRole                                            */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "UserRole")
begin
	drop table UserRole
end
go

create table UserRole
(
    ROLEID          int                    not null,
    ROLENAME        varchar(99)            not null,
    constraint UserRole_PK primary key (ROLEID)
)
go

/* ============================================================ */
/*   Index: ROLENAME                                            */
/* ============================================================ */
create unique index ROLENAME on UserRole (ROLENAME)
go

/* ============================================================ */
/*   Table: Visitor                                             */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "Visitor")
begin
	drop table Visitor
end
go

create table Visitor
(
    VISITORID       int                    not null,
    LOGINID         varchar(32)            not null,
    PASSWORD_VALUE  varchar(32)            not null,
    FIRST_NAME      varchar(99)            not null,
    LAST_NAME       varchar(99)            not null,
    ADDRESS1        varchar(255)           null    ,
    ADDRESS2        varchar(255)           null    ,
    CITY            varchar(255)           null    ,
    STATE           varchar(32)            null    ,
    POSTALCODE      varchar(32)            null    ,
    COUNTRY         varchar(99)            null    ,
    CITIZENSHIP     varchar(32)            null    ,
    PHONE           varchar(32)            null    ,
    ALTPHONE        varchar(32)            null    ,
    FAX             varchar(32)            null    ,
    CELL            varchar(32)            null    ,
    PAGER           varchar(32)            null    ,
    EMAIL           varchar(99)            null    ,
    MODIFIED        timestamp              not null,
    CREATED         char(10)               null    ,
    LASTLOGIN       datetime               not null,
    OBJECTDATA      char(10)               null    ,
    PREFIX_NAME     varchar(16)            null    ,
    MIDDLE_NAME     varchar(99)            null    ,
    SUFFIX_NAME     varchar(16)            null    ,
    COMPANY         varchar(255)           null    ,
    CONFIRM_VALUE   varchar(99)            null    ,
    constraint Visitor_PK primary key (VISITORID)
)
go

/* ============================================================ */
/*   Index: LOGINID                                             */
/* ============================================================ */
create unique index LOGINID on Visitor (LOGINID)
go

/* ============================================================ */
/*   Table: RolePermission                                      */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "RolePermission")
begin
	drop table RolePermission
end
go

create table RolePermission
(
    ROLEID          int                    not null,
    PERMISSIONID    int                    not null,
)
go

/* ============================================================ */
/*   Table: VisitorRole                                         */
/* ============================================================ */

if exists (select 1 from sysobjects where type = "U" and name = "VisitorRole")
begin
	drop table VisitorRole
end
go

create table VisitorRole
(
    VISITORID       int                    not null,
    ROLEID          int                    not null,
)
go

/* ============================================================ */
/*  Create constraints                                          */
/* ============================================================ */
alter table RolePermission
    add constraint FK_ROLEPERM_REF_83_PERMISSI foreign key  (PERMISSIONID)
       references Permission (PERMISSIONID)
go

alter table RolePermission
    add constraint FK_ROLEPERM_REF_86_USERROLE foreign key  (ROLEID)
       references UserRole (ROLEID)
go

alter table VisitorRole
    add constraint FK_VISITORR_REF_89_USERROLE foreign key  (ROLEID)
       references UserRole (ROLEID)
go

alter table VisitorRole
    add constraint FK_VISITORR_REF_92_VISITOR foreign key  (VISITORID)
       references Visitor (VISITORID)
go

